/*
 * DBMDatabase
 *
 *    This implements the DBMDatabase class, which simplifies access to 
 * ndbm(3) databases. Or maybe not, but it looks good.
 *
 * dbmdatabase.C,v 1.1 1993/11/27 01:38:50 gjb Exp
 */

#include <stdlib.h>
#include <dbmdatabase.H>
#include <objdce/error.H>
#include <sys/fcntl.h>
#include <sys/stat.h>

const int DBM_SUCCESS = 0;
const int DBM_ENTRY_NOT_FOUND = 1;
const int DBM_DUPLICATE_ENTRY = 2;
const int DBM_INTERNAL_ERROR = 3;
const int DBM_FILE_NOT_OPEN = 4;
const int DBM_END_OF_RANGE = 5;

DBMDatabase::DBMDatabase(const String& fileName)

{
  int err;

  latch.Lock();
  database = dbm_open((char*)((const char*)fileName),
		      O_RDWR | O_CREAT,
		      S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
  err = dbm_error(database);
  dbm_clearerr(database);
  latch.Unlock();
  if (err || (database == (DBM*)0))
    {
      ObjtranReport.VReport(NormalMsg, "Error opening database: %i\n", err);
      database = (DBM*)0;
    }
}

DBMDatabase::~DBMDatabase()

{
  if (CheckOpen())
    {
      latch.Lock();
      dbm_close(database);
      database = (DBM*)0;
      latch.Unlock();
    }
}

int DBMDatabase::CheckOpen() const

{
  if (database == (DBM*)0)
    {
      ObjtranReport.Report(NormalMsg, 
			   "Error: Couldn't use a closed DBM database\n");
      return 0;
    }
  else
    return 1;
}

int DBMDatabase::Fetch(const DBMDatum& key, DBMDatum& out)

{
  datum keyD = key;
  datum dataD;
  int err;
  int retVal;
  
  if (CheckOpen())
    {
      latch.Lock();
      dataD = dbm_fetch(database, keyD);
      err = dbm_error(database);
      dbm_clearerr(database);
      if ((dataD.dptr == (char*)0) && (err == 0))
	{
	  ObjtranReport.VReport(DebugMsg, 
				"No data found for %i byte key\n",
				keyD.dsize);
	  retVal = DBM_ENTRY_NOT_FOUND;
	}
      else if (err)
	{
	  ObjtranReport.VReport(NormalMsg,
				"Error %i reading from DBM file\n", err);
	  retVal = DBM_INTERNAL_ERROR;
	}
      else
	{
	  ObjtranReport.VReport(DebugMsg,
				"Read %i bytes from a %i byte key\n",
				dataD.dsize, keyD.dsize);
	  out = dataD;
	  retVal = DBM_SUCCESS;
	}
      latch.Unlock();
      return retVal;
    }
  else
    return DBM_FILE_NOT_OPEN;
}

int DBMDatabase::Insert(const DBMDatum& key, const DBMDatum& data)

{
  return DoStore(key, data, DBM_INSERT);
}

int DBMDatabase::Replace(const DBMDatum& key, const DBMDatum& data)

{
  return DoStore(key, data, DBM_REPLACE);
}

int DBMDatabase::DoStore(const DBMDatum& key, const DBMDatum& data,
			 int flags)

{
  datum keyD = key;
  datum dataD = data;
  int err;
  int retVal = DBM_SUCCESS;

  if (CheckOpen())
    {
      latch.Lock();
      err = dbm_store(database, keyD, dataD, flags);
      dbm_clearerr(database);
      latch.Unlock();
      if ((flags == DBM_INSERT) && (err == 1))
	{
	  ObjtranReport.VReport(DebugMsg,
				"Duplicate entry inserting a %i byte key\n",
				keyD.dsize);
	  retVal = DBM_DUPLICATE_ENTRY;
	}
      else if (err != 0)
	{
	  ObjtranReport.VReport(NormalMsg,
			       "DBM error %i inserting a %i byte key\n",
			       err, keyD.dsize);
	  retVal = DBM_INTERNAL_ERROR;
	}
      else
	{
	  ObjtranReport.VReport(DebugMsg,
				"Inserted %i bytes with a %i byte key\n", 
				dataD.dsize, keyD.dsize);
	}
      return retVal;
    }
  else
    return DBM_FILE_NOT_OPEN;
}

int DBMDatabase::Delete(const DBMDatum& key)

{
  datum keyD = key;
  int err;
  int retVal = DBM_SUCCESS;

  if (CheckOpen())
    {
      latch.Lock();
      err = dbm_delete(database, keyD);
      dbm_clearerr(database);
      latch.Unlock();
      if (err != 0)
	{
	  ObjtranReport.VReport(NormalMsg,
				"Internal DBM error %i deleting a %i byte key\n",
				err, keyD.dsize);
	  retVal = DBM_INTERNAL_ERROR;
	}
      else
	{
	  ObjtranReport.VReport(DebugMsg,
				"Successfully deleted a %i byte key\n",
				keyD.dsize);
	}
      return retVal;
    }
  else
    return DBM_FILE_NOT_OPEN;
}

int DBMDatabase::FirstKey(DBMDatum& out)

{
  datum outD;
  int err;
  int retVal = DBM_SUCCESS;

  if (CheckOpen())
    {
      latch.Lock();
      outD = dbm_firstkey(database);
      err = dbm_error(database);
      dbm_clearerr(database);
      if ((outD.dptr == (char*)0) && (err == 0))
	{
	  ObjtranReport.Report(DebugMsg,
			       "Reached end of range in DBM file\n");
	  retVal = DBM_END_OF_RANGE;
	}
      else if (err)
	{
	  ObjtranReport.VReport(NormalMsg,
				"Internal DBM error %i reading DBM range\n",
				err);
	  retVal = DBM_INTERNAL_ERROR;
	}
      else
	{
	  ObjtranReport.VReport(DebugMsg,
				"Successfully read a %i byte key\n", 
				outD.dsize);
	  out = outD;
	}
      latch.Unlock();
      return retVal;
    }
  else
    return DBM_FILE_NOT_OPEN;
}

int DBMDatabase::NextKey(DBMDatum& out)

{
  datum outD;
  int err;
  int retVal = DBM_SUCCESS;

  if (CheckOpen())
    {
      latch.Lock();
      outD = dbm_nextkey(database);
      err = dbm_error(database);
      dbm_clearerr(database);	
      if ((outD.dptr == (char*)0) && (err == 0))
	{
	  ObjtranReport.Report(DebugMsg,
			       "Reached end of range in DBM file\n");
	  retVal = DBM_END_OF_RANGE;
	}
      if (err)
	{
	  ObjtranReport.VReport(NormalMsg,
				"Internal DBM error %i reading in range\n",
				err);
	  retVal = DBM_INTERNAL_ERROR;
	}
      else
	{
	  ObjtranReport.VReport(DebugMsg,
				"Read %i byte key from DBM range\n",
				outD.dsize);
	  out = outD;
	}
      latch.Unlock();
      return retVal;
    }
  else
    return DBM_FILE_NOT_OPEN;
}






