/*
 * PhoneBoook
 *
 *    Actually do the phone book stuff.
 *
 * phonebook.C,v 1.1 1993/11/27 01:38:57 gjb Exp
 */

#include <phonebook.H>

static const PhoneBookEntryID MaxEntryKey = 0L;

PhoneBook::PhoneBook(const String& fileName) : db (fileName)

{
  ReadWholeDB();
}

PhoneBook::~PhoneBook()

{
}

int PhoneBook::ReadMaxEntry()

{
  DBMDatum maxKey;
  DBMDatum maxData;
  int dbmErr;

  maxLatch.Lock();
  maxKey.Assign((char*)&MaxEntryKey, sizeof(PhoneBookEntryID));
  dbmErr = db.Fetch(maxKey, maxData);
  if (dbmErr == 0)
    {
      memcpy((char*)&maxEntry, maxData.GetData(), maxData.GetDataLength());
    }
  else
    {
      maxEntry = 0;
      maxData.Assign((char*)&maxEntry, sizeof(PhoneBookEntryID));
      db.Insert(maxKey, maxData);
    }
  maxLatch.Unlock();
  return 0;
}

PhoneBookEntryID PhoneBook::GetNewMaxEntry()

{
  DBMDatum maxKey;
  DBMDatum maxData;
  PhoneBookEntryID retVal;

  maxLatch.Lock();
  maxEntry++;
  retVal = maxEntry;
  maxKey.Assign((char*)&MaxEntryKey, sizeof(PhoneBookEntryID));
  maxData.Assign((char*)&maxEntry, sizeof(PhoneBookEntryID));
  db.Replace(maxKey, maxData);
  maxLatch.Unlock();
  return retVal;
}

int PhoneBook::ReadWholeDB()

{
  int dbmErr;
  DBMDatum curKey;
  DBMDatum curData;
  PhoneBookEntry curEntry;
  PhoneIndexEntry curIndex;

  ReadMaxEntry();
  listLatch.Lock();
  Listiter<PhoneIndexEntry> iter(searchList);
  while (iter.remove_next());	// Empty the list
  dbmErr = db.FirstKey(curKey);
  while (dbmErr == 0)
    {
      dbmErr = db.Fetch(curKey, curData);
      if ((dbmErr == 0) &&
	  (curData.GetDataLength() != sizeof(PhoneBookEntryID)) &&
	  (curData.GetDataLength() > 0))
	{
	  curEntry.Unpack(curData.GetData(), curData.GetDataLength());
	  curIndex = curEntry;
	  searchList.put(curIndex);
	}
      dbmErr = db.NextKey(curKey);
    }
  listLatch.Unlock();
  return dbmErr;
}

int PhoneBook::Insert(PhoneBookEntry& newEntry)

{
  char tmpData[PhoneBookMaxEntryLength];
  PhoneBookEntryID newID = GetNewMaxEntry();
  DBMDatum newKey;
  DBMDatum newData;
  int newDataLen;
  int dbmErr;
  PhoneIndexEntry newIndex;

  newEntry.ID(newID);
  newKey.Assign((char*)&newID, sizeof(PhoneBookEntryID));
  newEntry.Pack(tmpData, newDataLen, PhoneBookMaxEntryLength);
  newData.Assign(tmpData, newDataLen);
  dbmErr = db.Insert(newKey, newData);
  if (dbmErr == 0)
    {
      newIndex = newEntry;
      listLatch.Lock();
      searchList.put(newIndex);
      listLatch.Unlock();
    }
  return dbmErr;
}

int PhoneBook::Replace(const PhoneBookEntryID id, 
		       const PhoneBookEntry& newEntry)
{
  char tmpData[PhoneBookMaxEntryLength];
  DBMDatum newKey;
  DBMDatum newData;
  PhoneBookEntry curEntry;
  PhoneIndexEntry curIndex;
  PhoneIndexEntry newIndex;
  int realDataLen;
  int retVal;

  listLatch.Lock();
  Listiter<PhoneIndexEntry> iter(searchList);
  while ((iter.next(curIndex)) && (curIndex != id));
  if (curIndex == id)
    {
      newIndex = newEntry;
      iter.replace_prev(newIndex);
      listLatch.Unlock();
      newKey.Assign((char*)&id, sizeof(PhoneBookEntryID));
      newEntry.Pack(tmpData, realDataLen, PhoneBookMaxEntryLength);
      newData.Assign(tmpData, realDataLen);
      retVal = db.Replace(newKey, newData);
    }
  else
    {
      listLatch.Unlock();
      retVal = PHONE_SERVER_NO_MATCHES;
    }
  return retVal;
}

int PhoneBook::Find(const String& key, List<PhoneBookEntry>& out,
		    PhoneSearchType type)
{
  PhoneIndexEntry foundIndex;
  PhoneIndexEntry compareIndex;
  PhoneBookEntry foundEntry;
  int retVal;

  Listiter<PhoneBookEntry> outIter(out);
  while (outIter.remove_next());
  listLatch.Lock();
  PhoneIndexEntry::SetSearchType(type);
  Const_listiter<PhoneIndexEntry> iter(searchList);
  compareIndex.Name(key);
  while (iter.find_next(compareIndex))
    {
      iter.next(foundIndex);
      Get(foundIndex.ID(), foundEntry);
      out.put(foundEntry);
    }
  listLatch.Unlock();
  if (out.length() > 0)
    retVal = 0;
  else
    retVal = PHONE_SERVER_NO_MATCHES;
  return retVal;
}

int PhoneBook::Get(const PhoneBookEntryID id, PhoneBookEntry& out)

{
  int dbmErr;
  DBMDatum key;
  DBMDatum data;
  
  key.Assign((char*)&id, sizeof(PhoneBookEntryID));
  dbmErr = db.Fetch(key, data);
  if (dbmErr == 0)
    {
      out.Unpack(data.GetData(), data.GetDataLength());
    }
  return dbmErr;
}

int PhoneBook::GetAll(List<PhoneBookEntry>& out)

{
  PhoneIndexEntry curIndex;
  PhoneBookEntry curEntry;

  Listiter<PhoneBookEntry> outIter(out);
  while (outIter.remove_next());
  listLatch.Lock();
  Const_listiter<PhoneIndexEntry> iter(searchList);
  while (iter.next(curIndex))
    {
      Get(curIndex.ID(), curEntry);
      out.put(curEntry);
    }
  listLatch.Unlock();
  return 0;
}

int PhoneBook::Delete(const PhoneBookEntryID id)

{
  PhoneIndexEntry foundIndex;
  DBMDatum deadKey;
  int retVal;

  listLatch.Lock();
  Listiter<PhoneIndexEntry> iter(searchList);
  while ((iter.next(foundIndex)) && (foundIndex != id));
  if (foundIndex == id)
    {
      iter.remove_prev();
      listLatch.Unlock();
      deadKey.Assign((char*)&id, sizeof(PhoneBookEntryID));
      retVal = db.Delete(deadKey);
    }
  else
    {
      listLatch.Unlock();
      retVal = PHONE_SERVER_NO_MATCHES;
    }
  return retVal;
}

  
