/*
 * mon_transaction.C
 *    Here we implement parts of the Transaction class and its related 
 * callbacks that are Monitor-specific. The TRAN-specific versions of these
 * calls are implemented in tran_transaction.C.
 *
 * mon_transaction.C,v 1.3 1994/01/18 17:14:05 gjb Exp
 */




    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/tranaux.H>
#include <objdce/objtranparams.H>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <objtran/transaction.H>
#include <objtran/tranproperty.H>
#include <objtran/tranmember.H>
#include <objdce/abortreason.H>
#include <objdce/error.H>
#include <String.h>
#include <objdce/mutex.H>
#include <objtran/rpctrancheckstatus.H>
extern "C"
{
#include <tpm/mon/mon_client.h>
#include <pthread.h>
#include <threadTid/threadTid.h>
}
#include <objdce/defs.H>
#if NEED_MALLOC_H
#include <malloc.h>
#endif

/*
 * MonAbortCall
 *    The Monitor calls this function when a transaction commits.
 * Here, we update the state flags in our TranEntry object, and read
 * the abort properties from TRAN that other Objtran applications put
 * there. If they aren't present, we use the Monitor-provided interface
 * for doing the same thing.
 */

void MonAbortCall(void* entryPtr, unsigned long, unsigned long)

{
  char* abortStr;
  AbortReason* reason;
  TranEntry** arrgh = (TranEntry**)entryPtr;
  TranEntry* entry = *arrgh;
  tran_tid_t tid = *entry;
  
  ExistingTransaction tran(tid);
  ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Aborted\n", tid);
  if (entry != nil)
    {
      entry->Latch();
      entry->SetFlag(TXN_ABORTED);
      entry->ClearFlag(TXN_COMMITTED);
      entry->Unlatch();
    }
  if (!tran.ReadAbortProperties())
    {
      if (mon_TxGetAbortCode(&abortStr) == MON_SUCCESS)
	{
	  entry->Latch();
	  reason = entry->GetAbortReason();
	  if (reason == (AbortReason*)0)
	    {
	      reason = new AbortReason();
	      entry->SetAbortReason(reason);
	    }
	  reason->SetReason(abortStr);
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"TID %lu: Read Monitor abort reason\n",
				tid);
	  entry->Unlatch();
	}
      else
	{
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"TID %lu: Couldn't read Monitor abort reason\n",
				tid);
	}
    }
}

/*
 * MonFinisheCall
 *    This function is not registered for all transactions. When it
 * is, we use it to mark this transaction's TranEntry for deletion.
 */

void MonFinishCall(void* entryPtr, unsigned long, unsigned long)

{
  TranEntry** arrgh = (TranEntry**)entryPtr;
  TranEntry* entry = *arrgh;
  tran_tid_t tid = *entry;

  ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Finished\n", 
			tid);
  if (entry != (TranEntry*)0)
    {
      entry->Latch();
      if (entry->InvalidateTranObj(true))
	{
	  entry->Unlatch();
	  Transaction::ClearTran(tid);
	}
      else
	entry->Unlatch();
    }
}

/*
 * Commit:
 *    This is simpler for the Monitor. Update the state flags on the 
 * TranEntry object, commit the transaction, and trace the results.
 */

void Transaction::Commit()

{
  mon_status_t err;

  myEntry->Latch();
  if (!myEntry->CheckFlag(TXN_MON_LOCAL_END))
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "TID %lu: About to end Monitor transaction\n",
			    tranRep);
      if (!myEntry->CheckFlag(TXN_ABORTED))
	{
	  myEntry->SetFlag(TXN_COMMITTED);
	}
      myEntry->SetFlag(TXN_MON_LOCAL_END);
      myEntry->Unlatch();
      err = mon_TxCommit();
      myEntry->Latch();
      if ((err != MON_SUCCESS) && (err != MON_TX_ABORTED) && 
	  (err != MON_FAILED))
	{
	  CheckTranStatus(err, false, 3000, ErrorLevels::WarnMsg);
	  ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 900);
	  exit(1);
	} 
      else if (err == MON_TX_ABORTED)
	{
	  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
		"TID %lu: Monitor transaction had aborted\n", 
				tranRep);
	}
      else
	{
	  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
				"TID %lu: Monitor transaction committed\n",
				tranRep);
	}
    }
  myEntry->Unlatch();
}

/*
 * Abort:
 *    Using the AbortReason object passed us, we set the abort properties
 * of the transaction so we can get it later. Also, set the transaction
 * state flags, and abort the transaction.
 */

void Transaction::Abort(const AbortReason& reason)

{
  String codeTxt;
  void* dataBytes;
  TransactionProperty keyVal;
  TransactionProperty dataVal;
  mon_status_t err = MON_SUCCESS;
  int dataLen;

  myEntry->Latch();
  if (!myEntry->CheckFlag(TXN_ABORTED))
    {
      if (myEntry->GetAbortReason() == (AbortReason*)0)
	{
	  AbortReason* tmpReason = new AbortReason(reason);
	  myEntry->SetAbortReason(tmpReason);

	  dataLen = reason.PackStringLength();
	  dataBytes = (void*)malloc(dataLen + 1);
	  reason.PackString(dataBytes);
	  keyVal.SetKey(TRAN_PROPERTY_KEY_ABORT_FORMAT);
	  keyVal.SetValue(ReasonName);
	  AddProperty(keyVal);
	  dataVal.SetKey(ReasonName);
	  dataVal.SetValue(dataBytes, dataLen);
	  AddProperty(dataVal);
	  free((char*)dataBytes);

	  dataLen = reason.PackLength();
	  dataBytes = (void*)malloc(dataLen + 1);
	  reason.Pack(dataBytes);
	  dataVal.SetKey(MoreReasonName);
	  dataVal.SetValue(dataBytes, dataLen);
	  AddProperty(dataVal);
	  free((char*)dataBytes);
	}
      
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "TID %lu: About to abort Monitor transaction\n",
			    tranRep);
      if (!myEntry->CheckFlag(TXN_ABORTED))
	{
	  myEntry->SetFlag(TXN_ABORTED | TXN_MON_LOCAL_END);
	  myEntry->ClearFlag(TXN_COMMITTED);
	  myEntry->Unlatch();
	  err = mon_TxAbort(ReasonName);
	  myEntry->Latch();
	}
      if (err != MON_SUCCESS)
	{
	  CheckTranStatus(err, false, 3001, ErrorLevels::ErrorMsg);
	  ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 900);
//	  abort();	
	  exit(1);
	}
    }
  myEntry->Unlatch();
}

void Transaction::Abort(const String& text, const int code, const int applNo)

{
    AbortReason* reason = new AbortReason(text, code, applNo);
    Abort(*reason);
    delete reason;
}

/*
 * UpdateState:
 *    The Monitor takes care of the threadTid stuff for us, so we don't
 * need to bother. However, here we must abort the transaction if it's
 * been aborted already. This the only way the Monitor finds out that
 * another participant aborted it. The latch on the TranEntry object
 * MUST be locked before we call this.
 */

void Transaction::UpdateState()

{
  unsigned long err;

  if ((myEntry->CheckFlag(TXN_ABORTED_OR_COMMITTED)) &&
      (!myEntry->CheckFlag(TXN_MON_LOCAL_END)))
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "TID %lu: Calling tx_Commit to end transaction\n", 
			    tranRep);
      myEntry->SetFlag(TXN_MON_LOCAL_END);
      myEntry->Unlatch();
      err = mon_TxCommit();
      myEntry->Latch();
      if ((err != MON_SUCCESS) && (err != MON_TX_ABORTED))
	{
	  CheckTranStatus(err, false, 3013, ErrorLevels::ErrorMsg);
	}
    }
}

/*
 * InitNewTran
 *    Do the "real work" associated with starting a transaction. For
 * the Monitor, all we really have to do it begin the transaction, set up 
 * the security key, and log some stuff.
 */

void NewTransaction::InitNewTran(const int timeout)

{
  mon_status_t err;

  if (ObjtranMainMode == 0)
    {
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 3002);	// Not initialized
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 900);		// Aborting
//      abort();
      exit(1);
    }
  err = mon_TxBegin(timeout);
  if (err != MON_SUCCESS)
    {
      CheckTranStatus(err, false, 3003, ErrorLevels::ErrorMsg);
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 900);
//      abort();
      exit(1);
    }
  else
    {
      tranRep = threadTid_Lookup();
      SetUp();			    // Register object and callbacks
#ifndef NO_TRAN_RANDOM_KEY
      securityKey = tran_SecurityKeyCreate((void*)&secureUUID, sizeof(uuid_t));
      tran_Secure(tranRep, securityKey);
#endif
    }
}

/*
 * RegisterBasicCallbacks
 *    Register the callback functions that must be called for every
 * transaction. In our case, this is just a pre-abort callback.
 */

void Transaction::RegisterBasicCallbacks()

{
  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"TID %lu: Registering Monitor abort callback\n",
			tranRep);
  mon_TxReg((void*)&myEntry, sizeof(TranEntry*), 
	    MonAbortCall, MON_ABORT);
}  

/* 
 * EndTimeout
 *    We don't do timeout processing ourselves, so this does nothing.
 */

void Transaction::EndTimeout(TranEntry*)

{
}

