/*
 * MonBinding
 *
 *    A class for Monitor explicit bindings.
 *
 * monbinding.C,v 1.1 1994/01/18 17:14:10 gjb Exp
 */
    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#include <monobjtran/monbinding.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objdce/error.H>
#include <objtran/rpctrancheckstatus.H>
#include <objdce/dcecheckstatus.H>
#include <objtran/tranbinding.H>

/*
 * MonMondList
 *    The constructor just initializes some stuff, then fills the list
 * of server hosts.
 */

MonMondList::MonMondList(const rpc_if_handle_t* ifH)

{
  ifHandle = *ifH;
  RefreshList();
}

/*
 * RefreshList
 *    call mon_GetMondList to fill in the list of server hosts.
 */

void MonMondList::RefreshList()

{
  unsigned long err;
  unsigned long count;
  char** serverList;
  
  Listiter<String> hostIter(mList);
  while (hostIter.remove_next());		// Empty the list
  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Getting a list of monds\n");
  err = mon_GetMondList(ifHandle, &count, &serverList);
  if (err != MON_SUCCESS)
    {
      CheckTranStatus(err, false, ErrorLevels::ErrorMsg,
		      "mon_GetMondList", __LINE__, __FILE__);
    }
  else
    {
      String tmpMond;
	
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Got a list of %li monds\n", count);
      for (unsigned long inc = 0; inc < count; inc++)
	{
	  tmpMond = serverList[inc];
	  mList.put(tmpMond);
	}
      err = mon_Free((void*)serverList);
      CheckTranStatus(err, false, ErrorLevels::ErrorMsg,
		      "mon_Free", __LINE__, __FILE__);
    }
}

/*
 * GetAnyMond
 *    Just return the first item on the list.
 */

const String MonMondList::GetAnyMond() const

{
  return mList[0];
}

/*
 * GetRandomMond
 *    This should randomly select a host to return. For now, just return
 * the first one on the list.
 */

const String MonMondList::GetRandomMond() const

{
  return mList[0];
}

/*
 * MonBinding
 *    The constructors set up internal information from the arguments,
 * then call Init to do the real work.
 */

MonBinding::MonBinding(const rpc_if_handle_t* ifH,
		       const String& mName,
		       LoginContext* login,
		       const char* sPName,
		       ReservationType rType,
		       WaitType wType) 
    : mondName(mName), lookupLatch(MutexLock::NONRECURSIVE)
{
  ifHandle = *ifH;
  if (login == (LoginContext*)0)
    {
      protLevel = LoginContext::NONE;
    }
  else
    {
      protLevel = login->GetLoginProtectionLevel();
    }
  svrPrinName = sPName;
  valid = false;
  looking = false;
  foundMond = false;
  foundPA = false;
  resType = GetResType(rType, wType);
  lastError = 0;
  Init();
}

MonBinding::MonBinding(const rpc_if_handle_t* ifH,
		       const String& mName,
		       LoginContext::ProtectionLevel pLvl,
		       const char* sPName,
		       ReservationType rType,
		       WaitType wType) 
    : mondName(mName), lookupLatch(MutexLock::NONRECURSIVE)
{
  ifHandle = *ifH;
  protLevel = pLvl;
  svrPrinName = sPName;
  valid = false;
  looking = false;
  foundMond = false;
  foundPA = false;
  resType = GetResType(rType, wType);
  lastError = 0;
  Init();
}

/*
 * Init
 *    Do the initial lookup of a binding. If a binding to the mond has
 * not yet been established, get one of those as well.
 */

void MonBinding::Init()

{
  lookupLatch.Lock();
  looking = true;
  lookupLatch.Unlock();
  valid = false;
  if (FindMondHandle())
    {
      if (FindPAHandle())
	{
	  if (SetAuthenticationLevel(protLevel, svrPrinName))
	    {
	      lookupLatch.Lock();
	      valid = true;
	      lookupLatch.Unlock();
	    }
	  else
	    {
	      lookupLatch.Lock();
	      valid = true;
	      lookupLatch.Unlock();
	    }
	}
    }
  lookupLatch.Lock();
  looking = false;
  lookupLatch.Unlock();
  lookupCondition.Broadcast();
}

/*
 * ~MonBinding
 *    Free anything we allocated while we were alive.
 */

MonBinding::~MonBinding()

{
  if (foundMond)
    {
      FreeMondHandle();
    }
  if (foundPA)
    {
      FreePAHandle();
    }
}

/*
 * IsValid
 *    Return whether the last lookup worked.
 */

Boolean MonBinding::IsValid()

{
  Boolean retVal;

  lookupLatch.Lock();
  retVal = valid;
  lookupLatch.Unlock();
  return retVal;
}

/*
 * RebindSameHost
 *    Simply find a PA handle for another server on the same host we
 * used last time.
 */

void MonBinding::RebindSameMond(ReservationType rType,
				WaitType wType)
{
  RebindNewMond(mondName, rType, wType);
}

/*
 * RebindNewHost
 *    Similar to the last one, but we also try to bind to a new mond
 * before looking for the PA. If the new host name is the same as the 
 * old one, don't bother to look up the Mond again.
 */

void MonBinding::RebindNewMond(const String& newMondName,
			       ReservationType rType,
			       WaitType wType)
{
  Boolean doRebind = false;
  Boolean foundIt = false;

  lookupLatch.Lock();
  valid = false;
  while (looking)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Waiting for a rebind to complete\n");
      lookupCondition.Wait(lookupLatch);
    }
  if (!valid)
    {
      doRebind = true;
      looking = true;
    }
  lookupLatch.Unlock();

  if (doRebind)
    {
      if (foundPA)
	{
	  FreePAHandle();
	}
      if (!foundMond || (newMondName != mondName))
	{
	  if (foundMond)
	    {
	      FreeMondHandle();
	    }
	  mondName = newMondName;
	  FindMondHandle();
	}
      if (foundMond)
	{
	  resType = GetResType(rType, wType);
	  foundIt = FindPAHandle();
	}
    }
  
  if (foundIt)
    {
      foundIt = SetAuthenticationLevel(protLevel, svrPrinName);
    }
  if (foundIt)
    {
      lookupLatch.Lock();
      valid = true;
      looking = false;
      lookupLatch.Unlock();
      lookupCondition.Broadcast();
    }
  else
    {
      lookupLatch.Lock();
      valid = false;
      looking = false;
      lookupLatch.Unlock();
      lookupCondition.Broadcast();
    }
}

/*
 * RebindIfNecessary
 *    Check the error type, and if it was a communication error, go
 * on and do a rebind. The first form doesn't take a hostname, so it
 * always rebinds using the same Mond it used before. The second
 * takes a hostname and binds to a new Mond.
 */

Boolean MonBinding::RebindIfNecessary(Transaction& tran,
                                      ReservationType rType,
                                      WaitType wType,
                                      const int numTries,
                                      const int sleepSeconds)
{
  return RebindIfNecessary(tran, mondName, rType, wType, 
			   numTries, sleepSeconds);
}

Boolean MonBinding::RebindIfNecessary(Transaction& tran,
				      const String& newMondName,
				      ReservationType rType,
                                      WaitType wType,
                                      const int numTries,
                                      const int sleepSeconds)
{  
  int curTry;
  timespec sleepTime;
  
  sleepTime.tv_sec = sleepSeconds;
  sleepTime.tv_nsec = 0;
  if (TranBinding::CheckErrorType(tran) == Binding::ET_COMMUNICATION_ERROR)
    {
      Boolean doRebind = false;

      lookupLatch.Lock();
      valid = false;
      while (looking)
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg,
			       "Waiting for another rebind to complete\n");
	  lookupCondition.Wait(lookupLatch);
	}
      if (!valid)
	{
	  doRebind = true;
	  looking = true;
	}
      lookupLatch.Unlock();

      if (doRebind)
	{
	  if (foundPA)
	    {
	      FreePAHandle();
	    }
	  if (!foundMond || (newMondName != mondName))
	    {
	      if (foundMond)
		{
		  FreeMondHandle();
		}
	      mondName = newMondName;
	      FindMondHandle();
	    }
	  resType = GetResType(rType, wType);

	  for (curTry = 0; (!IsValid()) && (curTry < numTries); curTry++)
	    {
	      if (curTry > 0)
		{
		  ObjtranReport.Report(ErrorLevels::TraceMsg,
			       "Sleeping before next rebind attempt...\n");
		  pthread_delay_np(&sleepTime);
		}
	      if (FindPAHandle())
		{
		  if (SetAuthenticationLevel(protLevel, svrPrinName))
		    {
		      lookupLatch.Lock();
		      valid = true;
		      lookupLatch.Unlock();
		    }
		  else
		    {
		      lookupLatch.Lock();
		      valid = false;
		      lookupLatch.Unlock();
		    }
		}
	    }
	  lookupLatch.Lock();
	  looking = false;
	  lookupLatch.Unlock();
	  lookupCondition.Broadcast();
	}
      return true;
    }
  else
    {
      return false;
    }
}

/*
 * SetAuthenticationLevel
 *    Set the authentication level of this binding. It may be worth 
 * mucking with inheritance, because this is just cut and pasted
 * from objdce/binding.C.
 */

Boolean MonBinding::SetAuthenticationLevel(LoginContext::ProtectionLevel pLvl, 
					   const char* svrPrin)
{
  unsigned long status = rpc_s_ok;
  unsigned char* tmpSvrName;
  unsigned char* strBinding;
  unsigned long dceProtLevel;
  String serverName;
  
  ObjtranReport.Report(ErrorLevels::TraceMsg, "In Set Authentication Level\n");
  protLevel = pLvl;
  if (pLvl != LoginContext::NONE)
    {
      handle_t rpcRep;
      status = trpc_GetRpcHandleFromBinding(paHandle, &rpcRep);
      CheckStatus(status, false, ErrorLevels::ErrorMsg,
		  "trpc_GetRpcHandleFromBinding", __LINE__, __FILE__);
      rpc_binding_to_string_binding(rpcRep, &strBinding, &status);
      if (status != error_status_ok)
	{
	  CheckStatus(status, false, ErrorLevels::ErrorMsg,
		      "rpc_binding_to_string_binding", 
		      __LINE__, __FILE__);
	}
      else
	{
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"Setting authentication on\n\t%s\n",
				strBinding);
	  rpc_string_free(&strBinding, &status);
	  CheckStatus(status, false, ErrorLevels::ErrorMsg,
		      "rpc_string_free", __LINE__, __FILE__);
	}

      if ((svrPrin == (char*)0) || (svrPrin[0] == '\0'))
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Calling rpc_mgmt_inq_server_princ_name\n");
	  rpc_mgmt_inq_server_princ_name(rpcRep, rpc_c_authn_dce_secret,
					 &tmpSvrName, &status);
	  if (status != error_status_ok)
	    {
	      CheckStatus(status, false, ErrorLevels::ErrorMsg, 
			  "rpc_mgmt_inq_server_princ_name",
			  __LINE__, __FILE__);
	      return false;
	    }
	  serverName = (const char*)tmpSvrName;
	  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
             "Setting server name to %s after inq_server_princ_name call\n", 
				(const char*)serverName);
	  rpc_string_free(&tmpSvrName, &status);
	  CheckStatus(status, true, ErrorLevels::FatalMsg, 
		      "rpc_string_free", __LINE__, __FILE__);
	}
      else
	{
	  serverName = svrPrin;
	}
      tmpSvrName = (unsigned char*)((const char*)serverName);
      dceProtLevel = LoginContext::DceProtectionLevel(pLvl);
      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
			    "Setting DCE Protection Level to %lu\n", 
			    dceProtLevel);
      ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "Calling rpc_binding_set_auth_info\n");
      rpc_binding_set_auth_info(rpcRep, tmpSvrName, dceProtLevel,
				rpc_c_authn_dce_secret, NULL, rpc_c_authz_dce,
				&status);
      if (status != error_status_ok)
	{
	  CheckStatus(status, false, ErrorLevels::ErrorMsg, 
		      "rpc_binding_set_auth_info", __LINE__, __FILE__);
	  lastError = status;
	  return false;
	}
      else
	{
	  lastError = status;
	  return true;
	}
    }
  else
    return true;
}

/*
 * FindMondHandle
 *    Get a binding to the Mond for a specific host. If we find it,
 * set the mond handle and "foundMond" flag to indicate we got one.
 * The "looking" flag must be set to "true" before calling this.
 */

Boolean MonBinding::FindMondHandle()

{
  unsigned long err;

  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"Getting a mond handle for \"%s\"\n",
			(const char*)mondName);
  err = mon_GetMondHandle(ifHandle, (char*)((const char*)mondName),
			  &mondHandle);
  if (err != MON_SUCCESS)
    {
      CheckTranStatus(err, false, ErrorLevels::ErrorMsg,
		      "mon_GetMondHandle", __LINE__, __FILE__);
      foundMond = false;
    }
  else
    {
      ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 3014,
			       (const char*)mondName);
      foundMond = true;
    }
  lastError = err;
  return foundMond;
}

/*
 * FreeMondHandle
 *    Free a mond handle returned by "FindMondHandle". The "looking"
 * flag must be set before we call this.
 */

void MonBinding::FreeMondHandle()

{
  unsigned long err;

  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Freeing the Mond handle\n");
  rpc_binding_free(&mondHandle, &err);
  CheckTranStatus(err, false, ErrorLevels::WarnMsg,
		  "rpc_binding_free", __LINE__, __FILE__);
  foundMond = false;
  lastError = err;
}

/*
 * FindPAHandle
 *    Find the handle to a PA. This requires that a handle has already
 * been found to an appropriate mond, and that the "looking" flag has
 * been set.
 */

Boolean MonBinding::FindPAHandle()

{
  unsigned long err;
  unsigned char* strB;

  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Looking up a PA handle\n");
  err = mon_GetPaHandle(mondHandle, resType, &paHandle);
  if (err != MON_SUCCESS)
    {
      CheckTranStatus(err, false, ErrorLevels::ErrorMsg,
		      "mon_GetPaHandle", __LINE__, __FILE__);
      foundPA = false;
    }
  else
    {
      foundPA = true;
      err = trpc_BindingToStringBinding(paHandle, &strB);
      if (err != 0)
	{
	  CheckTranStatus(err, false, ErrorLevels::WarnMsg,
			  "trpc_BindingToStringBinding", __LINE__, __FILE__);
	}
      else
	{
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 3015,
				   strB);
	  strBinding = (const char*)strB;
	  trpc_Free((void*)paHandle);	
	}
    }
  lastError = err;
  return foundPA;
}
	  
/*
 * FreePAHandle
 *    Free a PA handle found by FindPAHandle.
 */

void MonBinding::FreePAHandle()

{
  unsigned long err;

  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Freeing the PA handle\n");
  err = trpc_FreeBinding(paHandle);
  CheckTranStatus(err, false, ErrorLevels::WarnMsg,
		  "trpc_FreeBinding", __LINE__, __FILE__);
  foundPA = false;
  lastError = err;
}

/*
 * GetResType
 *    Given a ReservationType and a WaitType, "or" the bits together
 * into a proper mon_reservation_t by the Monitor.
 */

mon_reservation_t MonBinding::GetResType(ReservationType rType,
					 WaitType wType) const
{
  mon_reservation_t retVal = 0;
 
  switch (wType)
    {
    case WT_WAIT:
      retVal = MON_WAIT_IF_BUSY;
      break;
    case WT_RETURN:
      retVal = MON_RETURN_IF_BUSY;
      break;
    }
  switch (rType)
    {
    case RT_LONG:
      retVal |= MON_LONG_RESERVATION;
      break;
    case RT_TRANSACTIONAL:
      break;
    }
  return retVal;
}

/*
 * WaitIfRebind
 *    If another thread is doing a lookup, wait on the condition variable.
 * This means one thread can safely rebind while everyone else is trying
 * to make RPCs. Return true if a rebind operation was in progress.
 */

Boolean MonBinding::WaitIfRebind()

{
  Boolean retVal = false;

  lookupLatch.Lock();
  while (looking)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "Waiting while another thread rebinds\n");
      retVal = true;
      lookupCondition.Wait(lookupLatch);
    }
  lookupLatch.Unlock();
  return retVal;
}

