/**
 ** CBinding.C
 **
 **++
 **  FACILITY:
 **
 **      OBJDCE
 **
 **  MODULE DESCRIPTION:
 **
 **      This module contains various API routines for the OBJDCE Binding class.
 **
 **  AUTHORS:
 **
 **      James E. Bergman
 **
 **  CREATION DATE:
 **
 **      18-Oct-1993
 **
 **--
 **/





    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


//
// Include Files
//

#include <stdlib.h>
#include <strings.h>
#include <objdce/statics.H>
#include <objdce/login.H>
#include <objdce/error.H>
#include <objdce/c_api.h>


//
// Module-Wide Data
//


//
// Global References
//

extern LoginContext *c_api_contextPtr;		// Base pointer to all login contexts

//
//++
//  Functional Description
//
//      Bind_NameServer_NPL
//
//	    This procedure is used to create a binding using a cds name and interface
//	    specification to locate the server. It assumes a current context login,
//	    a server principal name of "" and no protection level.
//	
//
//  Formal Parameters
//
//
//	cdsName
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    Read Only
//	Mechanism:  by reference
//
//	    The name of the server as defined in the cell directory
//
//	ifHandle
//
//	Usage:	    interface specification
//	Type :	    rpc_if_handle_t
//	Access:	    read-only
//	Mechanism:  by reference
//
//	    Pointer to rpc interface handle
//
//
//  Return Value
//
//      bindingP    -- A binding handle -- (Valid only if returnStatus == 0)
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

bindingP Bind_NameServer_NPL(const char *cdsName,
			       const rpc_if_handle_t* ifHandle,
			       unsigned32 * returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding* bindPtr;

    //
    // Procedure: Bind_NameServer_NPL
    //

    ObjtranFirstInit();
    *returnStatus = 0;
    bindPtr = new Binding(cdsName, ifHandle, c_api_contextPtr);	    // Create binding instance
    if (bindPtr)
	{
	if (bindPtr->IsValid())
	    {
	    return ((bindingP)bindPtr);				    // return object's address
	    }
	else
	    {
	    *returnStatus = bindPtr->GetDCEError();		    // Get actual error
	    }
	}
    return((bindingP)0);					    // Return Null
}
//
//++
//  Functional Description
//
//      Bind_NameServer_PL
//
//	    This procedure is used to create a binding using a cds name and interface
//	    specification to locate the server. It allows for setting the protection
//	    level and server principal name.
//
//  Formal Parameters
//
//	cdsName
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    Read Only
//	Mechanism:  by reference
//
//	    The name of the server as it appears in the cell directory
//
//	ifHandle
//
//	Usage:	    interface specification
//	Type :	    rpc_if_handle_t
//	Access:	    read-only
//	Mechanism:  by reference
//
//	    Pointer to rpc interface handle
//
//	protLevel
//
//	Usage:	    dce security protection level
//	Type :	    enum ProtectionLevel
//	Access:	    read-only
//	Mechanism:  by reference
//
//	    protection level desired
//
//	svrPrinName
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    read-only
//	Mechanism:  by reference
//
//	    Principal name
//
//  Return Value
//
//      bindingP    -- binding handle -- (Valid only if returnStatus == 0)
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

bindingP Bind_NameServer_PL(const char *cdsName,
			      const rpc_if_handle_t* ifHandle,
			      ProtectionLevel protLevel,
			      const char *svrPrinName,
			      unsigned32 * returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr;


    //
    // Procedure: Bind_NameServer_PL
    //

    ObjtranFirstInit();
    *returnStatus = 0;
    bindPtr = new Binding(cdsName, ifHandle, ConvertProtectionLevel(protLevel), svrPrinName);
    if (bindPtr)
	{
	if (bindPtr->IsValid())
	    {
	    return ((bindingP)bindPtr);				    // return object's address
	    }
	else
	    {
	    *returnStatus = bindPtr->GetDCEError();		    // Get actual error
	    }
	}
    return((bindingP)0);					    // Return Null
}

//
//++
//  Functional Description
//
//      Bind_EndPoint_NPL
//
//	    This procedure is used to create a binding using a weel known end point
//	    specification to locate the server. It does allow for setting the protection
//	    level and server principal name.
//
//  Formal Parameters
//
//	hostName
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    Read Only
//	Mechanism:  by reference
//
//	    The name of the host where the well known endpoint lives
//
//	port
//
//	Usage:	    port
//	Type :	    longword integer
//	Access:	    read-only
//	Mechanism:  by value
//
//	    Endpoint specification
//
//	whatProtocol
//
//	Usage	 :  protocol
//	Type	 :  enum Protocol
//	Access	 :  read-only
//	Mechanism:  By value
//
//  Return Value
//
//      bindingP    -- binding handle -- (Valid only if returnStatus == 0)
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

bindingP Bind_EndPoint_NPL(const char *hostName,
			     const long port,
			     Protocol whatProtocol,
			     unsigned32 *returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr;

    //
    // Procedure: Bind_EndPoint_NPL
    //

    ObjtranFirstInit();
    *returnStatus = 0;
    bindPtr = new Binding(hostName,
			  port,
			  c_api_contextPtr,
			  (const rpc_if_handle_t*)0,
			  "",
			  CvtProtocol(whatProtocol));
    if (bindPtr)
	{
	if (bindPtr->IsValid())
	    {
	    return ((bindingP)bindPtr);				    // return object's address
	    }
	else
	    {
	    *returnStatus = bindPtr->GetDCEError();		    // Get actual error
	    }
	}
    return((bindingP)0);					    // Return Null
}
//
//++
//  Functional Description
//
//      Bind_EndPoint_PL
//
//	    This procedure is used to create a binding using a weel known end point
//	    specification to locate the server. It does allow for setting the protection
//	    level and server principal name.
//
//  Formal Parameters
//
//	hostName
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    Read Only
//	Mechanism:  by reference
//
//	    The name of the host where the well known endpoint lives
//
//	port
//
//	Usage:	    port
//	Type :	    longword integer
//	Access:	    read-only
//	Mechanism:  by value
//
//	    Endpoint specification
//
//	whatProtocol
//
//	Usage	 :  protocol
//	Type	 :  enum Protocol
//	Access	 :  read-only
//	Mechanism:  By value
//
//  Return Value
//
//      bindingP    -- binding handle -- (Valid only if returnStatus == 0)
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

bindingP Bind_EndPoint_PL(const char *hostName,
			    const rpc_if_handle_t* ifHandle,
			    const long port,
			    ProtectionLevel protLevel,
			    const char *svrPrinName,
			    Protocol whatProtocol,
			    unsigned32 * returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr;

    //
    // Procedure: Bind_EndPoint_PL
    //

    ObjtranFirstInit();
    *returnStatus = 0;
    bindPtr = new Binding(hostName,
			  port,
			  ConvertProtectionLevel(protLevel),
			  ifHandle,
			  svrPrinName,
			  CvtProtocol(whatProtocol));
    if (bindPtr)
	{
	if (bindPtr->IsValid())
	    {
	    return ((bindingP)bindPtr);				    // return object's address
	    }
	else
	    {
	    *returnStatus = bindPtr->GetDCEError();		    // Get actual error
	    }
	}
    return((bindingP)0);					    // Return Null
}
//
//++
//  Functional Description
//
//      Bind_StringBind_NPL
//
//	    This procedure is used to create a binding using a weel known end point
//	    specification to locate the server. It does allow for setting the protection
//	    level and server principal name.
//
//  Formal Parameters
//
//	bindingString
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    Read Only
//	Mechanism:  by reference
//
//	    The name of the host where the well known endpoint lives
//
//	returnStatus
//
//	Usage:	    dce status value
//	Type :	    longword integer
//	Access:	    modify
//	Mechanism:  by reference
//
//	    Where the dce status gets returned upon failure
//
//
//  Return Value
//
//      bindingP    -- binding handle -- (Valid only if returnStatus == 0)
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

bindingP Bind_StringBind_NPL(const char *bindingString,
			       unsigned32 * returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr;

    //
    // Procedure: Bind_StringBind_NPL
    //

    ObjtranFirstInit();
    *returnStatus = 0;
    bindPtr = new Binding(bindingString, c_api_contextPtr, "");
    if (bindPtr)
	{
	if (bindPtr->IsValid())
	    {
	    return ((bindingP)bindPtr);				    // return object's address
	    }
	else
	    {
	    *returnStatus = bindPtr->GetDCEError();		    // Get actual error
	    }
	}
    return((bindingP)0);					    // Return Null

}
//
//++
//  Functional Description
//
//      Bind_StringBind_PL
//
//	    This procedure is used to create a binding using a weel known end point
//	    specification to locate the server. It does allow for setting the protection
//	    level and server principal name.
//
//  Formal Parameters
//
//	hostName
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    Read Only
//	Mechanism:  by reference
//
//	    The name of the host where the well known endpoint lives
//
//	port
//
//	Usage:	    port
//	Type :	    longword integer
//	Access:	    read-only
//	Mechanism:  by value
//
//	    Endpoint specification
//
//
//  Return Value
//
//      bindingP    -- binding handle -- (Valid only if returnStatus == 0)
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

bindingP Bind_StringBind_PL(const char *bindingString,
			      ProtectionLevel protLevel,
			      const char *svrPrinName,
			      unsigned32 * returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr;

    //
    // Procedure: Bind_StringBind_PL
    //

    ObjtranFirstInit();
    *returnStatus = 0;
    bindPtr = new Binding(bindingString, ConvertProtectionLevel(protLevel), svrPrinName);
    if (bindPtr)
	{
	if (bindPtr->IsValid())
	    {
	    return ((bindingP)bindPtr);				    // return object's address
	    }
	else
	    {
	    *returnStatus = bindPtr->GetDCEError();		    // Get actual error
	    }
	}
    return((bindingP)0);					    // Return Null
}
//
//++
//  Functional Description
//
//      Bind_UUID_NPL
//
//
//  Formal Parameters
//
//	cdsName
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    Read Only
//	Mechanism:  by reference
//
//	    The name of the server as it appears in the cell directory
//
//	ifHandle
//
//	Usage:	    interface specification
//	Type :	    rpc_if_handle_t
//	Access:	    read-only
//	Mechanism:  by reference
//
//	    Pointer to rpc interface handle
//
//	objectUUID
//
//	Usage:	    uuid
//	Type:	    uuid_t**
//	Access:	    read-only
//	Mechanism:  by reference
//
//	    An object's uuid for which to look
//
//	whatProtocol
//
//	Usage	 :  protocol
//	Type	 :  enum Protocol
//	Access	 :  read-only
//	Mechanism:  By value
//
//
//	    Specifies the DCE protocol to use
//
//  Return Value
//
//      bindingP    address of binding handle -- (Valid only if returnStatus == 0)
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

bindingP Bind_UUID_NPL(const char *cdsName,
			 const rpc_if_handle_t* ifHandle,
			 uuid_t * objectUUID,
			 Protocol whatProtocol,
			 unsigned32 * returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr;
    DCEuuid *dceUuidPtr;

    //
    // Procedure: Bind_UUID_NPL
    //

 
    ObjtranFirstInit();
    dceUuidPtr = new DCEuuid(objectUUID);
    *returnStatus = 0;
    bindPtr = new Binding(cdsName,			// Server name in cds format
			  ifHandle,			// interface spec handle
			  dceUuidPtr,			// object's uuid
			  c_api_contextPtr,		// c_api login context
			  "",				// DCE principal ID (default)
			  CvtProtocol(whatProtocol));	// transport mechanism
    delete dceUuidPtr;
    if (bindPtr)
	{
	if (bindPtr->IsValid())
	    {
	    return ((bindingP)bindPtr);				    // return object's address
	    }
	else
	    {
	    *returnStatus = bindPtr->GetDCEError();		    // Get actual error
	    }
	}
    return((bindingP)0);					    // Return Null
}
//
//++
//  Functional Description
//
//      Bind_UUID_PL
//
//
//  Format Parameters
//
//	cdsName
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    Read Only
//	Mechanism:  by reference
//
//	    The name of the server as it appears in the cell directory
//
//	ifHandle
//
//	Usage:	    interface specification
//	Type :	    rpc_if_handle_t
//	Access:	    read-only
//	Mechanism:  by reference
//
//	    Pointer to rpc interface handle
//
//	objectUUID
//
//	Usage:	    uuid
//	Type:	    uuid_t*
//	Access:	    read-only
//	Mechanism:  by reference
//
//	    An object's uuid for which to look
//
//	protLevel
//
//	Usage	 :  protection level
//	Type	 :  enum ProtectionLevel
//	Access	 :  read-only
//	Mechanism:  By value
//
//	    This is the level of DCE security desired
//
//	svrPrinName
//
//	Usage	 :  character string
//	Type	 :  char *
//	Access	 :  read-only
//	Mechanism:  by reference
//
//	    The name of the DCE principal ID
//
//	whatProtocol
//
//	Usage	 :  protocol
//	Type	 :  enum Protocol
//	Access	 :  read-only
//	Mechanism:  By value
//
//
//	    Specifies the DCE protocol to use
//
//  Return Value
//
//      bindingP    address of binding handle -- (Valid only if returnStatus == 0)
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

bindingP Bind_UUID_PL(const char *cdsName,
			const rpc_if_handle_t* ifHandle,
			uuid_t * objectUUID,
			ProtectionLevel protLevel,
			const char * svrPrinName,
			Protocol whatProtocol,
			unsigned32 * returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr;
    DCEuuid *dceUuidPtr;

    //
    // Procedure: Bind_UUID_NPL
    //

    ObjtranFirstInit();
    dceUuidPtr = new DCEuuid(objectUUID);
    *returnStatus = 0;
    bindPtr = new Binding(cdsName,			// Server name in cds format
			  ifHandle,			// interface spec handle
			  dceUuidPtr,			// object's uuid
			  ConvertProtectionLevel(protLevel),
			  svrPrinName,			// DCE principal ID
			  CvtProtocol(whatProtocol));	// transport mechanism
    delete dceUuidPtr;
    if (bindPtr)
	{
	if (bindPtr->IsValid())
	    {
	    return ((bindingP)bindPtr);			// return object's address
	    }
	else
	    {
	    *returnStatus = bindPtr->GetDCEError();	// Get actual error
	    }
	}
    return((bindingP)0);					    // Return Null
}

//
//++
//  Functional Description
//
//      GetBindingHandle
//	
//
//  Formal Parameters
//
//	bPtr
//
//	Usage:	    Binding*
//	Type:	    bindingP
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.
//
//  Return Value
//
//      The rpc_binding_handle_t of the object
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

rpc_binding_handle_t GetBindingHandle(bindingP bPtr)

/***************************************************************************/

{

    //
    // Local Data
    //


    //
    // Procedure: GetBindingHandle
    //

    if (bPtr)
	{
	return(*((Binding*)bPtr));		    // Return rpc_binding_handle_t
	}
    else
	{
	return((rpc_binding_handle_t)0);	    // Return null
	}
}
//
//++
//  Functional Description
//
//      Bind_ReleaseHandle
//
//	    This procedure will invoke the destructor for the Binding object
//
//  Formal Parameters
//
//
//	bindingPtr
//
//	Usage:	    Binding*
//	Type :	    bindingP
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.  
//
//  Return Value
//
//      None
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

void Bind_ReleaseHandle(bindingP bPtr)

/***************************************************************************/

{

    //
    // Local Data
    //


    //
    // Procedure: Bind_ReleaseHandle
    //

    delete (Binding*)bPtr;		// Invoke destructor

}
//
//++
//  Functional Description
//
//      RebindSameHost
//
//	    Invoke the RebindSameHost method in the Binding class.
//
//  Formal Parameters
//
//
//	bindingPtr
//
//	Usage:	    Binding*
//	Type :	    bindingP
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.
//
//  Return Value
//
//      unsigned32	0 if successfull
//			other value if failed. Actual DCE error encountered
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

unsigned32 Bind_RebindSameHost(bindingP bindingPtr)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr = (Binding*)bindingPtr;
    unsigned32	retValue = 0;

    //
    // Procedure: RebindSameHost
    //

    bindPtr->RebindSameHost();
    if (!bindPtr->IsValid())
	{
	retValue = bindPtr->GetDCEError();		    // Get actual error
	}
    return(retValue);
}
//
//++
//  Functional Description
//
//      RebindNewHost
//
//	    Invokes the RebindNewHost method in the Binding class
//
//  Formal Parameters
//
//
//	bindingPtr
//
//	Usage:	    Binding*
//	Type :	    bindingP
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.
//
//
//  Return Value
//
//      unsigned32 -- 0 if successful
//		      other value if failed. Actual DCE error encountered
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

unsigned32 Bind_RebindNewHost(bindingP bindingPtr)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding *bindPtr = (Binding*)bindingPtr;
    unsigned32	returnValue = 0;

    //
    // Procedure: RebindSameHost
    //

    bindPtr->RebindNewHost();			    // Invoke method
    if (!bindPtr->IsValid())			    // Did it work?
	{
	returnValue = bindPtr->GetDCEError();	    // No. Get actual error
	}
    return(returnValue);
}
//
//++
//  Functional Description
//
//      IsServerListening
//
//	    Invokes the IsServerListening method in the Binding Class
//
//  Formal Parameters
//
//
//	bindingPtr
//
//	Usage:	    Binding*
//	Type :	    bindingP
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.
//
//	returnStatus
//
//	Usage:	    DCE status
//	Type :	    unsigned32
//	Access:	    write-only
//	Mechanism:  By reference
//
//	    This is the return status of the procedure.
//
//		    0	    Call was successfull
//		    other   DCE error Code
//
//  Return Value
//
//      Boolean -- True if server is listening
//		   False if server not listening
//
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

Boolean Bind_IsServerListening(bindingP bindingPtr, unsigned32 *returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding	*bindPtr = (Binding*)bindingPtr;	// Recast our internal handle
    Boolean	returnValue = true;			// Assume successful
    Boolean	serverListening;			// Return from method

    //
    // Procedure: IsServerListening
    //

    serverListening = bindPtr->IsServerListening();	// Invoke method
    if (serverListening == false)			// Is server listening?
	{
	*returnStatus = bindPtr->GetDCEError();		// Get actual error
	returnValue = false;				// Set return value
	}
    return(returnValue);
}
//
//++
//  Functional Description
//
//      StopServerListening
//
//	    Invokes the StopServerListening method in the Binding Class
//
//  Formal Parameters
//
//
//	bindingPtr
//
//	Usage:	    Binding*
//	Type :	    bindingP
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.
//
//	returnStatus
//
//	Usage:	    DCE status
//	Type :	    unsigned32
//	Access:	    write-only
//	Mechanism:  By reference
//
//	    This is the return status of the procedure.
//
//		    0	    Call was successfull
//		    other   DCE error Code
//
//  Return Value
//
//      Boolean -- True if server was successfully stopped
//		   False if error or not valid
//
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

Boolean Bind_StopServerListening(bindingP bindingPtr, unsigned32 *returnStatus)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding	*bindPtr = (Binding*)bindingPtr;	// Recast our internal handle
    Boolean	stopServerStatus, returnValue;

    //
    // Procedure: StopServerListening
    //

    returnValue = true;					// Assume success
    stopServerStatus = bindPtr->StopServerListening();	// Invoke method
    if (stopServerStatus == false)			// Stopped successfully?
	{
	*returnStatus = bindPtr->GetDCEError();		// Get dce error
	returnValue = false;
	}
    return(returnValue);
}
//
//++
//  Functional Description
//
//      RebindIfNecessary
//
//	    Invokes the RebindIfNecessary method in the Binding Class
//
//  Formal Parameters
//
//
//	bindingPtr
//
//	Usage:	    Binding*
//	Type :	    bindingP
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.
//
//	numTries
//
//	Usage:	    int
//	Type :	    int
//	Access:	    read-only
//	Mechanism:  By value
//
//	    Number of times to attempt rebind
//
//	sleepTime
//
//	Usage:	    timespec
//	Type :	    int
//	Access:	    read-only
//	Mechanism:  By value
//
//	    This is the number of seconds to wait in between rebind attempts
//
//  Return Value
//
//      Boolean -- True if server was successfully stopped
//		   False if error or not valid
//
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

Boolean Bind_RebindIfNecessary(bindingP bindingPtr,
			  error_status_t errorCode,
			  int numTries,
			  int sleepTime)

/***************************************************************************/

{

    //
    // Local Data
    //

    Binding	*bindPtr = (Binding*)bindingPtr;	// Recast our internal handle
    Boolean	returnValue;

    //
    // Procedure: RebindIfNecessary
    //

    returnValue = bindPtr->RebindIfNecessary(errorCode,
					     numTries,
					     sleepTime);
    return(returnValue);
}
//
//++
//  Functional Description
//
//      GetErrorType
//
//	    Invokes the RebindIfNecessary method in the Binding Class
//
//  Formal Parameters
//
//
//	bindingPtr
//
//	Usage:	    Binding*
//	Type :	    bindingP
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.
//
//	numTries
//
//	Usage:	    int
//	Type :	    int
//	Access:	    read-only
//	Mechanism:  By value
//
//	    Number of times to attempt rebind
//
//	sleepTime
//
//	Usage:	    timespec
//	Type :	    int
//	Access:	    read-only
//	Mechanism:  By value
//
//	    This is the number of seconds to wait in between rebind attempts
//
//  Return Value
//
//      Boolean -- True if server was successfully stopped
//		   False if error or not valid
//
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

ErrorType GetErrorType(error_status_t errorCode)

/***************************************************************************/

{

    //
    // Local Data
    //


    //
    // Procedure: GetErrorType
    //

    return(CvtErrorType(Binding::CheckErrorType(errorCode)));
}
//
//++
//  Functional Description
//
//      GetErrorString
//
//	    Invokes the CheckErrorType with string return  method in the Binding Class
//
//  Formal Parameters
//
//
//	errorCode
//
//	Usage:	    error status
//	Type :	    error_status_t
//	Access:	    read-only
//	Mechanism:  by value
//
//	    The binding handle returned from the Bind_xxx procedures.
//
//	returnString
//
//	Usage:	    character string
//	Type :	    char *
//	Access:	    modify
//	Mechanism:  by reference
//
//	returnStringSize
//
//	Usage:	    string length
//	Type:	    int
//	Access:	    read-only
//	Mechanism:  by value
//
//	    This is size of the returnString buffer
//
//  Return Value
//
//
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

void GetErrorString(error_status_t errorCode,
		    char *returnString,
		    int returnStringSize)

/***************************************************************************/

{

    //
    // Local Data
    //

    String	errorString;
    int		copyLength = returnStringSize;

    //
    // Procedure: GetErrorString
    //

    Binding::CheckErrorType(errorCode, errorString);
    if (errorString.length() < returnStringSize)
	{
	copyLength = errorString.length();			// Just copy what we need
	}
    strncpy(returnString, (const char*)errorString, copyLength - 1);
    returnString[copyLength] = '\0';		    // Null terminate
}
