/**
 ** CLogin.C
 **
 **++
 **  FACILITY:
 **
 **      OBJDCE
 **
 **  MODULE DESCRIPTION:
 **
 **      This module contains various API routines for the OBJDCE Login class.
 **
 **  AUTHORS:
 **
 **      James E. Bergman
 **
 **  CREATION DATE:
 **
 **      13-Oct-1993
 **
 **--
 **/


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


//
// Include Files
//

#include <stdlib.h>
#include <strings.h>
#include <objdce/statics.H>
#include <objdce/login.H>
#include <objdce/error.H>
#include <objdce/c_api.h>

//
// Module-Wide Data
//

static	Boolean		LoggedOn = false;   // If true, than Login performed

//
// Module-Wide Classes
//


//
// Global Declarations
//

//
// This is the login Context
//

LoginContext	*c_api_contextPtr = NULL; // Base pointer to all login contexts

//
// Here we subclass PasswordLoginContext so we can use our own
// RefreshPromptForPassword method.
//

class PswLoginCtxUserFnc : public PasswordLoginContext
    {
    public:
	PswLoginCtxUserFnc(void (*uRR)(char *, char *), const char *);    // Constructor
	virtual ~PswLoginCtxUserFnc();	    			    // Destructor
	void RefreshPromptForPassword(char *);			    // Define virtual function
	void (*UserRefreshRoutine)(char *, char *);		    // Hold's users routine
    private:
	char *prompt;						    // Password Prompt
    };

//
//++
//  Functional Description
//
//      This function implements the CurrentLoginContext Class member
//	function: LogIn
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  ProtectionLevel   By Value	    This is the protection level to be set
//
//  Return Value
//
//      LoginStatus -- enum defined in login.h
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

LoginStatus Obj_Login_CurrentCtx(ProtectionLevel protLevel)

/**************************************************************************/
{

     //
     //	Local Data
     //

    static  Boolean	  contextEstablished = false;	    // If true, than context allocated
    static  CurrentLoginContext * loginContext;		    // Ptr to context block
    LoginStatus		  returnValue = LGI_NONE;	    // Assume NULL return status
    LoginContext::LoginStatus	lStatus;		    // ObjDce login return status

    //
    // Procedure: Obj_Login_CurrentCtx
    //

    ObjtranFirstInit();
    if (LoggedOn == true)				    // Check if already logged in
	{
	returnValue = LGI_ALREADY_LOGGED_IN;		    // Yes. Set status
	}
    if (returnValue == LGI_NONE)			    // If we should attempt to perform login
	{
	if (contextEstablished == false)		    // Check if context block allocated
	    {
	    //
	    //	Have to establish a context
	    //
	    loginContext = new CurrentLoginContext();	    // No. Create an instance
	    c_api_contextPtr = loginContext;			    // Set module-wide context ptr
	    contextEstablished = true;			    // Set flag
	    }
	//
	// Try to Log In
	//
	lStatus = loginContext->LogIn(ConvertProtectionLevel(protLevel));
	if (lStatus != LoginContext::SUCCESS)
	    {
	    ObjtranReport.Report(ErrorLevels::ErrorMsg, "Failed To LogIn\n");
	    returnValue = LGI_FAILURE;
	    }
	else
	    {
	    returnValue = LGI_SUCCESS;
	    LoggedOn = true;		// We are in like flint
	    }
	}
    return (returnValue);
}

//
//++
//  Functional Description
//
//      This function implements the KeytableLoginContext Class member
//	function: LogIn
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  char *	     By Reference  This is the DCE principal name
//  char *	     By Reference  This is the full pathname of the keytable
//  ProtectionLevel  By Value	   This is the protection level to be set
//  unsigned32	     By Value	   Set to AUTO_MANAGE or AUTO_REFRESH
//
//  Return Value
//
//      LoginStatus -- enum defined in login.h
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

LoginStatus Obj_Login_Keytable(const char *principalName,
			       const char *keytableName,
			       ProtectionLevel protLevel,
			       unsigned32      flags)

/***************************************************************************/

{

    //
    // Local Data
    //

    static  KeytableLoginContext * loginContext;	    // Ptr to context block
    static  Boolean	   contextEstablished = false;	    // If true, context allocated
    LoginStatus		   returnValue = LGI_NONE;	    // Assume NULL return status
    LoginContext::LoginStatus lStatus;			    // ObjDce login return status

    //
    // Procedure: Obj_Login_Keytable
    //


    ObjtranFirstInit();
    if (LoggedOn == true)				    // Check if already logged in
	{
	returnValue = LGI_ALREADY_LOGGED_IN;		    // Yes. Set return status
	}
    if (returnValue == LGI_NONE)			    // If we should attempt to perform login
	{
        if (contextEstablished == false)		    // Context allocated?
	    {
	    loginContext = new KeytableLoginContext();	    // No. Allocate
	    c_api_contextPtr = loginContext;			    // Set module-wide context ptr
	    contextEstablished = true;			    // Set flag
	    }
	lStatus = loginContext->LogIn(principalName,
				      keytableName,
				      ConvertProtectionLevel(protLevel),
				      flags);
	if (lStatus != LoginContext::SUCCESS)
	    {
	    ObjtranReport.Report(ErrorLevels::ErrorMsg, "Failed To LogIn\n");
	    returnValue = LGI_FAILURE;
	    }
	else
	    {
	    returnValue = LGI_SUCCESS;
	    LoggedOn = true;		// We are in like flint
	    }
	}
    return(returnValue);
}

//
//++
//  Functional Description
//
//      This function implements the KeytableLoginContext Class member
//	function: LogIn using externally defined environmental symbols
//	OBJTRAN_PRINCIPAL and OBJTRAN_KEYTABLE.
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  ProtectionLevel  By Value	This is the protection level to be set
//  unsigned32	     By Value	Set to AUTO_MANAGE or AUTO_REFRESH
//
//  Return Value
//
//      LoginStatus -- enum defined in login.h
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

LoginStatus Obj_Login_Keytable_Default(ProtectionLevel protLevel,
				       unsigned32 flags)

/**************************************************************************/

{


     //
     //	Local Data
     //

    static  Boolean	  contextEstablished = false;	    // If true, than context allocated
    static  KeytableLoginContext * loginContext;	    // Ptr to context block
    LoginStatus		  returnValue = LGI_NONE;	    // Assume NULL return status
    LoginContext::LoginStatus	lStatus;		    // ObjDce login return status

    //
    // Procedure: Obj_Login_Keytable_Default
    //

    ObjtranFirstInit();
    if (LoggedOn == true)				    // Check if already logged in
	{
	returnValue = LGI_ALREADY_LOGGED_IN;		    // Yes. Set status
	}
    if (returnValue == LGI_NONE)			    // If we should attempt to perform login
	{
	if (contextEstablished == false)		    // Check if context block allocated
	    {
	    //
	    //	Have to establish a context
	    //
	    loginContext = new KeytableLoginContext();	    // No. Create an instance
	    c_api_contextPtr = loginContext;			    // Set module-wide context ptr
	    contextEstablished = true;			    // Set flag
	    }
	//
	// Try to Log In
	//
	lStatus = loginContext->LogIn(ConvertProtectionLevel(protLevel), flags);
	if (lStatus != LoginContext::SUCCESS)
	    {
	    ObjtranReport.Report(ErrorLevels::ErrorMsg, "Failed To LogIn\n");
	    returnValue = LGI_FAILURE;
	    }
	else
	    {
	    returnValue = LGI_SUCCESS;
	    LoggedOn = true;		// We are in like flint
	    }
	}
    return (returnValue);
}

//
//++
//  Functional Description
//
//      This function implements the PasswordLoginContext Class member
//	function: LogIn
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	     ---------------------------------------------
//
//  char *	      By Reference  The DCE Principal Name
//  ProtectionLevel   By Value	    This is the protection level to be set
//  unsigned32	      By Value	    Set to AUTO_MANAGE or AUTO_REFRESH
//
//  Return Value
//
//      LoginStatus -- enum defined in login.h
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N **************/

LoginStatus Obj_Login_Password(const char *principalName,
			       ProtectionLevel protLevel,
			       unsigned32 flags)

/**************************************************************************/

{


     //
     //	Local Data
     //

    static  Boolean	  contextEstablished = false;	    // If true, than context allocated
    static  PasswordLoginContext  *loginContext;	    // Ptr to context block
    LoginStatus		  returnValue = LGI_NONE;	    // Assume NULL return status
    LoginContext::LoginStatus	lStatus;		    // ObjDce login return status

    //
    // Procedure: Obj_Login_Password
    //

    ObjtranFirstInit();
    if (LoggedOn == true)				    // Check if already logged in
	{
	returnValue = LGI_ALREADY_LOGGED_IN;		    // Yes. Set status
	}
    if (returnValue == LGI_NONE)			    // If we should attempt to perform login
	{
	if (contextEstablished == false)		    // Check if context block allocated
	    {
	    //
	    //	Have to establish a context
	    //
	    loginContext = new PasswordLoginContext();	    // No. Create an instance
	    c_api_contextPtr = loginContext;			    // Set module-wide context ptr
	    contextEstablished = true;			    // Set flag
	    }
	//
	// Try to Log In
	//
	lStatus = loginContext->LogIn(principalName, ConvertProtectionLevel(protLevel),flags);
	if (lStatus != LoginContext::SUCCESS)
	    {
	    ObjtranReport.Report(ErrorLevels::ErrorMsg, "Failed To LogIn\n");
	    returnValue = LGI_FAILURE;
	    }
	else
	    {
	    returnValue = LGI_SUCCESS;
	    LoggedOn = true;		// We are in like flint
	    }
	}
    return (returnValue);
}

//
//++
//  Functional Description
//
//      This function implements the PasswordLoginContext Class member
//	function: LogIn
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	     ---------------------------------------------
//
//  ProtectionLevel   By Value	    This is the protection level to be set
//  unsigned32	      By Value	    Set to AUTO_MANAGE or AUTO_REFRESH
//
//  Return Value
//
//      LoginStatus -- enum defined in login.h
//--
//

/**************** F U N C T I O N    D E S C R I P T I O N *****************/

LoginStatus Obj_Login_Password_Default(ProtectionLevel protLevel,
				       unsigned32 flags)

/**************************************************************************/

{

     //
     //	Local Data
     //

    static  Boolean	  contextEstablished = false;	    // If true, than context allocated
    static  PasswordLoginContext * loginContext;	    // Ptr to context block
    LoginStatus		  returnValue = LGI_NONE;	    // Assume NULL return status
    LoginContext::LoginStatus	lStatus;		    // ObjDce login return status

    //
    // Procedure: Obj_Login_Password_Default
    //

    ObjtranFirstInit();
    if (LoggedOn == true)				    // Check if already logged in
	{
	returnValue = LGI_ALREADY_LOGGED_IN;		    // Yes. Set status
	}
    if (returnValue == LGI_NONE)			    // If we should attempt to perform login
	{
	if (contextEstablished == false)		    // Check if context block allocated
	    {
	    //
	    //	Have to establish a context
	    //
	    loginContext = new PasswordLoginContext();	    // No. Create an instance
	    c_api_contextPtr = loginContext;			    // Set module-wide context ptr
	    contextEstablished = true;			    // Set flag
	    }
	//
	// Try to Log In
	//
	lStatus = loginContext->LogIn(ConvertProtectionLevel(protLevel), flags);
	if (lStatus != LoginContext::SUCCESS)
	    {
	    ObjtranReport.Report(ErrorLevels::ErrorMsg, "Failed To LogIn\n");
	    returnValue = LGI_FAILURE;
	    }
	else
	    {
	    returnValue = LGI_SUCCESS;
	    LoggedOn = true;		// We are in like flint
	    }
	}
    return (returnValue);
}

//
//++
//  Functional Description
//
//      This function implements the Logout feature. Although not available from
//	the OBJ libraries, this function makes available the login context
//	destructor
//
//  Formal Parameters
//
//	None
//
//  Return Value
//
//      LoginStatus -- enum defined in login.h
//--
//

/**************** F U N C T I O N    D E S C R I P T I O N *****************/

LoginStatus Obj_LogOut()

/**************************************************************************/

{

    //
    // Local Data
    //

    LoginStatus	returnValue = LGI_SUCCESS;	    // Assume successful return status

    //
    // Procedure: Obj_LogOut
    //

    if (c_api_contextPtr)
	{
	ObjtranReport.Report(ErrorLevels::TraceMsg, "Deleting Login Context Block\n");
	delete c_api_contextPtr;
	c_api_contextPtr = (LoginContext*)0;
	}
    else
	{
	returnValue = LGI_NOT_LOGGED_IN;	    // User not logged in or no context
						    // established.
	}
    return (returnValue);
}
//
//++
//  Functional Description
//
//      This function implements the RefreshPromptForPassword  member
//	function in the PasswordLoginContext class
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  char *	     By Reference  This is the address of a buffer that contains the principal
//  ProtectionLevel  By Value	   This is the protection level to be set
//  unsigned32	     By Value	   Set to AUTO_MANAGE or AUTO_REFRESH
//  char *	     By Reference  This is the address of the buffer that contains the prompt or 0
//  fnc Ptr	     By Reference  Address of a callback routine that gets executed when the
//				   RefreshPromptForPassword member function gets called. 
//
//  Return Value
//
//      LoginStatus -- enum defined in login.h
//      
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

LoginStatus  Obj_Login_Password_SetRefresh(const char *principalName,
					   ProtectionLevel protLevel,
					   unsigned32 flags,
					   const char *promptBuffer,
					   void(*fn)(char *, char *))

/***************************************************************************/

{

     //
     //	Local Data
     //

    static  Boolean	  contextEstablished = false;	    // If true, than context allocated
    static  PswLoginCtxUserFnc * loginContext;		    // Ptr to context block
    LoginStatus		  returnValue = LGI_NONE;	    // Assume NULL return status
    LoginContext::LoginStatus	lStatus;		    // ObjDce login return status

    //
    // Procedure: Obj_Login_Password_SetRefresh
    //

    ObjtranFirstInit();
    if (LoggedOn == true)				    // Check if already logged in
	{
	returnValue = LGI_ALREADY_LOGGED_IN;		    // Yes. Set status
	}
    if (returnValue == LGI_NONE)			    // If we should attempt to perform login
	{
	if (contextEstablished == false)		    // Check if context block allocated
	    {
	    //
	    //	Have to establish a context
	    //
	    loginContext = new PswLoginCtxUserFnc(fn, promptBuffer); // No. Create an instance
	    c_api_contextPtr = loginContext;			    // Set module-wide context ptr
	    contextEstablished = true;			    // Set flag
	    }
	//
	// Try to Log In
	//
	lStatus = loginContext->LogIn(principalName,
				      ConvertProtectionLevel(protLevel),
				      flags);
	if (lStatus != LoginContext::SUCCESS)
	    {
	    ObjtranReport.Report(ErrorLevels::ErrorMsg, "Failed To LogIn\n");
	    returnValue = LGI_FAILURE;
	    }
	else
	    {
	    returnValue = LGI_SUCCESS;
	    LoggedOn = true;		// We are in like flint
	    }
	}
    return (returnValue);
}
//
//++
//  Functional Description
//
//      This function implements the Login member function in the PasswordLoginContext class.
//	This version uses the externally defined environmental symbols OBJTRAN_PRINCIPAL.
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  ProtectionLevel  By Value	   This is the protection level to be set
//  unsigned32	     By Value	   Set to AUTO_MANAGE or AUTO_REFRESH
//  char *	     By Reference  This is the address of the buffer that contains the prompt or 0
//  fnc Ptr	     By Reference  Address of a callback routine that gets executed when the
//				   RefreshPromptForPassword member function gets called. 
//
//  Return Value
//
//      LoginStatus -- enum defined in login.h

//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

LoginStatus  Obj_Login_Password_SetRefreshDef(ProtectionLevel protLevel,
					      unsigned32 flags,
					      const char *promptBuffer,
					      void(*fn)(char *, char *))

/***************************************************************************/

{

     //
     //	Local Data
     //

    static  Boolean	  contextEstablished = false;	    // If true, than context allocated
    static  PswLoginCtxUserFnc * loginContext;		    // Ptr to context block
    LoginStatus		  returnValue = LGI_NONE;	    // Assume NULL return status
    LoginContext::LoginStatus	lStatus;		    // ObjDce login return status

    //
    // Procedure: Obj_Login_Password_SetRefresh
    //

    ObjtranFirstInit();

    if (LoggedOn == true)				    // Check if already logged in
	{
	returnValue = LGI_ALREADY_LOGGED_IN;		    // Yes. Set status
	}
    if (returnValue == LGI_NONE)			    // If we should attempt to perform login
	{
	if (contextEstablished == false)		    // Check if context block allocated
	    {
	    //
	    //	Have to establish a context
	    //
	    loginContext = new PswLoginCtxUserFnc(fn, promptBuffer); // No. Create an instance
	    c_api_contextPtr = loginContext;			    // Set module-wide context ptr
	    contextEstablished = true;			    // Set flag
	    }
	//
	// Try to Log In
	//
	lStatus = loginContext->LogIn(ConvertProtectionLevel(protLevel), flags);
	if (lStatus != LoginContext::SUCCESS)
	    {
	    ObjtranReport.Report(ErrorLevels::ErrorMsg, "Failed To LogIn\n");
	    returnValue = LGI_FAILURE;
	    }
	else
	    {
	    returnValue = LGI_SUCCESS;
	    LoggedOn = true;		// We are in like flint
	    }
	}
    return (returnValue);
}
//
//++
//  Functional Description
//
//      This function implements the RefreshPromptForPassword  member
//	function in the PasswordLoginContext class
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------
//
//  char *	     By Reference  This is the address of the buffer will hold the password
//
//  Return Value
//
//      None
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

void PswLoginCtxUserFnc::RefreshPromptForPassword(char * passwordBuffer)

/***************************************************************************/

{

    //
    // Procedure: RefreshPromptForPassword
    //

    ObjtranReport.VReport(ErrorLevels::TraceMsg,
			  "About to call user defined password refresh routine. Prompt: %s\n",
			  prompt);
    (*UserRefreshRoutine)(passwordBuffer, prompt);
}
//
//++
//  Functional Description
//
//      This function implements the RefreshPromptForPassword  member
//	function in the PasswordLoginContext class
//
//  Formal Parameters
//
//       Type	      Mechanism			Description
// ----------------   ---------	   ---------------------------------------------------
//
//  void (*fn)()     By Reference  This is the address of the user psw refresh routine
//  char *	     By Reference  This is the address of the buffer that contains
//					    the prompt or 0
//
//  Return Value
//
//      None
//--
//

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

PswLoginCtxUserFnc::PswLoginCtxUserFnc(void (*fnP) (char *, char *),
				       const char *pmt)

/***************************************************************************/

{

    //
    // Procedure: PswLoginCtxUserFnc Constructor
    //

    prompt = new char [strlen(pmt) + 1];	// Allocate space for the prompt
    strcpy(prompt, pmt);			// Copy it to local storage
    UserRefreshRoutine  = fnP;			// Set address of refresh callback
}

/***************** F U N C T I O N    D E S C R I P T I O N ****************/

PswLoginCtxUserFnc::~PswLoginCtxUserFnc()

{
  if (prompt)
    {
      delete prompt;
    }
}
