/*
 * AbortReason
 *
 *    This class is used to return the reason for a transaction abort.
 * This subclass can contain both a string and integer abort reason, plus
 * the name of the source file, function, and line number at which the
 * transaction was aborted. Transactions get this before aborts and set it 
 * when the Transaction::Abort function is called.
 *
 * abortreason.H,v 1.9 1994/04/07 19:20:14 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef ABORT_REASON_H
#define ABORT_REASON_H

extern "C"
{
#include <pthread.h>
}
#include <objdce/defs.H>
#include <String.h>
#include <objdce/objtranparams.H>
#include <objdce/bdemem.H>
#include <objdce/statics.H>
#include <iostream.h>

/* Constants for packing into a portable void* */

extern const int ReasonLen;	// Amount of text for abort reason
extern const int FileLen;	// Length of file name in same
extern const int FunctionLen;	// Length of function name " "
extern const int ApplNameLen;	// Length of application name " "
extern const int PackedDataLen;	// Length of total packed structure

class _EXPORT AbortReason : public BdeMem

{
    friend ostream& operator << (ostream&, const AbortReason&);
public:
    AbortReason(const String& reason, const int code, const int applNo,
	        const String& file, const String& function, 
	        const String& application);
    AbortReason(const String& reason, const int code,
		const int applNo = 0);	// Use defaults for others
    AbortReason();			    // Just nothing
    AbortReason(const AbortReason&);
    virtual ~AbortReason();
    const String& GetReason() const;	    // Get string abort reason
    void SetReason(const String&);
    const int GetCode() const;		    // Get integer code
    void SetCode(const int);
    const int GetApplNo() const;	    // Get numeric ID of who "owns" code
    void SetApplNo(const int);		
    const String& GetFile() const;	    // Get name of file
    void SetFile(const String&);
    const String& GetFunction() const;	    // Get name of function
    void SetFunction(const String&);
    const String& GetApplication() const;    // Get name of application
    void SetApplication(const String&);
    virtual void Print(String&) const;	    // Put the whole thing in a string
    void Pack(void*) const;		    // Pack into an allocated byte stream
    int PackLength() const;
    void Unpack(void*);			    // Unpack from a byte stream
    void PackString(void*) const;
    int PackStringLength() const;
    void UnpackString(void*);
private:    
    String theReason;
    String theFile;
    String theFunction;
    String theApplication;
    int theCode;
    int applNumId;
};

extern AbortReason DefaultReason;

inline const String& AbortReason::GetReason() const

{ return theReason; }

inline const int AbortReason::GetCode() const

{ return theCode; }

inline const int AbortReason::GetApplNo() const

{ return applNumId; }

inline const String& AbortReason::GetFile() const

{ return theFile; }

inline const String& AbortReason::GetFunction() const

{ return theFunction; }

inline const String& AbortReason::GetApplication() const

{ return theApplication; }

#endif
