/*
 * aclcache.C
 *
 *    This is where the actual ACL checking takes place. Once a particular
 * permission (granted or denied) is looked up for a specific ACL on a
 * specific object, that permission is cached for future reference. This
 * should work well for most DCE ACL access, and not significantly slow
 * things down in the case where no ACL is ever looked up more than once.
 * 
 * aclcache.C,v 1.16 1994/03/17 18:43:17 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <time.h>
#include <objdce/objtranparams.H>
#include <objdce/aclcache.H>
#include <objdce/dcetime.H>
#include <objdce/error.H>
#include <Symbol.h>
#include <Map.h>

#ifdef DOS
#include <dce/daclf.h>
#else
#include <dce/daclif.h>
#endif

/*
 **** ACLValueMaps ****
 *
 *    There are two Maps associated with the ACL cache. PrintStringMap is
 * a map of the print strings for each ACL manager type we've ever
 * encountered. It's indexed by the manager type UUID of the ACL
 * manager in question. HandleMap is a map of ACL handles by name.
 * For each ACL object we look up, we cache the handle for efficiency.
 * These maps are in a separate class that isn't defined in a header.
 * This causes less template instantition when users compile objdce programs.
 */

class _EXPORT ACLValueMaps

{
  friend class ACLValue;
private:  
  static Map<DCEuuid, ACLPrintStringValue>* PrintStringMap;
  static Map<Symbol, sec_acl_handle_t>* HandleMap;
};

/*
 * Static variables:
 * All are initialized by the InitStatics routine, which depends on them
 * being set to 0 before it is called, which is what the compiler ought
 * to do. myMaps points to an ACLValueMaps object. PrintStringMap and 
 * HandleMap are members of ACLValueMaps. HandleLock is a lock on the map
 * of handles, and MapLock is a read-write lock on the map of print strings.
 * CacheLifetime is the maximum time cache entries can live.
 */

ACLValueMaps* ACLValue::myMaps = (ACLValueMaps*)0;
Map<DCEuuid, ACLPrintStringValue>* ACLValueMaps::PrintStringMap =
		(Map<DCEuuid, ACLPrintStringValue>*)0;
ReadWriteLock* ACLValue::MapLock = (ReadWriteLock*)0;
Map<Symbol, sec_acl_handle_t>* ACLValueMaps::HandleMap =
		(Map<Symbol, sec_acl_handle_t>*)0;
MutexLock* ACLValue::HandleLock = (MutexLock*)0;
time_t ACLValue::CacheLifetime;

/*
 * Constants
 *    DefaultNumStrings is the maximum number of print strings that may
 * be defined in an ACL. Future DCE changes may require a change here,
 * as the maximum number of strings is now less than 32.
 */

static unsigned int DefaultNumStrings = 32;

#ifdef _AIX
#include <Map.c>
#include <set.c>

#pragma define (Map<ACLKey, ACLValue>)
#pragma define (Mapiter<ACLKey, ACLValue>)
#pragma define (Map<Symbol, sec_acl_handle_t>)
#pragma define (Mapiter<Symbol, sec_acl_handle_t>)
#pragma define (Map<DCEuuid, ACLPrintStringValue>)
#pragma define (Mapiter<DCEuuid, ACLPrintStringValue>)
#pragma define (Set<String>)
#endif

/**** ACLPrintStringValue ****
 *    This class holds the print strings for a particular ACL manager type,
 * and is accessed by ACL manager type UUID.
 */

ACLPrintStringValue::ACLPrintStringValue()

{
  strings = (sec_acl_printstring_t*)malloc(
		   sizeof(sec_acl_printstring_t) * DefaultNumStrings);
  lastUpdate = (time_t)0;
  numStrings = 0;
  mgrInfo.printstring[0] = '\0';
  mgrInfo.helpstring[0] = '\0';
  mgrInfo.permissions = 0;
}

ACLPrintStringValue::ACLPrintStringValue(const ACLPrintStringValue& val)

{
  strings = (sec_acl_printstring_t*)malloc(
		   sizeof(sec_acl_printstring_t) * DefaultNumStrings);
  this->operator = (val);
}

ACLPrintStringValue::~ACLPrintStringValue()

{
  if (strings != (sec_acl_printstring_t*)0)
    {
      free((char*)strings);
    }
}

ACLPrintStringValue& ACLPrintStringValue::operator = 
			(const ACLPrintStringValue& val)
{
  unsigned long inc;

  numStrings = val.numStrings;
  lastUpdate = val.lastUpdate;
  strcpy((char*)(mgrInfo.printstring),
	 (char*)(val.mgrInfo.printstring));
  strcpy((char*)(mgrInfo.helpstring),
	 (char*)(val.mgrInfo.helpstring));
  mgrInfo.permissions = val.mgrInfo.permissions;
	 
  for (inc = 0; inc < numStrings; inc++)
	{
	  strcpy((char*)(strings[inc].printstring), 
		 (char*)(val.strings[inc].printstring));
	  strcpy((char*)(strings[inc].helpstring), 
		 (char*)(val.strings[inc].helpstring));
	  strings[inc].permissions = val.strings[inc].permissions;
	}
  for (inc = numStrings; inc < DefaultNumStrings; inc++)
	{
	  strings[inc].printstring[0] = '\0';
	  strings[inc].helpstring[0] = '\0';
	  strings[inc].permissions = 0;
	}
  mgrUUID = val.mgrUUID;
  return *this;
}

/*
 * Update
 *    Get a new set of print strings from the ACL manager itself.
 * This must be called before this object is of any use. Usually, it's
 * only called when the manager type in question is first used, and never
 * called again.
 */

Boolean ACLPrintStringValue::Update(sec_acl_handle_t handle,
					DCEuuid& uuid)
{
  unsigned long status;
  uuid_t nextUUID;
  boolean32 tokenize;
  unsigned long realTotal;
  String uuidStr;

  if (numStrings != 0)
	return true;
  else
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Getting PrintString from ACL Manager\n");
      mgrUUID = uuid;
      sec_acl_get_printstring(handle, uuid, DefaultNumStrings,
			      &nextUUID, &mgrInfo, &tokenize,
			      &realTotal, &numStrings, strings, &status);
      if (status != 0)
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Failed on sec_acl_get_printstring\n");
	  CheckStatus(status, false, ErrorLevels::WarnMsg, 1526);
	  numStrings = 0;
	  lastUpdate = 0;
	  return false;
	}
      else
	{
	  if (ObjtranReport.Active(ErrorLevels::TraceMsg))
		{
		  String uuidStr;
		  uuid.GetUUID(uuidStr);
		  ObjtranReport.VReport(ErrorLevels::TraceMsg,
					"Got print strings for ACL object %s\n",
					(const char*)uuidStr);
		}
	  lastUpdate = time(NULL);
	  return true;
	}
    }
}

/*
 * GetPerms
 *    These two functions take a permission (or a list of permissions)
 * and translate it into a bit mask by searching through the list of
 * printstrings stored by this object. This is potentially a slow process,
 * but it's the only way I can think of right now. Both functions return
 * a bit mask that can be passed directly to one of the sec_acl_test_access
 * functions.
 */

sec_acl_permset_t ACLPrintStringValue::GetPerms(const String& perm)

{
  int inc;
  sec_acl_permset_t permMask = 0;
  const char* permString = perm;

  for (inc = 0; (inc < numStrings) && (permMask == 0); inc++)
	{
	  if (!strcmp(permString, (char*)(strings[inc].printstring)))
	permMask |= strings[inc].permissions;
	}
  return permMask;
}

sec_acl_permset_t ACLPrintStringValue::GetPerms(const List<String>& perms)

{
  String perm;
  Const_listiter<String> iter(perms);
  sec_acl_permset_t permMask = 0;
  
  while (iter.next(perm))
	permMask |= GetPerms(perm);
  return permMask;
}

/**** ACLKey ****
 *    A very simple object intended to be used as a key in a map of ACLs.
 * The key consists of the name of the ACL object in question and the
 * PAC that is trying to look it up.
 */

ACLKey::ACLKey()

{
}

/*
 * Constructor
 *
 *    This is the "main" constructor. It simply stores the object name and
 * the UUID from the PAC for future reference. If the PAC is NULL, then
 * the NULL UUID is used. In this code, the NULL UUID represents 
 * the permissions for the calling process, rather than for another process.
 */

ACLKey::ACLKey(const String& name, const sec_id_pac_t* pac)

{
  objName = name;
  if (pac != (sec_id_pac_t*)0)
    {
      principal = &(pac->principal.uuid);
    }
  else
    {
      principal.CreateNilUUID();
    }
}

ACLKey::ACLKey(const ACLKey& key)

{
  this->operator = (key);
}

ACLKey::~ACLKey()

{
}

ACLKey& ACLKey::operator = (const ACLKey& key)

{
  objName = key.objName;
  principal = key.principal;
  return *this;
}

/*
 * operators ==, <, and >
 *
 *    These functions compare both the principal UUID (from the PAC)
 * and the object name. The < and > operators should work sufficiently well
 * to produce a sorted list of ACLKeys if necessary.
 */

int ACLKey::operator == (const ACLKey& key) const

{
  return ((principal == key.principal) && (objName == key.objName));
}

int ACLKey::operator < (const ACLKey& key) const

{
  if (objName < key.objName)
	return 1;
  else if (objName > key.objName)
	return 0;
  else
	{   
	  if (principal < key.principal)
	return 1;
	  else
	return 0;
	}
}

int ACLKey::operator > (const ACLKey& key) const

{
  if (objName > key.objName)
	return 1;
  else if (objName < key.objName)
	return 0;
  else
	{   
	  if (principal > key.principal)
	return 1;
	  else
	return 0;
	}
}

/**** ACLValue ****
 *    The value in the map of cached ACL information. The realPerms bit mask
 * stores permissions we've looked up. The cacheMask mask defines which of those
 * bits are valid. When it comes time to look up an ACL, we check the realPerms
 * mask if the permissions we're looking for are in the cacheMask. Otherwise,
 * we actually talk to the ACL manager, then store the result.
 */

ACLValue::ACLValue()

{
  realPerms = 0;
  cacheMask = 0;
  oldestEntry = 0;
  strings = (ACLPrintStringValue*)0;
  mgrHandle = sec_acl_default_handle;
  InitStatics();
}

ACLValue::ACLValue(const ACLValue& val)

{
  InitStatics();
  this->operator = (val);
}

/*
 * InitStatics
 *
 *    The ACLValue class depends on a number of static variables, which 
 * must be initialized. Rather than rely on C++ to do this for us, we
 * call this function when creating an ACLValue object, which creates
 * all the necessary objects using "new".
 */

void ACLValue::InitStatics()

{
  if (myMaps == (ACLValueMaps*)0)
	{
	  myMaps = new ACLValueMaps();
	}
  if (myMaps->PrintStringMap == (Map<DCEuuid, ACLPrintStringValue>*)0)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Creating PrintStringMap\n");
      myMaps->PrintStringMap = new Map<DCEuuid, ACLPrintStringValue>();
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Creating ReadWriteLock\n");
      MapLock = new ReadWriteLock();
      myMaps->HandleMap = new Map<Symbol, 
		sec_acl_handle_t>((sec_acl_handle_t)sec_acl_default_handle);
      HandleLock = new MutexLock();
      char* lifeStr = getenv("OBJTRAN_ACL_CACHE_LIFE");
      if (lifeStr == (char*)0)
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg,
			       "Using default ACL cache lifetime\n");
	  CacheLifetime = DefaultACLCacheLife * 60;
	}
      else
	{
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"OBJTRAN_ACL_CACHE_LIFE = %s\n", lifeStr);
	  CacheLifetime = (time_t)(atoi(lifeStr) * 60);
	}
      if (ObjtranReport.Active(ErrorLevels::TraceMsg))
	{
	struct  tm *tmP;

#if HAVE_GMTIME_R
	struct tm timeBuf;
	gmtime_r(&CacheLifetime, &timeBuf);
	tmP = &timeBuf;
#else
	tmP = gmtime(&CacheLifetime);
#endif
	ObjtranReport.VReport(ErrorLevels::TraceMsg,
				  "Setting CacheLifeTime to %02.02d:%02.02d:%02.02d\n",
				  tmP->tm_hour, tmP->tm_min, tmP->tm_sec);
	}
	}  
}

ACLValue::~ACLValue()

{
  latch.Lock(); 
  // Do this so the object isn't deleted while it's in use
  latch.Unlock();
}

ACLValue& ACLValue::operator = (const ACLValue& val)

{
  latch.Lock();
  realPerms = val.realPerms;
  cacheMask = val.cacheMask;
  oldestEntry = val.oldestEntry;
  strings = val.strings;
  mgrHandle = val.mgrHandle;
  latch.Unlock();
  return *this;
}

/*
 * IsExpired
 *
 *    Return whether this ACLValue object is expired. oldestEntry is set
 * when we got the very first permission for this particular ACL -- an
 * ACL being a combination of an object and a PAC. An ACLValue is expired
 * when the first permission looked up is older than the default cache lifetime 
 * for the entire ACL cache (there's only one cache per process). 
 */

Boolean ACLValue::IsExpired() const 

{
  time_t now = time(NULL);
  return (now > (oldestEntry + CacheLifetime));
}

/*
 * NeedsUpdate
 *
 *    Check if this ACLValue needs to be updated before checking the
 * permissions specified in perms. perms represents which permissions
 * we want to check. Here, we check cacheMask, which contains a 1 bit
 * for every permission that's been checked before. If a particular 
 * permission isn't in cacheMask, it must be updated. In addition, the
 * whole ACLValue must be updated if it has expired. IsExpired tells us
 * if it's expired. Update the cached values by setting cacheMask to 0
 * to indicate nothing is cached, and by setting realPerms because no
 * access is granted any more.
 */

Boolean ACLValue::NeedsUpdate(sec_acl_permset_t perms) 

{
  if ((perms & cacheMask) != perms)
	return true;
  else
	{
	  if (IsExpired())
	{
	  cacheMask = 0;
	  realPerms = 0;
	  return true;
	}
	  else
	return false;
	}
}

/*
 * IsAuthorized
 *
 *    Actually check the permission. First, get the handle to this ACL.
 * If it doesn't exist, don't bother with the rest.
 * Next, get the print strings for this object if necessary, then match them 
 * to the desired permissions (as a String or a List<String>) by using
 * GetPerms to turn them into a bitmap. Check if those permissions are in 
 * the cache using NeedsUpdate, and call Update to update them if they're not. 
 *    Once that's been done,
 * simply check if the desired permission bits are set in realPerms,
 * which contains the cached permission bits. The second form of this
 * function is similar to the first, but uses a list of String objects
 * for its permissions. It simply calls GetPerms using the list rather
 * than using a single value.
 */

Boolean ACLValue::IsAuthorized(const String& object, sec_id_pac_t* pac,
			       const String& perm) 

{
  sec_acl_permset_t wantedPerms;
  Boolean retVal;
  
  latch.Lock();
  if (mgrHandle == sec_acl_default_handle)
    {
      mgrHandle = GetHandle(object);
    }
  if (mgrHandle == sec_acl_default_handle)
    {
      retVal =  false;
    }
  else
    {
      if (strings == (ACLPrintStringValue*)0)
	{
	  GetStrings();
	}
      if (strings == (ACLPrintStringValue*)0)
	{
	  retVal = false;
	}
      else
	{
	  MapLock->ReadLock();
	  wantedPerms = strings->GetPerms(perm);
	  MapLock->Unlock();
	  if (wantedPerms == 0)
	    {
	      retVal = false;
	    }
	  else
	    {
	      if (NeedsUpdate(wantedPerms))
		{
		  Update(pac, wantedPerms);	
		  ObjtranReport.VReport(ErrorLevels::TraceMsg,
					"Updated %s permissions on ACL %s\n",
					(const char*)perm,
					(const char*)object);
		}
	      if ((wantedPerms & realPerms) == wantedPerms)
		{
		  retVal = true;
		  ObjtranReport.Report(ErrorLevels::TraceMsg, 
				       "Is Authorized -- Wanted Perms == Real Perms\n");
		}
	      else
		{
		  ObjtranReport.Report(ErrorLevels::TraceMsg, 
				       "Is Not Authorized for permission requested\n");
		  retVal = false;
		}
	    }
	}
    }
  latch.Unlock();
  return retVal;
}

Boolean ACLValue::IsAuthorized(const String& object, sec_id_pac_t* pac,
				const List<String>& perms) 

{
  sec_acl_permset_t wantedPerms;
  Boolean retVal;
  
  latch.Lock();
  if (mgrHandle == sec_acl_default_handle)
    {
      mgrHandle = GetHandle(object);
    }
  if (mgrHandle == sec_acl_default_handle)
    {
      retVal =  false;
    }
  else
    {
      if (strings == (ACLPrintStringValue*)0)
	{
	  GetStrings();
	}
      if (strings == (ACLPrintStringValue*)0)
	{
	  retVal = false;
	}
      else
	{
	  MapLock->ReadLock();
	  wantedPerms = strings->GetPerms(perms);
	  MapLock->Unlock();
	  if (wantedPerms == 0)
	    {
	      retVal = false;
	    }
	  else
	    {
	      if (NeedsUpdate(wantedPerms))
		{
		  Update(pac, wantedPerms);
		  ObjtranReport.VReport(ErrorLevels::TraceMsg,
					"Updated permissions on ACL %s\n",
					(const char*)object);
		}
	      if ((wantedPerms & realPerms) == wantedPerms)
		{
		  retVal = true;
		  ObjtranReport.Report(ErrorLevels::TraceMsg, 
				       "Is Authorized -- Wanted Perms == Real Perms\n");
		}
	      else
		{
		  ObjtranReport.Report(ErrorLevels::TraceMsg, 
				       "Is Not Authorized for permission requested\n");
		  retVal = false;
		}
	    }
	}
    }
  latch.Unlock();
  return retVal;
}

/*
 * GetHandle
 *
 *    Get a handle to the desired ACL object. First, check the
 * HandleMap to see if there is one already. If so, everything is done.
 * Otherwise, get a new handle using sec_acl_bind and put it in the map.
 * Since handle lookups are expensive, we never delete them. This
 * may prove to be a silly idea at some point. sec_acl_default_handle
 * is returned if no handle could be found.
 */

sec_acl_handle_t ACLValue::GetHandle(const String& object) 

{
  sec_acl_handle_t gotHandle;
  unsigned long status;
  
  HandleLock->Lock();
  gotHandle = (*(myMaps->HandleMap))[object];
  HandleLock->Unlock();
  if (gotHandle != sec_acl_default_handle)
	return gotHandle;
  else
    {
      sec_acl_bind((unsigned char*)((const char*)object), false,
		   &gotHandle, &status);
      if ((status == error_status_ok) && 
	  (gotHandle != sec_acl_default_handle))
	{
	  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
				"Got ACL binding to %s\n", (const char*)object);
	  HandleLock->Lock();
	  (*(myMaps->HandleMap))[object] = gotHandle;
	  HandleLock->Unlock();
	  return gotHandle;
	}
	  else
	{
	  if ((status == sec_acl_object_not_found) ||
	      (status == sec_acl_no_acl_found))
	    ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 
				     1549, (const char*)object);
	  else
		CheckStatus(status, false, ErrorLevels::WarnMsg, 1525);
	  return sec_acl_default_handle;
	}
    }
}

/*
 * GetStrings
 *
 *    This is a wrapper around the map of ACLPrintStringValue objects.
 * It gets the manager type UUID for the manager in question, then
 * calls Update on the ACLPrintStringValue object for that type
 * to fill it with the proper value. Remember that the Map class
 * inserts a new object when you do a lookup with a value that 
 * isn't already there, so the ACLPrintStringValue object doesn't
 * appear to be created in this function, but it is.
 */

void ACLValue::GetStrings() 

{
  unsigned long status;
  uuid_t mgrTypes[1];
  DCEuuid mgrUUID;
  unsigned long realNumTypes;
  unsigned long numTypes;
  
  if (mgrHandle != sec_acl_default_handle)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Calling sec_acl_get_manager_types\n");
      sec_acl_get_manager_types(mgrHandle, sec_acl_type_object, 1,
				&numTypes, &realNumTypes,
				mgrTypes, &status);
      if ((status != 0) && (numTypes > 0))
	{
	  CheckStatus(status, false, ErrorLevels::WarnMsg, 1524);
	  strings = (ACLPrintStringValue*)0;
	}
      else
	{
	  mgrUUID = &(mgrTypes[0]);
	  MapLock->WriteLock();
	  strings = &((*(myMaps->PrintStringMap))[mgrUUID]);
	  strings->Update(mgrHandle, mgrUUID);
	  MapLock->Unlock();
	}
	}
}

/*
 * Update
 *
 *    Other functions have already determined that the permissions on this
 * ACL need updating, so update them. If the pac is NULL, use 
 * sec_acl_test_access to test the permissions using the PAC of the calling
 * process. Otherwise, use sec_acl_test_access_on_behalf to try them
 * using the PAC of another process.
 */

void ACLValue::Update(sec_id_pac_t* pac, sec_acl_permset_t perms) 

{
  if (strings != (ACLPrintStringValue*)0)
    {
      DCEuuid mgrUUID = strings->GetUUID();
      unsigned long status;
      boolean32 granted;
      
      if (cacheMask == 0)
	{
	  oldestEntry = time(NULL);
	}
      if (pac == (sec_id_pac_t*)0)
	{
	  granted = sec_acl_test_access(mgrHandle, mgrUUID,
					perms, &status);
	}
      else
	{
	  granted = sec_acl_test_access_on_behalf(mgrHandle, mgrUUID, pac,
						  perms, &status);
	}
      if (status != 0)
	CheckStatus(status, false, ErrorLevels::WarnMsg, 1527);
	  else 
	{
	  cacheMask |= perms;
	  if (granted)
		realPerms |= perms;
	}
    }
}
