/*
 * aclcache.H
 *
 *    This class does all the work necessary when testing ACLs. There
 * should be one per program for efficiency's sake. It maintains three data
 * structures:
 *    * A map keyed by manager type UUID contains the print strings
 *      for its ACLs.
 *    * A map keyed by the principal UUID and the ACL object name (together)
 *      contains the ACL permission set for that principal on that object.
 *      This makes lookups fast for ACLs that have been looked up recently.
 *
 * aclcache.H,v 1.13 1994/04/07 19:20:17 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef ACL_CACHE_H
#define ACL_CACHE_H

extern "C"
{
#include <pthread.h>
#include <dce/rgynbase.h>
}

#include <objdce/defs.H>

#ifdef DOS
#include <dce/seclgn.h>
#include <\borlandc\include\time.h>
#else
extern "C"
{
#include <dce/sec_login.h> //pc-dce different
}
#include <sys/time.h> 
#endif

extern "C"
{
#include <dce/keymgmt.h>
#include <dce/aclbase.h>
}
#include <String.h>
#ifndef DOS
#include <Path.h>
#endif
#include <List.h>
#include <objdce/uuid.H>
#include <objdce/mutex.H>
#include <objdce/readwrite.H>

/*
 * ACLPrintStringValue is stored for every ACL manager type that's
 * been used. It holds the printstrings for the manager in question.
 * they are updated automatically when GetPerms is called.
 */

class _EXPORT ACLValueMaps;

class _EXPORT ACLPrintStringValue

{
public:
  ACLPrintStringValue();
  ACLPrintStringValue(const ACLPrintStringValue&);
  ~ACLPrintStringValue();
  ACLPrintStringValue& operator = (const ACLPrintStringValue&);

  sec_acl_permset_t GetPerms(const String&);
  sec_acl_permset_t GetPerms(const List<String>&);

  const DCEuuid& GetUUID() const;
  Boolean Update(sec_acl_handle_t, DCEuuid&);
private:
  DCEuuid mgrUUID;          // Our manager type UUID
  sec_acl_printstring_t* strings;   // All the printstrings
  unsigned long numStrings;     // Number of those used
  time_t lastUpdate;            // Last time they were updated
  sec_acl_printstring_t mgrInfo;    // Info on this manager
};

inline const DCEuuid& ACLPrintStringValue::GetUUID() const

{ return mgrUUID; }

/*
 * ACLKey is the key in the map of cached ACLs. It consists of the name
 * of the ACL to test, and the UUID of the principal to test access for.
 */

class _EXPORT ACLKey

{
public:
  ACLKey();
  ACLKey(const String& objName, const sec_id_pac_t* principal);
  ACLKey(const ACLKey&);
  ~ACLKey();
  ACLKey& operator = (const ACLKey&);
  int operator == (const ACLKey&) const;
  int operator < (const ACLKey&) const;
  int operator > (const ACLKey&) const;
  const String GetObject() const;
  const DCEuuid& GetPrincipal() const;
private:
  String objName;   
	// used to be:
	//  Symbol objName;
  DCEuuid principal;
};

inline const String ACLKey::GetObject() const

{ return objName; } 
	  // used to be:
	  //  { return objName.the_string(); }

inline const DCEuuid& ACLKey::GetPrincipal() const

{ return principal; }

/*
 * ACLValue is the set of permissions we have for a particular key.
 * The permissions are updated automatically when IsAuthorized is called,
 * and are only recieved from the ACL manager if they're too old or if
 * they were never looked up in the first place.
 */

class _EXPORT ACLValue

{
public:
  ACLValue();
  ACLValue(const ACLValue&);
  ~ACLValue();
  ACLValue& operator = (const ACLValue&);

  Boolean IsAuthorized(const String& object, sec_id_pac_t* pac,
		       const String& perms);
  Boolean IsAuthorized(const String& object, sec_id_pac_t* pac,
		       const List<String>& perms);
  Boolean IsExpired() const;
private:
  void GetStrings();
  Boolean NeedsUpdate(sec_acl_permset_t);
  void Update(sec_id_pac_t* pac, sec_acl_permset_t perms);
  sec_acl_handle_t GetHandle(const String& object);
  void InitStatics();

  sec_acl_permset_t realPerms;		// The permset we know about
  sec_acl_permset_t cacheMask;		// perms we've actually checked
  time_t oldestEntry;			// When we got the first entry
  MutexLock latch;			// Lock on this object
  ACLPrintStringValue* strings;		// Pointer to printstrings
  sec_acl_handle_t mgrHandle;		// Handle to the ACL manager

  static time_t CacheLifetime;		// Time for cache entries to live
  static ReadWriteLock* MapLock;	// Lock on the map
  static MutexLock* HandleLock;		// Lock on handles
  static ACLValueMaps*	myMaps;
};    

#endif
