/*
 * aclClient.C
 *
 * aclclient.C,v 1.15 1994/04/07 19:20:20 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


extern "C"
{
#include <pthread.h>
}
#include <stdlib.h>
#include <objdce/statics.H>
#include <objdce/error.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/aclclient.H>
#include <objdce/thread.H>

/* Statics (really global) */

Map<ACLKey, ACLValue>* ACLclient::CachedACLMap = (Map<ACLKey, ACLValue>*)0;
MutexLock* ACLclient::ACLMapLatch = (MutexLock*)0;
ACLCleanupThread* ACLclient::CleaningThread;

/*
 * Important constant:
 */

static const long DefaultCleanupSleepSecs = 300;

/*
 **** ACLCleanupThread ****
 *    This thread wakes up periodically and cleans up old entries in 
 * the ACL cache.
 */

class _EXPORT ACLCleanupThread : public ThreadObject

{
public:
  ACLCleanupThread();
  virtual ~ACLCleanupThread();
  virtual void* Run();
};

ACLCleanupThread::ACLCleanupThread()

{
}

ACLCleanupThread::~ACLCleanupThread()

{
}

/*
 * ACLCleanupThreadCleanupHandler
 *    Print out a message when this thread is terminated.
 */

static void ACLCleanupThreadCleanupHandler(void*)

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "The ACL cleanup thread was terminated\n");
}

/*
 * ACLCleanupThread::Run
 *    In a loop, sleep for the default time (five minutes), then delete all
 * the expired ACLs from the map. Each ACLValue knows how old it is, so this
 * is pretty straightforward. 
 */

void* ACLCleanupThread::Run()

{
  Boolean forever = true;
  Map_pair<ACLKey, ACLValue>* curItem;

  pthread_cleanup_push(ACLCleanupThreadCleanupHandler, (void*)0);
  SetKillable(true);
  while (forever)
	{
	  SleepMe(DefaultCleanupSleepSecs, 0);
	  ACLclient::ACLMapLatch->Lock();
	  ObjtranReport.Report(ErrorLevels::TraceMsg, "Cleaning up expired ACLs...\n");
	  Mapiter<ACLKey, ACLValue> cleanupIter(*ACLclient::CachedACLMap);
	  while ((curItem = cleanupIter.next()) != (Map_pair<ACLKey, ACLValue>*)0)
	{
	  if (curItem->value.IsExpired())
		{
		  cleanupIter.remove();
		  if (ObjtranReport.Active(ErrorLevels::TraceMsg))
		{
		  String tmpName = curItem->key.GetObject();
		  ObjtranReport.VReport(ErrorLevels::TraceMsg,
					"Cleaning up a cached ACL for %s\n",
					(const char*)(tmpName));
		}
		}
	}
	  ACLclient::ACLMapLatch->Unlock();
	}
  pthread_cleanup_pop(true);
  return (void*)0;
}

/*
 **** ACLclient ****
 *    Here we implement the rest of the ACL client stuff. However, the 
 * ACLKey and ACLValue classes do the real work. 
 *    Here, we also start up the thread that will periodically delete old
 * ACL entries.
 */

ACLclient::ACLclient()

{
  ObjtranFirstInit();
  if (CachedACLMap == (Map<ACLKey, ACLValue>*)0)
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, "Creating the ACL cache\n");
	  CachedACLMap = new Map<ACLKey, ACLValue>();
	  ACLMapLatch = new MutexLock();
	  CleaningThread = new ACLCleanupThread();
	  AsyncThread* startCleanup = new AsyncThread(*CleaningThread);
	  delete startCleanup;
	}
}

ACLclient::~ACLclient()

{
  /* Don't delete cached stuff because other copies need it */
}

Boolean ACLclient::IsAuthorized(const String& object, sec_id_pac_t* pac, 
				const String& perm)

{
  Boolean isAuth;
  ACLKey key(object, pac);

  ACLMapLatch->Lock();
  ACLValue& val = (*CachedACLMap)[key];
  ACLMapLatch->Unlock();
  isAuth = val.IsAuthorized(object, pac, perm);
  return isAuth;
}

Boolean ACLclient::IsAuthorized(const String& object, sec_id_pac_t* pac, 
				const List<String>& perms)

{
  Boolean isAuth;
  ACLKey key(object, pac);

  ACLMapLatch->Lock();
  ACLValue& val = (*CachedACLMap)[key];
  ACLMapLatch->Unlock();
  isAuth = val.IsAuthorized(object, pac, perms);
  return isAuth;
}
