/*
 * Binding:
 *
 *    This class represents a DCE RPC binding handle. It provides
 * much of the functionality of the DCE RPC API in a class. Clients of
 * DCE RPC servers must create an instance of this class for each server
 * to which they want to talk, then use the object as a parameter to RPC calls.
 *    When used with the DCE name service, a new Binding object refers to
 * the first compatible binding it finds. The binding will match the interface
 * id, name, and (optional) object UUID and protocol. This class tests 
 * every binding it finds to see if the server is actually reachable. 
 * If "IsValid" returns "true", it found a binding to a reachable server. 
 *    If the name given to this class is a DCE group or profile, it will randomly
 * select a compatible binding. The RebindNewHost method can be used to 
 * iterate through the available bindings. Optionally, the BindingGroup and
 * BindingProfile classes can be used to view the list of object names in
 * the group or profile.
 *
 * binding.H,v 1.12 1994/04/07 19:20:28 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/



#ifndef BINDING_H
#define BINDING_H

extern "C"
{
#include <pthread.h>
#include <dce/rpc.h>
}
#include <String.h>
#include <stdio.h>
#include <objdce/defs.H>
#include <objdce/mutex.H>
#include <objdce/condition.H>
#include <objdce/login.H>
#include <objdce/bindingDefs.H>
#ifndef DOS
#include <Path.h>
#endif


/* Values for communications timeout setting */

class _EXPORT InterfaceStats;
class _EXPORT LoginContext;
class _EXPORT DCEuuid;

class _EXPORT Binding 

{
    friend ostream& operator << (ostream&, Binding&);
  public:
    enum CommTimeout { CT_MIN, CT_DEFAULT, CT_MAX, CT_INFINITE };
    enum ErrorType  {ET_SUCCESS, ET_AUTHENTICATION_ERROR, 
		     ET_COMMUNICATION_ERROR, ET_APPLICATION_ERROR,
		     ET_REMOTE_FAULT, ET_OTHER_DCE_ERROR };
    enum Protocol { ALL, IP_TCP, IP_UDP, IP_ALL, 
		    ALL_DG, ALL_CN };

    Binding();
    Binding(const String& nsName, 
	    const rpc_if_handle_t*,
	    LoginContext* login = nil,
	    const char * serverPrincName = "");	// Bind using nameserver
    Binding(const String& nsName, 
	    const rpc_if_handle_t*,
	    LoginContext::ProtectionLevel protLevel,
	    const char * serverPrincName = "");	// Bind using nameserver
    Binding(const String& nsName,
	    const rpc_if_handle_t*,
	    DCEuuid* objectUUID,
	    LoginContext* login = nil,
	    const char * serverPrincName = "",
	    Protocol whatProtocol = ALL);
    Binding(const String& nsName,
	    const rpc_if_handle_t*,
	    DCEuuid* objectUUID,
	    LoginContext::ProtectionLevel protLevel,
	    const char * serverPrincName = "",
	    Protocol whatProtocol = ALL);
    Binding(const String& host, 
	    const long port,
	    LoginContext* login = nil,
	    const rpc_if_handle_t* = (rpc_if_handle_t*)0,
	    const char * serverPrincName = "",
	    Protocol whatProtocol = ALL);	// Bind to well-known endpt.
    Binding(const String& host, 
	    const long port,
	    LoginContext::ProtectionLevel protLevel,
	    const rpc_if_handle_t* = (rpc_if_handle_t*)0,
	    const char * serverPrincName = "",
	    Protocol whatProtocol = ALL);	// Bind to well-known endpt.
    Binding(const String&,
	    LoginContext* login = nil,
	    const char * serverPrincName = "");	// From string binding
    Binding(const String&,
	    LoginContext::ProtectionLevel protLevel,
	    const char * serverPrincName = "");	// From string binding
    Binding(const Binding&);			// Copy constructor

    virtual ~Binding();				    // Free binding handle
    Binding& operator = (const Binding&);
    Boolean operator == (const Binding&) const;	    // Equality		

// Conversion operators
    operator const String&() const;		    // Conversion operators
    operator const rpc_binding_handle_t*();
    operator const rpc_binding_handle_t();

// Utility routines
    Boolean IsValid();			    // Is this binding valid?
    static ErrorType CheckErrorType(error_status_t error);  // Classify an error
    static ErrorType CheckErrorType(error_status_t error,
				    String& outString); // Return in a string
    virtual Boolean RebindIfNecessary(error_status_t error,
				      const int numTries = 1,
				      const int sleepSeconds = 10);
    virtual void RebindSameHost();		    // Reset to another server
    virtual void RebindNewHost();		    // Reset to a new host
    Boolean IsServerListening();		    // Return if server listening
    Boolean StopServerListening();		    // Stop the server
    unsigned32 GetDCEError() const;		    // Get error code 

// Parameter-setting and -getting routines
    virtual CommTimeout GetCommTimeout() const;	    // Get communications timeout
    virtual void SetCommTimeout(const CommTimeout); // Set it
    virtual void GetUUID(DCEuuid&) const;	    // Get object UUID
    virtual void SetUUID(DCEuuid&);	   	    // Set it
    Boolean SetAuthenticationLevel(LoginContext::ProtectionLevel,
				   const char* serverPrin = "");
    void GetStatistics(InterfaceStats&) const;	    // Get statistics on server
    const String& GetNSName() const;		    // Get name in namespace
    void DisableCDSCache();			    // Disable CDS caching

// Useful in other classes
    static Boolean ProtocolOK(const String& protocolName,
			      Protocol desiredProt);  // Does name match?
    static void ProtocolToString(Protocol desiredProt,
				 String& out);
protected:
// Internal routines
    Boolean LoopLookup(const String& nsName, 	    // Do actual lookup
		       const rpc_if_handle_t* ifHandle,
		       DCEuuid* objUUID);
    Boolean ResolveAndPing(const rpc_binding_handle_t& tmpHandle,
			   const rpc_if_handle_t* ifHandle);
    Boolean WaitIfRebind();			    // Wait if rebind in progress
    void FirstInit();				    // Do initial setup
    void Init(const String&, const rpc_if_handle_t*,  // Called by constructors
	      LoginContext::ProtectionLevel, const char*);
    void Init(const String&, const rpc_if_handle_t*,
	      DCEuuid*, LoginContext::ProtectionLevel,
	      const char*);
    void Init(const String&, const long,
	      LoginContext::ProtectionLevel, const rpc_if_handle_t*,
	      const char*);
    void Init(const String&, LoginContext::ProtectionLevel, const char*);
    void CleanUpImport();
    void FreeMainBinding();

    rpc_binding_handle_t rpcRep;		    // The actual handle    
    rpc_if_handle_t svrHandle;			    // Handle to server def.
    rpc_ns_import_handle_t iHandle;		    // Handle of imported bindings
    String strName;				    // A string version
    String namespaceName;			    // Name in the namespace
    Boolean beganImport;			    // If we started an import
    Boolean valid;				    // We got the binding
    Boolean importing;				    // An import is in progress
    DCEuuid* myObject;				    // My object UUID
    LoginContext::ProtectionLevel protLevel;	    // Authentication level
    String serverName;				    // Server principal 
    MutexLock lookupLatch;			    // Locked when looking up
    Condition lookupCondition;			    // To wait for lookup
    Protocol protocol;				    // What protocol to use
    unsigned long longExpiration;		    // Long expiration period
    unsigned32 lastDCEError;
};

inline Binding::operator const String&() const

{ return strName; }

inline Binding::operator const rpc_binding_handle_t*() 

{ WaitIfRebind(); return &rpcRep; }
    
inline Binding::operator const rpc_binding_handle_t()

{ WaitIfRebind(); return rpcRep; }

inline const String& Binding::GetNSName() const

{ return namespaceName; }

inline unsigned32 Binding::GetDCEError() const

{ return lastDCEError; }

#endif

