/*
 * bindingaux.C
 *
 *    This file contains all the functions of the Binding class that aren't
 * in binding.C. In particular, the "internal" functions that users don't
 * usually call are in here. A few of the "less commonly used " functions 
 * are here as well.
 *
 * bindingaux.C,v 1.8 1994/03/17 18:43:25 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/binding.H>
#include <stdio.h>
#include <stdlib.h>
#include <objdce/dcecheckstatus.H>
#include <objdce/error.H>
#include <objdce/uuid.H>
#include <objdce/login.H>
#include <dce/rpcbase.h>
#include <objdce/stats.H>
#include <objdce/defs.H>               

/*
 * We initialize the CDS expiration age to eight hours. Here's eight
 * hours in seconds.
 */

static const unsigned long EightHours = 28800;

/*
 * Init
 *
 *    This first of the "Init" functions binds to a server using the name
 * service.
 */

void Binding::Init(const String& nsName, 
		   const rpc_if_handle_t* ifHandle,
		   LoginContext::ProtectionLevel pLvl,
		   const char * serverPrincName)
{
  valid = false;
  
  FirstInit();
  protLevel = pLvl;
  lookupLatch.Lock();
  importing = true;
  lookupLatch.Unlock();

  ObjtranReport.Report(ErrorLevels::TraceMsg, "Performing Binding::Init\n");
  namespaceName = nsName;
  serverName = serverPrincName;
  svrHandle = *ifHandle;
  if (LoopLookup(nsName, ifHandle, (DCEuuid*)0))
    {
      valid = true;
      if (protLevel != LoginContext::NONE)
	{
	ObjtranReport.Report(ErrorLevels::TraceMsg, 
			     "Calling SetAuthenticationLevel\n");
	valid = SetAuthenticationLevel(protLevel, serverName);
	}
      if (valid)
	{
	  lookupLatch.Lock();
	  importing = false;
	  lookupCondition.Broadcast();
	  lookupLatch.Unlock();
	}
    }
  else
    {
      ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1130, 
			       (const char*)namespaceName);
      lookupLatch.Lock();
      valid = false;
      importing = false;
      lookupCondition.Broadcast();
      lookupLatch.Unlock();
    }
}

/*
 * Init
 *
 *    This is the same as the last Init function, but it also takes an
 * object UUID.
 */

void Binding::Init(const String& nsName, const rpc_if_handle_t* ifHandle, 
		   DCEuuid* objUUID, LoginContext::ProtectionLevel pLvl,
		   const char * serverPrincName)
{
  valid = false;
  beganImport = false;
  
  FirstInit();
  lookupLatch.Lock();
  importing = true;
  lookupLatch.Unlock();

  serverName = serverPrincName;
  namespaceName = nsName;
  protLevel = pLvl;
  svrHandle = *ifHandle;
  myObject = objUUID;
  if (LoopLookup(nsName, ifHandle, objUUID))
    {
      valid = true;
      if (protLevel != LoginContext::NONE)
	{
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"Calling SetAuthenticationLevel -- ProtLevel: %d, ServerName: %s\n",
				protLevel, (const char*)serverName);
	  valid = SetAuthenticationLevel(protLevel, serverName);
	}
      if (valid)
	{
	  lookupLatch.Lock();
	  importing = false;
	  lookupCondition.Broadcast();
	  lookupLatch.Unlock();
	}
    }
  else
    {
      ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1130, (const char*)namespaceName);
      lookupLatch.Lock();
      valid = false;
      importing = false;
      lookupCondition.Broadcast();
      lookupLatch.Unlock();
    }
}

/*
 * Init
 *
 *    This version of "Init" takes a host name and port number to create
 * a binding to a well-known endpoint.
 */

void Binding::Init(const String& host, const long port,
		   LoginContext::ProtectionLevel pLvl,
		   const rpc_if_handle_t* ifHandle, 
		   const char * serverPrincName)
{
  char* hostName;
  char* portName;
  unsigned char* tmpBinding;
  error_status_t status = rpc_s_ok;
  String rpcProtocol;
  
  FirstInit();
  if ((host.length() == 0) && (pLvl != LoginContext::NONE))
    {
      // We can't use security with broadcast RPC
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1169);
    }
  else if ((pLvl != LoginContext::NONE) && 
	   (ifHandle == (rpc_if_handle_t*)0))
    {
      // ifHandle can't be NULL if security is in use
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1168);
    }
  else
    {
      if (ifHandle != (rpc_if_handle_t*)0)
	{
	  svrHandle = *ifHandle;
	}
      serverName = serverPrincName;
      protLevel = pLvl;
      if (port == 0)
	{
	  portName = (char*)0;
	}
      else
	{
	  portName = (char*)malloc(sizeof(char) * 32);
#if HAVE_CMA_SPRINTF
	  cma_sprintf(portName, "%li", port);
#else
	  sprintf(portName, "%li", port);
#endif
	  ObjtranReport.VReport(ErrorLevels::TraceMsg, 
				"Setting portName to %s\n", portName);
	}
      if (host == "")
	hostName = (char*)0;
      else
	hostName = (char*)((const char*)host);
      ProtocolToString(protocol, rpcProtocol);
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Calling rpc_string_binding_compose\n");
      rpc_string_binding_compose((unsigned char*)0,
				 (unsigned char*)((const char*)rpcProtocol),
				 (unsigned char*)hostName,  
				 (unsigned char*)portName, 
				 (unsigned char*)0, 
				 &tmpBinding, &status);
      CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "rpc_string_binding_compose",
		  __LINE__, __FILE__);
      strName = (char *) tmpBinding;
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Initial binding: %s\n", tmpBinding);
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Calling rpc_binding_from_string_binding\n");
      rpc_binding_from_string_binding(tmpBinding, &rpcRep, &status);
      CheckStatus(status, true, ErrorLevels::FatalMsg,
		  "rpc_binding_from_string_binding", __LINE__, __FILE__);
      rpc_string_free(&tmpBinding, &status);
      CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "rpc_string_free", __LINE__, __FILE__);
      if (portName != (char*)0)
	{
	  free((char*)portName);
	}
      if ((protLevel != LoginContext::NONE) && (host.length() > 0))
	{
	  if (ResolveAndPing(rpcRep, &svrHandle))
	    {
	      ObjtranReport.VReport(ErrorLevels::TraceMsg,
				    "Calling SetAuthenticationLevel - ProtLevel: %d, Server Name: %s\n",
				    protLevel, (const char*)serverName);
	      valid = SetAuthenticationLevel(protLevel, serverName);
	    }
	}
      else
	{
	  valid = true;
	}
      importing = false;
      lastDCEError = status;
    }
}

/*
 * Init
 *
 *    This last of the "Init" functions takes a string, and can be used to 
 * create a binding from a string binding.
 */

void Binding::Init(const String& strBinding,
		   LoginContext::ProtectionLevel pLvl,
		   const char * serverPrincName)
{
  error_status_t status = rpc_s_ok;
  const char* tmpStrBinding = strBinding;
    
  FirstInit();
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling rpc_binding_from_string_binding\n");
  rpc_binding_from_string_binding((unsigned char*)tmpStrBinding,
				  &rpcRep, &status);
  CheckStatus(status, true, ErrorLevels::FatalMsg,
	      "rpc_binding_from_string_binding", __LINE__, __FILE__);
  strName = strBinding;
  serverName = serverPrincName;
  valid = true;
  protLevel = pLvl;
  if (protLevel != LoginContext::NONE)
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Calling SetAuthenticationLevel - ProtLevel: %d, Server Name: %s\n", 
			    protLevel, (const char*)serverPrincName);
      valid = SetAuthenticationLevel(protLevel, serverName);
    }
  lastDCEError = status;
}

/*
 * FirstInit
 *
 *    This function is used by most constructors to set up default values for 
 * some of the class's data members. All constructors (i.e. Init
 * routines) call this before doing anything else.
 */

void Binding::FirstInit()

{
  valid = false;
  beganImport = false;
  importing = false;
  myObject = (DCEuuid*)0;
  protLevel = LoginContext::NONE;
  longExpiration = EightHours;
  lastDCEError = rpc_s_ok;
}

/*
 * ResolveAndPing
 *
 *    This function resolves the partially-bound endpoint and 
 * pings the server. It returns "true" if the server is registered and
 * listening. It's used by LoopLookup and RebindSameHost.
 */

Boolean Binding::ResolveAndPing(const rpc_binding_handle_t& tmpHandle,
                                const rpc_if_handle_t* ifHandle)

{
  unsigned long rpcErr = rpc_s_ok;
  unsigned char* tmpStrName;
  Boolean retVal = false;

  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "Calling rpc_ep_resolve_binding\n");
  rpc_ep_resolve_binding(tmpHandle, *ifHandle, &rpcErr);
  if (rpcErr == rpc_s_ok)		// If we could resolve
    {
      rpc_binding_to_string_binding(tmpHandle, &tmpStrName, 
				    &rpcErr);
      CheckStatus(rpcErr, true, ErrorLevels::FatalMsg,
		  "rpc_binding_to_string_binding", 
		  __LINE__, __FILE__);
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Pinging server at\n\t%s\n", tmpStrName);
      if (rpc_mgmt_is_server_listening(tmpHandle, &rpcErr))	
	{					// It worked!
	  retVal = true;
	  strName = (char*)tmpStrName;
	}
      else
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg, 
			       "Not Listening\n");
	}
      rpc_string_free(&tmpStrName, &rpcErr);
      CheckStatus(rpcErr, true, ErrorLevels::FatalMsg,
		  "rpc_string_free", __LINE__, __FILE__);
    }
  else if (rpcErr != ept_s_not_registered)	
    {				// A more serious problem
      CheckStatus(rpcErr, false, ErrorLevels::ErrorMsg, 
		  "rpc_ep_resolve_binding", __LINE__, __FILE__);
    }
  else	
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Couldn't resolve binding\n");
    }
  lastDCEError = rpcErr;
  return retVal;
}  


/*
 * LoopLookup
 *
 *    This function does the actual binding lookup. If no binding import
 * has been begun, it imports a vector of bindings from CDS. It then gives
 * one out, after first testing if the binding is registered with the remote
 * rpcd with and checking to see if the remote server is listening. It only
 * returns a binding if all these conditions are true, and loops through the
 * binding vector until it finds one that works. If it finds none, it tries
 * getting the vector again, this time with the CDS clerk cache disabled. 
 *    If this function is called again later, it just gets the next binding
 * from the vector until it reaches the end. Then, it starts the import again.
 *    The caller of this function is responsible for ensuring that multiple
 * threads do not call it at the same time. The "Rebind" methods of the
 * binding class take care of that via a mutex, condition variable, and
 * a Boolean. The conversion operator for rpc_binding_handle_t checks the
 * Boolean and won't return until the server is properly re-bound.
 */

Boolean Binding::LoopLookup(const String& nsName,
			    const rpc_if_handle_t* ifHandle,
			    DCEuuid* objUUID)
{
  unsigned long rpcErr;
  rpc_binding_handle_t tmpHandle;
  unsigned char* tmpName = (unsigned char*)((const char*)nsName);
  unsigned char* tmpStrName;
  unsigned char* protocolName;
  uuid_t* objectUUID;
  Boolean retVal = true;
  Boolean moreBindings;
  Boolean foundIt = false;
  Boolean freeTmpBinding = false;
  Boolean didItOnceAlready = false;
  Boolean protocolRight = false;
  unsigned long expAge = longExpiration;

  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1160,
			   (const char*)nsName);
  if (objUUID != (DCEuuid*)0)
    objectUUID = *objUUID;
  else
    objectUUID = (uuid_t*)0;
  while (!foundIt && retVal)		// Until we've succeeded or failed
    {
      if (!beganImport)			// Start import if necessary
	{
	  if (!didItOnceAlready)
	    didItOnceAlready = true;
	  else
	    expAge = 0;		    // If this is 2nd import, disable cache
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"Starting import of %s\n", tmpName);
	  rpc_ns_binding_import_begin(rpc_c_ns_syntax_dce, tmpName,
				      *ifHandle, objectUUID, &iHandle, &rpcErr);
	  retVal = CheckStatus(rpcErr, false, ErrorLevels::ErrorMsg,
			       "rpc_ns_binding_import_begin",
			       __LINE__, __FILE__);
	  if (retVal)
	    {
	      beganImport = true;
	    }
	}
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "Setting name service expiration age to %li\n", 
			    expAge);
      rpc_ns_mgmt_handle_set_exp_age(iHandle, expAge, &rpcErr);
      retVal = CheckStatus(rpcErr, false, ErrorLevels::ErrorMsg,
			   "rpc_ns_mgmt_handle_set_exp_age",
			   __LINE__, __FILE__);

      moreBindings = true;
      while (moreBindings && retVal)	// While there are bindings left
	{
	  if (freeTmpBinding)
	    {
	      rpc_binding_free(&tmpHandle, &rpcErr);
	      CheckStatus(rpcErr, false, ErrorLevels::WarnMsg,
			  "rpc_binding_free", __LINE__, __FILE__);
	      freeTmpBinding = false;
	    }
	  rpc_ns_binding_import_next(iHandle, &tmpHandle, &rpcErr);
	  if ((rpcErr == rpc_s_no_more_bindings) ||
	      (rpcErr == rpc_s_entry_not_found))
	    moreBindings = false;
	  else
	    {
	      if (!CheckStatus(rpcErr, false, ErrorLevels::ErrorMsg,
			       "rpc_ns_binding_import_next", __LINE__,
			       __FILE__))
		{
		  retVal = false;
		  moreBindings = false;
		}
	      else
		freeTmpBinding = true;
	    }
	  if (moreBindings)		// If there's really a binding
	    {
	      rpc_binding_to_string_binding(tmpHandle, &tmpStrName, &rpcErr);
	      CheckStatus(rpcErr, true, ErrorLevels::FatalMsg, 
			  "rpc_binding_to_string_binding",
			  __LINE__, __FILE__);
	      ObjtranReport.VReport(ErrorLevels::TraceMsg, 
				    "Resolving binding\n\t%s\n", tmpStrName);
	      rpc_string_binding_parse(tmpStrName, 
				       (unsigned char**)0,
				       &protocolName,
				       (unsigned char**)0,
				       (unsigned char**)0,
				       (unsigned char**)0,
				       &rpcErr);
	      CheckStatus(rpcErr, true, ErrorLevels::FatalMsg,
			  "rpc_string_binding_parse", 
			  __LINE__, __FILE__);
	      protocolRight = ProtocolOK((char*)protocolName, protocol);
	      if (!protocolRight)
		{
		  ObjtranReport.VReport(ErrorLevels::TraceMsg,
					"%s doesn't match desired protocols\n",
					(const char*)protocolName);
		}
	      rpc_string_free(&protocolName, &rpcErr);
	      CheckStatus(rpcErr, true, ErrorLevels::FatalMsg, 
			  "rpc_string_free", __LINE__, __FILE__);
	      rpc_string_free(&tmpStrName, &rpcErr);
	      CheckStatus(rpcErr, true, ErrorLevels::FatalMsg, 
			  "rpc_string_free", __LINE__, __FILE__);
	      if (protocolRight)
		{
		  if (ResolveAndPing(tmpHandle, ifHandle))
		    {
		      foundIt = true;
		      moreBindings = false;
		    }
		}
	    }
	}
      if (!foundIt)
	{
	  CleanUpImport();
	  if (expAge == 0)
	    retVal = false;		// Second phase failed, so exit
	}
    }
  if (foundIt)
    {
      rpc_binding_copy(tmpHandle, &rpcRep, &rpcErr);
      CheckStatus(rpcErr, true, ErrorLevels::FatalMsg,
		  "rpc_binding_copy", __LINE__, __FILE__);
      ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1161,
			       (const char*)nsName, (const char*)strName);
    }
  else
    {
      ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 1165);
    }
  if (freeTmpBinding)
    {
      rpc_binding_free(&tmpHandle, &rpcErr);
      CheckStatus(rpcErr, false, ErrorLevels::WarnMsg,
		  "rpc_binding_free", __LINE__, __FILE__);
    }
  lastDCEError = rpcErr;
  return foundIt;
}

/*
 * CleanUpImport
 *
 *    If rpc_ns_import_begin has been called, clean up the import context.
 * This is used in LoopLookup, the = operator and the destructor.
 */

void Binding::CleanUpImport()

{
  error_status_t rpcErr;

  if (beganImport)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			    "Ending binding import\n");
      rpc_ns_binding_import_done(&iHandle, &rpcErr);
      CheckStatus(rpcErr, false, ErrorLevels::ErrorMsg, 
		  "rpc_ns_binding_import_done",
		  __LINE__, __FILE__);
      beganImport = false;
    }
}

/*
 * FreeMainBinding
 *
 *    Free the main binding handle. This is used in the destructor, the
 * copy constructor, and elsewhere.
 */

void Binding::FreeMainBinding()

{
  error_status_t status;

  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"Calling rpc_binding_free\n");
  rpc_binding_free(&rpcRep, &status);
  CheckStatus(status, true, ErrorLevels::FatalMsg, 
	      "rpc_binding_free", __LINE__, __FILE__);
}

/*
 * GetCommTimeout
 *
 *    Return the communications timeout for RPCs issued using this binding.
 */

Binding::CommTimeout Binding::GetCommTimeout() const

{
    if (!valid)
	return CT_DEFAULT;
    else
    {
	CommTimeout retVal;
	error_status_t status = rpc_s_ok;
	unsigned32 value;
	rpc_mgmt_inq_com_timeout(rpcRep, &value, &status);
	CheckStatus(status, false, ErrorLevels::WarnMsg, 
		    "rpc_mgmt_inq_com_timeout", __LINE__, __FILE__);
	switch (value)
	{
	case rpc_c_binding_min_timeout:
	    retVal = CT_MIN;
	    break;
	case rpc_c_binding_default_timeout:
	    retVal = CT_DEFAULT;
	    break;
	case rpc_c_binding_max_timeout:
	    retVal = CT_MAX;
	    break;
	case rpc_c_binding_infinite_timeout:
	    retVal = CT_INFINITE;
	    break;
	default:
	    retVal = CT_DEFAULT;
	    break;
	}
	return retVal;
    }
}

/*
 * SetCommTimeout
 *
 *    Set the communications timeout for this binding.
 */

void Binding::SetCommTimeout(const CommTimeout timeout)

{
    if (valid)
    {
      error_status_t status = rpc_s_ok;
	long value;
	switch (timeout)
	{
	case CT_MIN:
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "Setting Comm Timeout to: MIN\n");
	    value = rpc_c_binding_min_timeout;
	    break;
	case CT_DEFAULT:
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "Setting Comm timeout to: DEFAULT\n");
	    value = rpc_c_binding_default_timeout;
	    break;
	case CT_MAX:
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "Setting Comm timeout to: MAX\n");
	    value = rpc_c_binding_max_timeout;
	    break;
	case CT_INFINITE:
	    ObjtranReport.Report(ErrorLevels::TraceMsg, 
				 "Setting Comm timeout to: INFINITE\n");
	    value = rpc_c_binding_infinite_timeout;
	    break;
	}
	ObjtranReport.Report(ErrorLevels::TraceMsg, 
			     "Calling rpc_mgmt_set_com_timeout\n");
	rpc_mgmt_set_com_timeout(rpcRep, value, &status);
	CheckStatus(status, false, ErrorLevels::WarnMsg, 
		    "rpc_mgmt_set_com_timeout", __LINE__, __FILE__);
      lastDCEError = status;
    }
}

/*
 * GetUUID
 *
 *    Return the object UUID associated with this binding, if it has
 * been properly bound.
 */

void Binding::GetUUID(DCEuuid& uuid) const

{

  ObjtranReport.Report(ErrorLevels::TraceMsg, "In GetUUID\n");
  if (valid)
    {
      uuid_t* tmpUuid = nil;
      error_status_t status = rpc_s_ok;

      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Calling rpc_binding_inq_object\n");
      rpc_binding_inq_object(rpcRep, tmpUuid, &status);
      CheckStatus(status, false, ErrorLevels::WarnMsg, 
		  "rpc_binding_inq_object", __LINE__, __FILE__);
      uuid = tmpUuid;
    }
  else
    {
      DCEuuid nullUUID;
      uuid = nullUUID;
    }
}

/*
 * SetUUID
 *
 *    Set the object UUID we want to contact using this UUID.
 */

void Binding::SetUUID(DCEuuid& uuid)

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, "In SetUUID\n");
  if (valid)
    {
      error_status_t status = rpc_s_ok;

      ObjtranReport.Report(ErrorLevels::TraceMsg, 
			   "Calling rpc_binding_set_object\n");
      rpc_binding_set_object(rpcRep, uuid, &status);
      CheckStatus(status, false, ErrorLevels::WarnMsg, 
		  "rpc_binding_set_object", __LINE__, __FILE__);
      if (myObject != (DCEuuid*)0)
	{
	  *myObject = uuid;
	}
      else
	{
	  myObject = new DCEuuid(uuid);
	}
      lastDCEError = status;
    }
}

/*
 * GetStatistics
 *
 *    Get statistics on the RPC runtime use of RPCs and packets.
 */

void Binding::GetStatistics(InterfaceStats& stats) const

{
  if (valid)
    {
      rpc_stats_vector_t* tmpStats = nil;
      error_status_t status = rpc_s_ok;

      rpc_mgmt_inq_stats(rpcRep, &tmpStats, &status);
      CheckStatus(status, false, ErrorLevels::WarnMsg, 
		  "rpc_mgmt_inq_stats", __LINE__, __FILE__);
      stats = tmpStats;
      rpc_mgmt_stats_vector_free(&tmpStats, &status);
      CheckStatus(status, true, ErrorLevels::FatalMsg, 
		  "Failed on rpc_mgmt_stats_vector_free");
    }
}

/*
 * DisableCDSCache
 *
 *    When CDS lookups are done the first time around, the name service cache
 * refresh time is set to "longExpiration" in seconds. This time defaults
 * to EightHours. This function sets that value to 0, so no caching will be
 * done.
 */

void Binding::DisableCDSCache()

{
  lookupLatch.Lock();
  longExpiration = 0;
  lookupLatch.Unlock();
  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 1166, 
			   (const char*)namespaceName);
}

