/*
 * ProfileElement :
 *        
 *        This class represents the information stored in each profile element.
 *
 * BindingProfile :
 *        
 *        This object represents a "profile" entry in DCE directory service.
 */

#include <objdce/statics.H>
#include <bindingprof.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/error.H>
#include <iostream.h>

//
// Profile Element Class.
//
ProfileElement::ProfileElement()
{
    ObjtranFirstInit();
}

ProfileElement::ProfileElement(const String& name,
                const rpc_if_id_t& ifId,
                unsigned long prtyVal,
                const char* annoStr)
{
    ObjtranFirstInit();
    memberName  = name;
    annotation = annoStr;
    priority = (unsigned32) prtyVal;
    memberId = ifId;
}

ProfileElement::ProfileElement(const String& name,
                const rpc_if_handle_t* ifHandle,
                unsigned long prtyVal,
                const char* annoStr)
{
    ObjtranFirstInit();
    unsigned32 status;

    memberName  = name;
    annotation = annoStr;
    priority = (unsigned32) prtyVal;

    ObjtranReport.Report(ErrorLevels::TraceMsg,"Calling rpc_if_inq_id::ProfileElement\n");
    rpc_if_inq_id(*ifHandle,
            &memberId,
            &status);
    CheckStatus(status,true,ErrorLevels::FatalMsg,
        "rpc_if_inq_id::ProfileElement",
        __LINE__,__FILE__);
}

ProfileElement::ProfileElement(const ProfileElement& elem)
{
    ObjtranFirstInit();
    this->operator = (elem);
}

ProfileElement::~ProfileElement()
{
}

ProfileElement& ProfileElement::operator = (const ProfileElement& elem)
{
    memberName = elem.memberName;
    annotation = elem.annotation;
    priority   = elem.priority  ;
    memberId   = elem.memberId  ;

    return *this;
}

Boolean ProfileElement::operator == (const ProfileElement& elem) const
{
    unsigned long err;
    uuid_t uuid1,uuid2;

    if ((memberName == elem.memberName) &&
        (memberId.vers_major == (elem.memberId).vers_major) &&
        (memberId.vers_minor == (elem.memberId).vers_minor))
      {
	uuid1 = memberId.uuid;
	uuid2 = elem.memberId.uuid;
	return (uuid_compare(&uuid1, &uuid2, &err));
      }
    else
      {
        return ( 1 == 0 );
      }
}


////////////////////////////////////////////////////////////////////////
//
// Binding Profile Class.
//
////////////////////////////////////////////////////////////////////////
BindingProfile::BindingProfile(const String& name)
{
	ObjtranFirstInit();
    profileName = name;
}

BindingProfile::BindingProfile(const BindingProfile& profile)
{
	ObjtranFirstInit();
    this->operator = (profile);
}

BindingProfile::~BindingProfile()
{
}

BindingProfile& BindingProfile::operator = (const BindingProfile& profile)
{
	ObjtranFirstInit();
    profileName = profile.profileName;
    return *this;
}

Boolean BindingProfile::operator == (const BindingProfile& profile) const
{
    return (profileName == profile.profileName);
}

Boolean BindingProfile::Add(const ProfileElement& newElem)
{
    unsigned long err;
    unsigned char* pName = (unsigned char*)((const char*)profileName);
    unsigned char* mName = (unsigned char*)((const char*)(newElem.GetMemberName()));
    unsigned char* annoStr = (unsigned char*)((const char*)newElem.GetAnnotation());
    rpc_if_id_t ifId = newElem.GetMemberId();

    ObjtranReport.VReport(ErrorLevels::TraceMsg,
                          "Adding %s to binding profile %s\n",
                          mName, pName);
    rpc_ns_profile_elt_add(rpc_c_ns_syntax_dce, pName,
                           &ifId,
                           rpc_c_ns_syntax_dce, mName,
                           newElem.GetPriority(),annoStr,&err);
    return (CheckStatus(err, false, ErrorLevels::ErrorMsg, 
            "Failed on rpc_ns_profile_elt_add"));
}

Boolean BindingProfile::AddDefault(const String& memberName,
                                   const char* annoStr)
{
    unsigned long err;
    unsigned char* pName = (unsigned char*)((const char*)profileName);
    unsigned char* mName = (unsigned char*)((const char*)memberName);

    ObjtranReport.VReport(ErrorLevels::TraceMsg,
                          "Adding %s to binding profile %s\n",
                          mName, pName);
    rpc_ns_profile_elt_add(rpc_c_ns_syntax_dce, pName,
                           (rpc_if_id_t*)0,
                           rpc_c_ns_syntax_dce, mName,
                           0,(unsigned char*)annoStr,&err);
    return (CheckStatus(err, false, ErrorLevels::ErrorMsg, 
            "Failed on rpc_ns_profile_elt_add"));
}

Boolean BindingProfile::Remove(const ProfileElement& deadElem)
{
    unsigned long err;
    unsigned char* pName = (unsigned char*)(const char*)profileName;
    unsigned char* mName = (unsigned char*)(const char*)(deadElem.GetMemberName());
    rpc_if_id_t ifId = deadElem.GetMemberId();

    ObjtranReport.VReport(ErrorLevels::TraceMsg,
                          "Removing dead element %s from binding profile %s\n",
                          mName, pName);
    rpc_ns_profile_elt_remove(rpc_c_ns_syntax_dce, pName,
                              &ifId,
                              rpc_c_ns_syntax_dce, mName, &err);
    return (CheckStatus(err, false, ErrorLevels::ErrorMsg,
            "Failed on rpc_ns_profile_elt_remove"));
}

Boolean BindingProfile::RemoveDefault(const String& memberName)
{
    unsigned long err;
    unsigned char* pName = (unsigned char*)((const char*)profileName);
    unsigned char* mName = (unsigned char*)((const char*)memberName);

    ObjtranReport.VReport(ErrorLevels::TraceMsg,
                          "Removing dead element %s from binding profile %s\n",
                          mName, pName);
    rpc_ns_profile_elt_remove(rpc_c_ns_syntax_dce, pName,
                              (rpc_if_id_t *)0,
                              rpc_c_ns_syntax_dce, mName, &err);
    return (CheckStatus(err, false, ErrorLevels::ErrorMsg,
            "Failed on rpc_ns_profile_elt_remove"));
}

Boolean BindingProfile::DestroyProfile()
{
    unsigned long err;
    unsigned char* pName = (unsigned char*)((const char*)profileName);

    ObjtranReport.VReport(ErrorLevels::TraceMsg, "Destroying binding profile  %s\n", pName);
    rpc_ns_profile_delete(rpc_c_ns_syntax_dce, pName, &err);
    return (CheckStatus(err, false, ErrorLevels::ErrorMsg, 
            "Failed on rpc_ns_profile_delete"));
}

BindingProfile::DefaultStatus BindingProfile::GetDefault(ProfileElement& outElem)
{
    List<ProfileElement> out;
    enum DefaultStatus retVal = DS_SUCCESS;

    if(GetEntries(rpc_c_profile_default_elt,(rpc_if_id_t*)0,
                  VC_IGNORE,(unsigned char*)0,out))
    {
        if (out.length() == 0)
            retVal = DS_NOTEXISTS;
        else
            out.get(outElem);
    }
    else
        retVal = DS_FAILURE;

    return retVal;
}

Boolean BindingProfile::GetAll(List<ProfileElement>& out)
{
    Boolean retVal;

    retVal = GetEntries(rpc_c_profile_all_elts,
                        (rpc_if_id_t*)0,VC_IGNORE,
                          (unsigned char*)0,out);
    return retVal;
}

Boolean BindingProfile::GetByInterface(const ProfileElement& elem,
                                       const VersionCheck VerOption,
                                       List<ProfileElement>& out)
{
    Boolean retVal;
    rpc_if_id_t ifId;
    
    ifId = elem.GetMemberId();
    retVal = GetEntries(rpc_c_profile_match_by_if,
                        &ifId, VerOption,
                        (unsigned char*)0,out);

    return retVal;
}

Boolean BindingProfile::GetByInterface(const rpc_if_id_t& ifId,
                                       const VersionCheck VerOption,
                                       List<ProfileElement>& out)
{
    Boolean retVal;
    retVal = GetEntries(rpc_c_profile_match_by_if,
                        (rpc_if_id_t*)&ifId, VerOption,
                        (unsigned char*)0,out);
    return retVal;
}

Boolean BindingProfile::GetByInterface(const rpc_if_handle_t* ifHandle,
                                       const VersionCheck VerOption,
                                       List<ProfileElement>& out)
{
    unsigned long err;
    Boolean retVal = false;
    rpc_if_id_t ifId;
    
    ObjtranReport.Report(ErrorLevels::TraceMsg,"Calling rpc_if_inq_id::BindingProfile\n");
    rpc_if_inq_id(*ifHandle, &ifId, &err);

    if (CheckStatus(err, false, ErrorLevels::ErrorMsg))
        retVal = GetEntries(rpc_c_profile_match_by_if,
                            &ifId, VerOption,
                            (unsigned char*)0,out);

    return retVal;
}

Boolean BindingProfile::GetByName(const ProfileElement& elem,
                                  List<ProfileElement>& out)
{
    Boolean retVal;
    unsigned char* mName;

    mName = (unsigned char*)(const char*)(elem.GetMemberName());
    retVal = GetEntries(rpc_c_profile_match_by_mbr,
                        (rpc_if_id_t*)0, VC_IGNORE,
                        mName,out);

    return retVal;
}

Boolean BindingProfile::GetByName(const String& memStr,
                                  List<ProfileElement>& out)
{
    Boolean retVal;
    unsigned char* mName;

    mName = (unsigned char*)(const char*)memStr;
    retVal = GetEntries(rpc_c_profile_match_by_mbr,
                        (rpc_if_id_t*)0, VC_IGNORE,
                        mName,out);

    return retVal;
}

Boolean BindingProfile::GetByNameAndIf(const ProfileElement& elem,
                                       const VersionCheck VerOption,
                                       List<ProfileElement>& out)
{
    Boolean retVal;
    unsigned char* mName;
    rpc_if_id_t ifId;
    
    ifId = elem.GetMemberId();
    mName = (unsigned char*)(const char*)(elem.GetMemberName());

    retVal = GetEntries(rpc_c_profile_match_by_both,
                        &ifId, VerOption,
                        mName,out);

    return retVal;
}

Boolean BindingProfile::GetEntries(unsigned long inqType, rpc_if_id_t* ifId,
                                   const VersionCheck VerOption,unsigned char* mName,
                                   List<ProfileElement>& out)
{
    unsigned long err;
    rpc_ns_handle_t nsContext;
    unsigned char* pName = (unsigned char*)((const char*)profileName);
    Boolean retVal = false;

    String tmpStr;
    unsigned char* outName;
    unsigned char* outAnnotation;
    rpc_if_id_t outIfId;
    unsigned long outPriority;

    if (ifId != (rpc_if_id_t*)0)
    {
        if (VerOption == VC_IGNORE)
            ifId->vers_major=ifId->vers_minor=0;

        if (VerOption == VC_MAJOR_ONLY)
            ifId->vers_minor=0;
    }

    ObjtranReport.VReport(ErrorLevels::TraceMsg, 
                          "Getting Entries from binding profile %s\n", pName);
    switch(VerOption)
    {
        case VC_IGNORE :
                rpc_ns_profile_elt_inq_begin(rpc_c_ns_syntax_dce, pName,
                                 inqType,ifId,rpc_c_vers_all,
                                 rpc_c_ns_syntax_dce, mName,
                                 &nsContext, &err);
                break;
        case VC_COMPATIBLE :
                rpc_ns_profile_elt_inq_begin(rpc_c_ns_syntax_dce, pName,
                                 inqType,ifId,rpc_c_vers_compatible,
                                 rpc_c_ns_syntax_dce, mName,
                                 &nsContext, &err);
                break;
        case VC_EXACT :
                rpc_ns_profile_elt_inq_begin(rpc_c_ns_syntax_dce, pName,
                                 inqType,ifId,rpc_c_vers_exact,
                                 rpc_c_ns_syntax_dce, mName,
                                 &nsContext, &err);
                break;
        case VC_MAJOR_ONLY :
                rpc_ns_profile_elt_inq_begin(rpc_c_ns_syntax_dce, pName,
                                 inqType,ifId,rpc_c_vers_major_only,
                                 rpc_c_ns_syntax_dce, mName,
                                 &nsContext, &err);
                break;
        case VC_UPTO :
                rpc_ns_profile_elt_inq_begin(rpc_c_ns_syntax_dce, pName,
                                 inqType,ifId,rpc_c_vers_upto,
                                 rpc_c_ns_syntax_dce, mName,
                                 &nsContext, &err);
                break;
    }

    if (CheckStatus(err, false, ErrorLevels::ErrorMsg))
    {
        rpc_ns_mgmt_handle_set_exp_age(nsContext, 0, &err);
        err = rpc_s_ok;
        while (err == rpc_s_ok)
        {
            rpc_ns_profile_elt_inq_next(nsContext, &outIfId, &outName,
                                        &outPriority,&outAnnotation,&err);
            if (err == rpc_s_ok)
            {
                tmpStr = (char*)outName;
                ProfileElement tmpElem(tmpStr,outIfId,outPriority,(const char*)outAnnotation);
                out.put(tmpElem);
                rpc_string_free(&outName, &err);
                rpc_string_free(&outAnnotation, &err);
              }
        }

        if (err == rpc_s_no_more_elements)
            retVal = true;
        else
            retVal = CheckStatus(err, false, ErrorLevels::ErrorMsg, 
                                 "Failed on rpc_ns_profile_elt_inq_next");
    }

    rpc_ns_profile_elt_inq_done(&nsContext, &err);

    return retVal;
}
