/*
 * dcetime.C
 *
 *    These classes absract the DCE Time Service, letting us do arithmetic
 * and other cool stuff on DCE timestamps.
 *
 * dcetime.C,v 1.9 1994/03/17 18:43:40 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef DOS
#include <objdce/dcetime.H>
#include <objdce/error.H>
#include <math.h>

static const unsigned int TimeStringLen = 256;
static const unsigned int TimeStringChars = TimeStringLen - 1;
static const int NanoOverMilli = 1000000;
static const double NanoDouble = 1000000000.0;

DCETime::DCETime()

{
  timeRep = new utc_t;
}

DCETime::~DCETime()

{
  delete timeRep;
}

void DCETime::BoundTime(const DCETime& timeOne, const DCETime& timeTwo)

{
  utc_boundtime(timeRep, timeOne.timeRep, timeTwo.timeRep);
}

void DCETime::PointTime(DCETime& lo, DCETime& mid, DCETime& hi) const

{
  utc_pointtime(lo.timeRep, mid.timeRep, hi.timeRep, timeRep);
}

void DCETime::SpanTime(const DCETime& timeOne, const DCETime& timeTwo)

{
  utc_spantime(timeRep, timeOne.timeRep, timeTwo.timeRep);
}

/*
 * RelativeTime
 *    This class represents a relative timestamp, and constrains the 
 * UTC operations possible to those that work on relative timestamps.
 */

RelativeTime::RelativeTime()

{
  reltimespec_t timeSpec;
  timespec_t inaccSpec;
  int err;

  timeSpec.tv_sec = 0;
  timeSpec.tv_nsec = 0;
  inaccSpec.tv_sec = 0;
  inaccSpec.tv_nsec = 0;
  err = utc_mkbinreltime(timeRep, &timeSpec, &inaccSpec);
  if (err != 0)
    ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
}

RelativeTime::RelativeTime(const RelativeTime& relTime)

{
  this->operator = (relTime);
}

RelativeTime::RelativeTime(const utc_t* utcTime)

{
  this->operator = (utcTime);
}

RelativeTime::~RelativeTime()

{
}

RelativeTime& RelativeTime::operator = (const RelativeTime& relTime)

{
  reltimespec_t timeSpec;
  timespec_t inaccSpec;
  int err;

  err = utc_binreltime(&timeSpec, &inaccSpec, relTime.timeRep);
  if (err != 0)
    ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
  else
    {
      err = utc_mkbinreltime(timeRep, &timeSpec, &inaccSpec);
      if (err != 0)
	ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
    }
  return *this;
}

RelativeTime& RelativeTime::operator = (const utc_t* utcTime)

{
  reltimespec_t timeSpec;
  timespec_t inaccSpec;
  int err;

  err = utc_binreltime(&timeSpec, &inaccSpec, (utc_t*)utcTime);
  if (err != 0)
    ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
  else
    {
      err = utc_mkbinreltime(timeRep, &timeSpec, &inaccSpec);
      if (err != 0)
	ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
    }
  return *this;
}

int RelativeTime::operator == (const RelativeTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return (compare == utc_equalTo);
}

int RelativeTime::operator != (const RelativeTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return (compare != utc_equalTo);
}

int RelativeTime::operator < (const RelativeTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return (compare == utc_lessThan);
}

int RelativeTime::operator > (const RelativeTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return (compare == utc_greaterThan);
}

int RelativeTime::operator <= (const RelativeTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return ((compare == utc_equalTo) || (compare == utc_lessThan));
}

int RelativeTime::operator >= (const RelativeTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return ((compare == utc_equalTo) || (compare == utc_greaterThan));
}


RelativeTime RelativeTime::operator + (const RelativeTime& relTime) const

{
  utc_t tmpTime;
  utc_addtime(&tmpTime, timeRep, relTime.timeRep);
  return RelativeTime(&tmpTime);
}

RelativeTime& RelativeTime::operator += (const RelativeTime& relTime)

{
  utc_addtime(timeRep, timeRep, relTime.timeRep);
  return *this;
}

RelativeTime RelativeTime::operator - (const RelativeTime& relTime) const

{
  utc_t tmpTime;
  utc_subtime(&tmpTime, timeRep, relTime.timeRep);
  return RelativeTime(&tmpTime);
}

RelativeTime& RelativeTime::operator -= (const RelativeTime& relTime)

{
  utc_subtime(timeRep, timeRep, relTime.timeRep);
  return *this;
}

RelativeTime RelativeTime::operator * (const double factor) const

{
  utc_t tmpTime;
  utc_mulftime(&tmpTime, timeRep, factor);
  return RelativeTime(&tmpTime);
}

RelativeTime& RelativeTime::operator *= (const double factor)

{
  utc_mulftime(timeRep, timeRep, factor);
  return *this;
}

RelativeTime RelativeTime::operator * (const long factor) const

{
  utc_t tmpTime;
  utc_multime(&tmpTime, timeRep, factor);
  return RelativeTime(&tmpTime);
}

RelativeTime& RelativeTime::operator *= (const long factor)

{
  utc_multime(timeRep, timeRep, factor);
  return *this;
}

RelativeTime RelativeTime::operator / (const double factor) const

{
  utc_t tmpTime;
  utc_mulftime(&tmpTime, timeRep, (1.0 / factor));
  return RelativeTime(&tmpTime);
}

RelativeTime& RelativeTime::operator /= (const double factor)

{
  utc_mulftime(timeRep, timeRep, (1.0 / factor));
  return *this;
}

void RelativeTime::SetTime(const tm* timeTm, const long nanos,
			   const tm* inaccTm, const long inaccNanos)
{
  tm tmpTm;

  if (timeTm == (const tm*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "SetTime");
      return;
    }
  if (inaccTm == (const tm*)0)
    {
      memset((char*)&tmpTm, 0, sizeof(tm));
      utc_mkreltime(timeRep, (tm*)timeTm, nanos, &tmpTm, inaccNanos);
    }
  else
    utc_mkreltime(timeRep, (tm*)timeTm, nanos, (tm*)inaccTm, inaccNanos);
}

void RelativeTime::SetTime(const reltimespec_t* relSpec,
			   const timespec_t* relInacc)
{
  timespec_t tmpSpec;

  if (relSpec == (const reltimespec_t*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "SetTime");
      return;
    }
  if (relInacc == (const timespec_t*)0)
    {
      memset((char*)&tmpSpec, 0, sizeof(timespec_t));
      utc_mkbinreltime(timeRep, (reltimespec_t*)relSpec, &tmpSpec);
    }	
  else
    utc_mkbinreltime(timeRep, (reltimespec_t*)relSpec, (timespec_t*)relInacc);
}

void RelativeTime::SetTime(const time_t newTime)

{
  reltimespec_t relSpec;
  timespec_t inaccSpec;

  memset((char*)&inaccSpec, 0, sizeof(timespec_t));
  relSpec.tv_sec = (long)newTime;
  relSpec.tv_nsec = 0;
  utc_mkbinreltime(timeRep, &relSpec, &inaccSpec);
}

void RelativeTime::GetTime(tm* timeTm, long* nanos,
			   tm* inaccTm, long* inaccNanos) const
{
  tm tmpTm;
  long tmpLong;

  if (timeTm == (tm*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "GetTime");
      return;
    }
  if (nanos == (long*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2301, "GetTime");
      return;
    }

  utc_reltime((tm*)timeTm, nanos, &tmpTm, &tmpLong, timeRep);
  if (inaccTm != (tm*)0)
    *inaccTm = tmpTm;
  if (inaccNanos != (long*)0)
    *inaccNanos = tmpLong;
}

void RelativeTime::GetTime(reltimespec_t* relSpec,
			   timespec_t* relInacc) const
{
  timespec_t fakeSpec;

  if (relSpec == (reltimespec_t*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "GetTime");
      return;
    }
  if (relInacc == (timespec_t*)0)
    utc_binreltime(relSpec, &fakeSpec, timeRep);
  else
    utc_binreltime(relSpec, relInacc, timeRep);
}

void RelativeTime::GetTime(time_t* outTime) const

{
  if (outTime == (time_t*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "GetTime");
      return;
    }
  time_t retVal = *this;
  *outTime = retVal;
}

void RelativeTime::GetTime(double* outTime) const

{
  if (outTime == (double*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "GetTime");
      return;
    } 
  double retVal = *this;
  *outTime = retVal;
}

double RelativeTime::CalculateRate(unsigned long numOps) const

{
  double tmpOps = (double)numOps;
  double tmpTime = *this;
  return (tmpOps / tmpTime);
}

RelativeTime::operator double() const

{
  double retVal;	
  double retFrac;
  reltimespec_t relSpec;
  timespec_t relInacc;

  utc_binreltime(&relSpec, &relInacc, timeRep);
  retVal = relSpec.tv_sec;
  retFrac = (double)(relSpec.tv_nsec) / NanoDouble;
  retVal += retFrac;
  return retVal;
}

RelativeTime::operator time_t() const

{
  reltimespec_t relSpec;
  timespec_t relInacc;

  utc_binreltime(&relSpec, &relInacc, timeRep);
  return (time_t)(relSpec.tv_sec);
}

void RelativeTime::GetWholeTime(String& out) const

{
  char tmpString[TimeStringLen];
  utc_ascreltime(tmpString, TimeStringChars, timeRep);
  out = tmpString;
}

void RelativeTime::GetTime(String& out) const

{
  reltimespec_t relSpec;
  timespec_t relInacc;
  long hours;
  long minutes;
  long tmpMinutes;
  long seconds;
  long thousandths;
  char sign;
  char tmpString[TimeStringLen];
  char fracString[4];
  char secString[3];
  char minString[3];
  int err;
  
  err = utc_binreltime(&relSpec, &relInacc, timeRep);
  if (err != 0)
    {
      out = "";
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
    }
  else
    {
      hours = abs((int)(relSpec.tv_sec) / 3600);
      tmpMinutes = abs((int)(relSpec.tv_sec) % 3600);
      minutes = tmpMinutes / 60;
      seconds = tmpMinutes % 60;
      // CAST here is bad. Look in to this.
      thousandths = abs((int)(relSpec.tv_nsec / NanoOverMilli));
      
      if ((relSpec.tv_sec < 0) || (relSpec.tv_nsec < 0))
	sign = '-';
      else
	sign = '+';
      
      if (thousandths < 10)
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(fracString, "00%li", thousandths);
#else
	  sprintf(fracString, "00%li", thousandths);
#endif
	}
      else if (thousandths < 100)
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(fracString, "0%li", thousandths);
#else
	  sprintf(fracString, "0%li", thousandths);
#endif
	}
      else
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(fracString, "%li", thousandths);
#else
	  sprintf(fracString, "%li", thousandths);
#endif
	}
      
      if (seconds < 10)
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(secString, "0%li", seconds);
#else
	  sprintf(secString, "0%li", seconds);
#endif
	}
      else
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(secString, "%li", seconds);
#else
	  sprintf(secString, "%li", seconds);
#endif
	}
      
      if ((hours != 0) && (minutes < 10))
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(minString, "0%li", minutes);
#else
	  sprintf(minString, "0%li", minutes);
#endif
	}
      else
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(minString, "%li", minutes);
#else
	  sprintf(minString, "%li", minutes);
#endif
	}
      
      if (hours != 0)
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(tmpString, "%c%li:%s:%s.%s", sign, hours, 
		      minString, secString, fracString);
#else
	  sprintf(tmpString, "%c%li:%s:%s.%s", sign, hours, 
		  minString, secString, fracString);
#endif
	}
      else
	{
#if HAVE_CMA_SPRINTF
	  cma_sprintf(tmpString, "%c%s:%s.%s", sign, minString,
		      secString, fracString);
#else
	  sprintf(tmpString, "%c%s:%s.%s", sign, minString,
		  secString, fracString);
#endif
	}
      out = tmpString;
    }
}

/*
 * AbsoluteTime
 *    This class represents an absolute timestamp -- i.e. one that
 * represents a single point in time.
 */

AbsoluteTime::AbsoluteTime()

{
  utc_getusertime(timeRep);
  zone = UserZone;
}

AbsoluteTime::AbsoluteTime(const AbsoluteTime& absTime)

{
  this->operator = (absTime);
}

AbsoluteTime::AbsoluteTime(const RelativeTime& relTime)

{
  this->operator = (relTime);
}

AbsoluteTime::AbsoluteTime(const utc_t* utcTime)

{
  this->operator = (utcTime);
}

AbsoluteTime::~AbsoluteTime()

{
}
  
AbsoluteTime& AbsoluteTime::operator = (const AbsoluteTime& absTime)

{
  timespec_t timeSpec;
  timespec_t inaccSpec;
  long tdf;
  int err;

  err = utc_bintime(&timeSpec, &inaccSpec, &tdf, absTime.timeRep);
  if (err != 0)
    ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
  else
    {
      err = utc_mkbintime(timeRep, &timeSpec, &inaccSpec, tdf);
      if (err != 0)
	ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
    }
  return *this;
}

AbsoluteTime& AbsoluteTime::operator = (const RelativeTime& relTime)

{
  const utc_t* tmpTime = relTime;
  utc_abstime(timeRep, (utc_t*)tmpTime);
  return *this;
}

AbsoluteTime& AbsoluteTime::operator = (const utc_t* utcTime)

{
  timespec_t timeSpec;
  timespec_t inaccSpec;
  long tdf;
  int err;

  err = utc_bintime(&timeSpec, &inaccSpec, &tdf, (utc_t*)utcTime);
  if (err != 0)
    ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
  else
    {
      err = utc_mkbintime(timeRep, &timeSpec, &inaccSpec, tdf);
      if (err != 0)
	ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 2302);
    }
  return *this;
}

int AbsoluteTime::operator == (const AbsoluteTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return (compare == utc_equalTo);
}

int AbsoluteTime::operator != (const AbsoluteTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return (compare != utc_equalTo);
}

int AbsoluteTime::operator < (const AbsoluteTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return (compare == utc_lessThan);
}

int AbsoluteTime::operator > (const AbsoluteTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return (compare == utc_greaterThan);
}

int AbsoluteTime::operator <= (const AbsoluteTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return ((compare == utc_equalTo) || (compare == utc_lessThan));
}

int AbsoluteTime::operator >= (const AbsoluteTime& relTime) const

{
  utc_cmptype compare;
  utc_cmpmidtime(&compare, timeRep, relTime.timeRep);
  return ((compare == utc_equalTo) || (compare == utc_greaterThan));
}

AbsoluteTime AbsoluteTime::operator + (const RelativeTime& relTime) const

{
  utc_t tmpTime;
  const utc_t* relTmpTime = relTime;
  utc_addtime(&tmpTime, timeRep, (utc_t*)relTmpTime);
  return AbsoluteTime(&tmpTime);
}

AbsoluteTime& AbsoluteTime::operator += (const RelativeTime& relTime)

{
  const utc_t* relTmpTime = relTime;
  utc_addtime(timeRep, timeRep, (utc_t*)relTmpTime);
  return *this;
}

RelativeTime AbsoluteTime::operator + (const AbsoluteTime& absTime) const

{
  utc_t tmpTime;
  utc_addtime(&tmpTime, timeRep, absTime.timeRep);
  return RelativeTime(&tmpTime);
}

RelativeTime AbsoluteTime::operator - (const AbsoluteTime& absTime) const

{
  utc_t tmpTime;
  utc_subtime(&tmpTime, timeRep, absTime.timeRep);
  return RelativeTime(&tmpTime);
}

AbsoluteTime AbsoluteTime::operator - (const RelativeTime& relTime) const

{
  utc_t tmpTime;
  const utc_t* relTmpTime = relTime;
  utc_subtime(&tmpTime, timeRep, (utc_t*)relTmpTime);
  return AbsoluteTime(&tmpTime);
}

AbsoluteTime& AbsoluteTime::operator -= (const RelativeTime& relTime)

{
  const utc_t* relTmpTime = relTime;
  utc_subtime(timeRep, timeRep, (utc_t*)relTmpTime);
  return *this;
}

void AbsoluteTime::SetZone(WhatZone newZone)

{
  zone = newZone;
}

AbsoluteTime& AbsoluteTime::SetCurrentTime()

{
  utc_getusertime(timeRep);
  return *this;
}

void AbsoluteTime::SetTime(const tm* timeTm, const long nanos,
			   const tm* inaccTm, const long inaccNanos)
{
  tm tmpTm;

  if (timeTm == (const tm*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "SetTime");
      return;
    }
  if (inaccTm == (const tm*)0)
    memset((char*)&tmpTm, 0, sizeof(tm));
  else
    tmpTm = *inaccTm;
  if (zone == UTCZone)
    utc_mkgmtime(timeRep, (tm*)timeTm, nanos, &tmpTm, inaccNanos);
  else
    utc_mklocaltime(timeRep, (tm*)timeTm, nanos, &tmpTm, inaccNanos);
}

void AbsoluteTime::SetTime(const timespec_t* timeSpec,
			   const timespec_t* timeInacc)
{
  timespec_t tmpSpec;

  if (timeSpec == (const timespec_t*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "SetTime");
      return;
    }
  if (timeInacc == (const timespec_t*)0)
    memset((char*)&tmpSpec, 0, sizeof(timespec_t));
  else
    tmpSpec = *timeInacc;
  utc_mkbintime(timeRep, (timespec_t*)timeSpec, &tmpSpec, 0);
}

void AbsoluteTime::SetTime(const time_t newTime)

{
  timespec_t timeSpec;
  timespec_t inaccSpec;

  memset((char*)&inaccSpec, 0, sizeof(timespec_t));
  timeSpec.tv_sec = (long)newTime;
  timeSpec.tv_nsec = 0;
  utc_mkbintime(timeRep, &timeSpec, &inaccSpec, 0);
}

void AbsoluteTime::GetTime(tm* timeTm, long* nanos,
			   tm* inaccTm, long* inaccNanos) const
{
  tm tmpTm;
  long tmpLong;

  if (timeTm == (tm*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "GetTime");
      return;
    }
  if (nanos == (long*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2301, "GetTime");
      return;
    }

  if (zone == UTCZone)
    utc_gmtime(timeTm, nanos, &tmpTm, &tmpLong, timeRep);
  else
    utc_localtime(timeTm, nanos, &tmpTm, &tmpLong, timeRep);
  if (inaccTm != (tm*)0)
    *inaccTm = tmpTm;
  if (inaccNanos != (long*)0)
    *inaccNanos = tmpLong;
}

void AbsoluteTime::GetTime(timespec_t* timeSpec,
			   timespec_t* timeInacc) const
{
  timespec_t fakeSpec;
  long tdf = 0;

  if (timeSpec == (timespec_t*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "GetTime");
      return;
    }
  utc_bintime(timeSpec, &fakeSpec, &tdf, timeRep);
  if (timeInacc != (timespec_t*)0)
    *timeInacc = fakeSpec;
}

void AbsoluteTime::GetTime(time_t* outTime) const

{
  if (outTime == (time_t*)0)
    {
      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2300, "GetTime");
      return;
    }
  time_t retVal = *this;
  *outTime = retVal;
}

AbsoluteTime::operator time_t() const

{
  timespec_t timeSpec;
  timespec_t timeInacc;
  long tdf = 0;

  utc_bintime(&timeSpec, &timeInacc, &tdf, timeRep);
  return (time_t)(timeSpec.tv_sec);
}

void AbsoluteTime::GetWholeTime(String& out) const

{
  char tmpString[TimeStringLen];

  if (zone == UTCZone)
    utc_ascgmtime(tmpString, TimeStringChars, timeRep);
  else
    utc_asclocaltime(tmpString, TimeStringChars, timeRep);
  out = tmpString;
}

void AbsoluteTime::GetTime(String& out) const

{
  char tmpString[TimeStringLen];
  tm timeTm;
  long nanos;
  tm inaccTm;
  long inaccNanos;

  if (zone == UTCZone)
    utc_gmtime(&timeTm, &nanos, &inaccTm, &inaccNanos, timeRep);
  else
    utc_localtime(&timeTm, &nanos, &inaccTm, &inaccNanos, timeRep);
#if HAVE_STRFTIME
  strftime(tmpString, TimeStringChars, "%X", &timeTm);
  out = tmpString;
#else
  GetWholeTime(out);
#endif
}

void AbsoluteTime::GetTimeAndDate(String& out) const

{
  char tmpString[TimeStringLen];
  tm timeTm;
  long nanos;
  tm inaccTm;
  long inaccNanos;

  if (zone == UTCZone)
    utc_gmtime(&timeTm, &nanos, &inaccTm, &inaccNanos, timeRep);
  else
    utc_localtime(&timeTm, &nanos, &inaccTm, &inaccNanos, timeRep);
#if HAVE_STRFTIME
  strftime(tmpString, TimeStringChars, "%x %X", &timeTm);
  out = tmpString;
#else
  GetWholeTime(out);
#endif
}

void AbsoluteTime::GetZone(String& zoneName, Boolean* daylight,
			   long* tdf) const
{
  int isDst;
  long realTdf;
  char zoneString[TimeStringLen];

  if (zone == UserZone)
    utc_localzone(zoneString, TimeStringChars, &realTdf, 
		  &isDst, timeRep);
  else
    utc_gmtzone(zoneString, TimeStringChars, &realTdf, 
		&isDst, timeRep);
  zoneName = zoneString;
  if (daylight != (Boolean*)0)
    *daylight = isDst;
  if (tdf != (long*)0)
    *tdf = realTdf;
}

#endif
  


