/*
 * Login.H
 *
 * login.H,v 1.12 1994/04/07 19:20:55 gjb Exp
 */



    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef LOGIN_H
#define LOGIN_H

extern "C"
{
#include <pthread.h>
#include <dce/rgynbase.h>
#include <dce/keymgmt.h>
#ifdef DOS
#include <dce/seclgn.h>
#else
#include <dce/sec_login.h> //pc-dce different
#endif
#include <dce/rpcbase.h>
}

#include <String.h>
#include <objdce/defs.H>
#include <objdce/aclclient.H>
#include <objdce/transignals.H>

class _EXPORT KeytableRefreshThread;
class _EXPORT KeytableManageThread;
class _EXPORT PasswordRefreshThread;
class _EXPORT AsyncThread;	

/*
 * LoginContext
 *    This is an abstract base class used by the other types of login context
 * classes. Users should never create a LoginContext object.
 */

class _EXPORT LoginContext : public Terminated
{
public:
  enum LoginStatus { SUCCESS, RESET_PASSWORD, FAILURE, UNSET };
  enum ProtectionLevel { DEFAULT, NONE, CONNECT, CALL, PACKET,
			 PACKET_AUTHENTICATION, PACKET_PRIVACY };
  static const unsigned long AUTO_REFRESH;
  static const unsigned long AUTO_MANAGE;
protected:
  LoginContext();
public:
  virtual ~LoginContext();
  unsigned32 GetProtectionLevel() const;
  ProtectionLevel GetLoginProtectionLevel() const;
  unsigned32 GetAuthorizationService() const;
  const String& GetPrincipalName() const;
  sec_login_auth_src_t GetAuthorizationSource() const;
  LoginStatus GetStatus() const;
  sec_login_handle_t GetLoginContext() const;
  LoginStatus CertifyLoginContext(Boolean refreshOnly = false);
  virtual LoginStatus RefreshIdentity();
  Boolean IsValid() const;
  static unsigned32 DceProtectionLevel(ProtectionLevel l);
  virtual void HandleTermination() = 0;
protected:
  void ZeroKey();

  unsigned32 authorizationService;
  unsigned32 protectionLevel;
  ProtectionLevel pLvl;
  sec_login_auth_src_t authorizationSource;
  LoginStatus loginStatus;
  sec_login_handle_t loginContextHandle;
  sec_passwd_rec_t* passwordRec;
  sec_id_pac_t loginPAC;
  String principalName;
  Boolean refreshOneOnly;
  Boolean isValid;
  static int ContextCreated;
};

/*
 * CurrentLoginContext
 *    This is the simplest login context. It gets the context from the
 * environment of its parent process.
 */

class _EXPORT CurrentLoginContext : public LoginContext
{
public:
  CurrentLoginContext();
  virtual ~CurrentLoginContext();
  LoginStatus LogIn(ProtectionLevel protLevel);
  virtual void HandleTermination();
};

/*
 * KeytableLoginContext
 *    This class is used to create a login context from a key table file.
 * Client programs can use this to get a login context without having the
 * user type a password.
 */

class _EXPORT KeytableLoginContext : public LoginContext
{
public:
  KeytableLoginContext();
  virtual ~KeytableLoginContext();
  LoginStatus LogIn(const String& principalName, 
		    const String& keytableName,
		    ProtectionLevel protLevel,
		    const unsigned32 flags = 0);
  LoginStatus LogIn(ProtectionLevel protLevel,
		    const unsigned32 flags = 0);
  virtual LoginStatus RefreshIdentity();
  virtual unsigned32 GetMostRecentKeyVersion();
  const String& GetKeyTable() const;
  virtual void HandleTermination();
private:
  void CreateRefreshThread();
  void CreateManageThread();

  AsyncThread* refreshThread;
  KeytableRefreshThread* tgtRefresh;
  AsyncThread* manageThread;
  KeytableManageThread* tgtManage;
  String keyTable;
};

/*
 * PasswordLoginContext
 *    Using this class, a program may prompt a user for the DCE password
 * rather than use a keytable or rely on the dce_login program. It may
 * only be used for client-side-only RPC programs.
 */

class _EXPORT PasswordLoginContext : public LoginContext

{
public:
  PasswordLoginContext();
  virtual ~PasswordLoginContext();
  LoginStatus LogIn(const String& principalName,
		    ProtectionLevel protLevel,
		    const unsigned32 flags = 0);
  LoginStatus LogIn(ProtectionLevel protLevel,
		    const unsigned32 flags = 0);
  virtual LoginStatus RefreshIdentity();
  virtual void RefreshPromptForPassword(char*);
  virtual void HandleTermination();
protected:
  void CreateRefreshThread();

  AsyncThread* refreshThread;
  PasswordRefreshThread* tgtRefresh;
};

/*
 * ServerLoginContext
 *    This is the only type of login context that may be used by an RPC
 * server. It is based on the KeytableLoginContext class because it uses 
 * a keytable to store its password. Thus, all servers that wish to receive
 * authenticated RPCs must have a keytable.
 */

class _EXPORT ServerLoginContext : public KeytableLoginContext

{
public:
  ServerLoginContext();
  virtual ~ServerLoginContext();
  LoginStatus LogIn(ProtectionLevel protLevel,
		    const unsigned32 flags = 0);
  LoginStatus LogIn(const String& principalName,
		    const String& keytableName,
		    const String& directoryName,
		    ProtectionLevel protLevel,
		    const unsigned32 flags = 0);
  Boolean IsClntAuthorized(rpc_binding_handle_t& handle,
			   const String& ACLkey,
			   const String& ACLperm);
  Boolean IsClntAuthorized(rpc_binding_handle_t& handle,
			   const String& ACLkey,
			   const List<String>& ACLpermList);
  Boolean IsClntAuthorized(rpc_binding_handle_t& handle);
private:
  Boolean GetCheckClntAuthInfo(rpc_binding_handle_t& handle,
			       sec_id_pac_t* clientPac);

  ACLclient* CDSaclClient;  // used to check access to aclDir
  String aclDir;
};

/*
 * PasswordClass
 *    This class is an example of a class that prompts the user
 * to type in a password at the terminal. It is implemented in
 * refreshdefs.C.
 */

class _EXPORT PasswordClass

{
public:
  PasswordClass(const char* pnote = "Enter password: ");
  void GetPassword(char* pwd);
  void ZeroPassword();
  virtual void Prompt(void*);
protected:
  const char* promptNote;
  sec_passwd_str_t pwdbuf;
};

/*
 * Inline functions:
 * 
 * LoginContext:
 */

inline unsigned32 LoginContext::GetProtectionLevel() const

 { return protectionLevel; }

inline LoginContext::ProtectionLevel LoginContext::GetLoginProtectionLevel() const

{ return pLvl; }

inline unsigned32 LoginContext::GetAuthorizationService() const

 { return authorizationService; }

inline const String& LoginContext::GetPrincipalName() const

 { return principalName; }

inline sec_login_auth_src_t LoginContext::GetAuthorizationSource() const

{ return authorizationSource; }

inline LoginContext::LoginStatus LoginContext::GetStatus() const

{ return loginStatus; }

inline sec_login_handle_t LoginContext::GetLoginContext() const

{ return loginContextHandle; } 

inline Boolean LoginContext::IsValid() const

{ return isValid; }

/*
 * KeytableLoginContext 
 */

inline const String& KeytableLoginContext::GetKeyTable() const

{ return keyTable; }

/*
 * PasswordClass
 */

inline PasswordClass::PasswordClass(const char* pnote)

{ memset((char*)pwdbuf, 0, (sizeof pwdbuf)); promptNote = pnote; }

inline void PasswordClass::GetPassword(char* pwd)

{ strcpy(pwd, (char*)pwdbuf); }

inline void PasswordClass::ZeroPassword()
     
{ memset((char*)pwdbuf, 0, sizeof(pwdbuf)); }

#endif
