/*
 * MutexLock
 *
 *    This class represents a non-transactional mutex lock. This type of
 * lock is best used to provide mutually exclusive shared access to data
 * that is not normally updated during a transaction.
 *
 * mutex.H,v 1.20 1994/04/07 19:20:59 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef MUTEX_LOCK_H
#define MUTEX_LOCK_H

extern "C"
{
#include <pthread.h>
#ifdef DOS
#include <dce/cmaerrno.h>
#else
#if HAVE_CMA_ERRNO_H
#include <dce/cma_errno.h>
#endif
#endif
}

#include <objdce/defs.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/errorLevels.H>
#include <errno.h>

class _EXPORT MutexLock

{
public:
	enum MutexType { FAST, RECURSIVE, NONRECURSIVE, DEFAULT };
	MutexLock();                // Create a DEFAULT type mutex
	MutexLock(MutexType type);  // Create a mutex of type "type"
	~MutexLock();               // Destroy lock, perhaps unlocking
	void Lock();                // Lock the lock
	void Unlock();              // Unlock it
	Boolean Test();             // Try to lock it, hold if we can
	operator pthread_mutex_t*();        // Conversion operator
	static void SetDefault(MutexType);  // Debug mutexes?
private:
	void Init(MutexType type);
	pthread_mutex_t mutexRep;           // The internal lock rep.
	pthread_mutexattr_t mutexAttr;      // Its attributes object
	static MutexType defaultType;       // Type for DEFAULT mutexes
};

/*
 * Wait until the mutex is free, then lock it.
 */


					
inline void MutexLock::Lock()

{ 
  if (pthread_mutex_lock(&mutexRep))
   CheckStatus(errno,(const Boolean)false, ErrorLevels::WarnMsg, 1455);
}

/* 
 * Unlock it.
 */

inline void MutexLock::Unlock()

{ 
  if (pthread_mutex_unlock(&mutexRep))
   CheckStatus(errno,(const Boolean)false, ErrorLevels::WarnMsg, 1456);
}

/*
 * Try to lock the mutex. If it could be locked right away, return
 * TRUE. Otherwise, return FALSE immediately.
 */

inline Boolean MutexLock::Test()

{
  Boolean ret;
  if ((ret = pthread_mutex_trylock(&mutexRep) < 0))
   CheckStatus(errno,(const Boolean)false, ErrorLevels::WarnMsg, 1457);
  return ret;
}

inline MutexLock::operator pthread_mutex_t*()

{ return &mutexRep; }

#endif
