/*
 * NonTranServer
 *
 *    This class represents a non-transactional server implemented using
 * standard DCE RPCs.
 *
 * nontranserver.H,v 1.16 1994/04/07 19:21:02 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef NON_TRAN_SERVER_H
#define NON_TRAN_SERVER_H

extern "C"
{
#include <pthread.h>
#include <dce/rpc.h>
}
#include <objdce/objtranparams.H>
#include <String.h>

#ifndef DOS
#include <Path.h>
#endif

#include <objdce/binding.H>
#include <objdce/defs.H>
#include <objdce/transignals.H>

class _EXPORT InterfaceSpec;
class _EXPORT InterfaceStats;
class _EXPORT ServerLoginContext;

/* Constant */

extern const unsigned long TotalMaxCalls;

typedef enum
{
  TCP, UDP, DECnet, ALL
} RPCCommProtocol;

/*
 * GenericRPCServer:
 *
 *    This class contains functions used by both DCE and Encina servers.
 * This is an abstract class. Both clients and servers can use this to 
 * add interfaces to an existing server (before starting it with StartServer)
 * and check the list of bindings and count of RPCs for the entire process.
 */

class _EXPORT GenericRPCServer : public Terminated

{
public:
  GenericRPCServer();
  virtual ~GenericRPCServer();
  virtual void StartServer(const int numThreads = 20) = 0;  // Listen for RPC calls
  void AddInterface(rpc_if_handle_t*, const String& nameServiceName,
		    const String& annotationString, 
		    rpc_mgr_epv_t* entryVector = nil,
		    DCEuuid* typeUUID = nil, DCEuuid* objUUID = nil);
  void StopServer();					// Shut down this server
  void GetBindingList(String&, char separator = '\n');
  void GetStatistics(InterfaceStats&) const;	// Get stats on THIS server
  Boolean IsClntAuthorized(rpc_binding_handle_t& handle,
			   const String& ACLkey, const String& perm);
  Boolean IsClntAuthorized(rpc_binding_handle_t& handle,
			   const String& ACLkey, const List<String>& perms);
  Boolean IsClntAuthorized(rpc_binding_handle_t& handle);
  void DisableRemoteShutdown();				// Disallow shutdown
  void ClearBinding(const String& nsName,
		    rpc_if_handle_t* ifHandle);		// Clear stale info
  virtual void HandleTermination();			// Called by exit handler
protected:			
  void RegisterManagementAuthFunction();
  virtual void SetProtocols(const long port = -1);	// Used by subclasses
  void ClearInterfaces();				// Delete InterfaceSpecs

  rpc_binding_vector_t* bVector;			// Our interface bindings
  RPCCommProtocol protocol;				// What protocol to use
  List_of_p<InterfaceSpec> interfaces;			// Interfaces we exported
  ServerLoginContext* loginContext;			// Our security params.
};

/*
 * NonTranServer:
 *
 *    This is the class that users should actually use. There are four
 * types of constructor. The first two register with the DCE name service,
 * with the second allowing an optional annotation for rpcd, a choice of
 * protocol, and the ability to set type and object UUIDs. The third
 * constructor uses a well-known endpoint, and the last is only used for
 * stand-alone ACL managers that do not export an IDL interface. 
 */

class _EXPORT NonTranServer : public GenericRPCServer, public Signaler, public Signaled
 
{
public:
  NonTranServer(rpc_if_handle_t*, 			// Use CDS
		const String& nameServiceName,
		ServerLoginContext* login = nil,
		rpc_mgr_epv_t* entryVector = nil);
  NonTranServer(rpc_if_handle_t*,			// Add annotation
		const String& nameServiceName,
		const String& annotationString,
		const RPCCommProtocol RPCprotocol,
		ServerLoginContext* login = nil,
		rpc_mgr_epv_t* entryVector = nil,
		DCEuuid* typeUUID = nil, DCEuuid* objUUID = nil);
  NonTranServer(rpc_if_handle_t*,			// Use wk endpoint
		const int portNo,
		const RPCCommProtocol RPCprotocol,
		ServerLoginContext* login = nil,
		rpc_mgr_epv_t* entryVector = nil);
  virtual ~NonTranServer();    
  virtual void StartServer(const int numThreads = 20);
  void DisableKeyboardSignals();
  virtual Signaled::SigStatus HandleSignal(const int);
private:
  void SetUpSignals();
  Boolean allowKeyboardSignals;
};

/*
 * InterfaceSpec:
 *
 *    This is an internal class that represents a single RPC interface entry.
 * The entry consists of a required RPC interface handle, plus an optional
 * name service name, manager entry point vector, rpcd annotation string,
 * type and object UUIDs. This class registers its interface with the CDS
 * and rpcd if a name is specified, and uses the type and object UUIDs if
 * they are specified. If no object UUID is given, this class generates one
 * with which to register the server.
 */

class _EXPORT InterfaceSpec

{
public:
  InterfaceSpec();
  InterfaceSpec(rpc_if_handle_t*, rpc_mgr_epv_t*, 
		const String& nsName, const String& annotString,
	        DCEuuid* typeUUID, DCEuuid* objUUID);
  ~InterfaceSpec();
  void RegisterHandle();
  void RegisterWithRPCD(rpc_binding_vector_t*);
  void RegisterWithCDS(rpc_binding_vector_t*);
  const String& GetNSName() const;
private:
  void ClearBinding();					// Get rid of old ones
  void SetObjUUID(uuid_t* uuid);			// Set object UUID

  rpc_if_handle_t* ifHandle;				// This interface
  rpc_mgr_epv_t* mgrEpv;				// Optional entry point
  rpc_binding_vector_t* bVector;			// Bindings interface uses
  String nameServiceName;				// Name in the CDS
  String annotation;					// Annotation for rpcd
  Boolean intRegistered;				// Interface rgstd?
  Boolean rpcdRegistered;				// Whether we exported 
  Boolean cdsdRegistered;
  uuid_t* typeUUID;					// Type UUID of interface
  uuid_vector_t* objUUIDs;				// Object UUIDs of interface
  DCEuuid* thisObj;					// Default obj. UUID
};

inline const String& InterfaceSpec::GetNSName() const

{ return nameServiceName; }

#endif

