/*
 * ReadWriteLock:
 *    This file implements the ReadWriteLock class.
 *
 * readwrite.C,v 1.6 1993/12/02 14:52:30 lcp Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/readwrite.H>
#include <objdce/error.H>

ReadWriteLock::ReadWriteLock()

{
    pthread_cond_init(&condition, pthread_condattr_default);
    pthread_mutex_init(&latch, pthread_mutexattr_default);
    state = NoLock;
    readLockCount = 0;
    unlockPriority = DEFAULT_PRIORITY;
}

ReadWriteLock::ReadWriteLock(Priority p)

{

    ObjtranReport.Report(ErrorLevels::TraceMsg, "Initing condition/latch locks\n");
    pthread_cond_init(&condition, pthread_condattr_default);
    pthread_mutex_init(&latch, pthread_mutexattr_default);
    state = NoLock;
    readLockCount = 0;
    unlockPriority = p;
}

ReadWriteLock::~ReadWriteLock()

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Destroying condition/latch locks\n");
    pthread_mutex_destroy(&latch);
    pthread_cond_destroy(&condition);
}

void ReadWriteLock::SetPriority(Priority p)

{
    ObjtranReport.VReport(ErrorLevels::TraceMsg,
			  "Setting Priority to %d on read/write lock\n",
			  (int)p);
    unlockPriority = p;
}

Boolean ReadWriteLock::NotLocked() const

{
  return ((state == NoLock) || (state == RWait) || (state == WWait));
}

Boolean ReadWriteLock::ReadLocked() const

{
  return ((state == RLock) || (state == RLockWWait));
}

Boolean ReadWriteLock::WriteLocked() const

{
  return ((state == WLock) || (state == WLockRWait) || (state == WLockWWait));
}

void ReadWriteLock::ReadLock()

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Obtaining Read lock\n");
    pthread_mutex_lock(&latch);
    while (WriteLocked() || (state == WWait))
      {
	if ((unlockPriority == READ_PRIORITY) && (state == WLock))
	  state = WLockRWait;
	pthread_cond_wait(&condition, &latch);
      }
    state = RLock;
    readLockCount++;
    pthread_mutex_unlock(&latch);
}

void ReadWriteLock::WriteLock()

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Obtaining Write lock\n");
    pthread_mutex_lock(&latch);
    while (!NotLocked())
      {
	if (unlockPriority == WRITE_PRIORITY)
	  {
	    if (state == RLock)
	      state = RLockWWait;
	    else if (state == WLock)
	      state = WLockWWait;
	  }
	pthread_cond_wait(&condition, &latch);
      }
    state = WLock;
    pthread_mutex_unlock(&latch);
}

Boolean ReadWriteLock::TestReadLock()

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, "Testing Read lock\n");
  pthread_mutex_lock(&latch);
  Boolean lockMe = (!WriteLocked() && (state != WWait));
  pthread_mutex_unlock(&latch);
  if (lockMe)		// If NOT locked
    {
      ReadLock();
      return true;
    }
  else			// If locked
    {
      return false;
    }
}

Boolean ReadWriteLock::TestWriteLock()

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Testing Write lock\n");
  pthread_mutex_lock(&latch);
  Boolean lockMe = NotLocked();
  pthread_mutex_unlock(&latch);
  if (lockMe) 		// if it's NOT locked
    {
      WriteLock();
      return true;
    }
  else			// If locked
    {
      return false;
    }
}

void ReadWriteLock::Demote()

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Demoting lock\n");
    pthread_mutex_lock(&latch);
    if (WriteLocked())
      {
	state = RLock;
	readLockCount++;
	pthread_cond_broadcast(&condition);
      }
    pthread_mutex_unlock(&latch);
}

void ReadWriteLock::Unlock()

{
    ObjtranReport.Report(ErrorLevels::TraceMsg, "Unlocking...\n");
    pthread_mutex_lock(&latch);
    switch (state)
      {
      case NoLock:
      case RWait:
      case WWait:
	break;
      case RLock:
        ObjtranReport.Report(ErrorLevels::TraceMsg, "Unlocking Read Lock\n");
	readLockCount--;
	if (readLockCount == 0)
	  {
	    state = NoLock;
	    pthread_cond_broadcast(&condition);
	  }
	break;
      case RLockWWait:
        ObjtranReport.Report(ErrorLevels::TraceMsg, "Unlocking Read Lock WWait\n");
	readLockCount--;
	if (readLockCount == 0)
	  {
	    state = WWait;
	    pthread_cond_broadcast(&condition);
	  }
	break;
      case WLock:
        ObjtranReport.Report(ErrorLevels::TraceMsg, "Unlocking Write Lock\n");
	state = NoLock;
	pthread_cond_broadcast(&condition);
	break;
      case WLockRWait:
        ObjtranReport.Report(ErrorLevels::TraceMsg, "Unlocking Write Lock RWait\n");
	state = RWait;
	pthread_cond_broadcast(&condition);
	break;
      case WLockWWait:
        ObjtranReport.Report(ErrorLevels::TraceMsg, "Unlocking Write Lock WWait\n");
	state = WWait;
	pthread_cond_broadcast(&condition);
	break;
      }
    pthread_mutex_unlock(&latch);
}
