/*
 * thread.C,v 1.12 1994/03/17 18:44:18 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/thread.H>
#include <objdce/objtranparams.H>
#include <stdio.h>
#include <objdce/error.H>
#include <objdce/defs.H>
#include <objdce/dcecheckstatus.H>
#include <objdce/transignals.H>

#ifdef DOS
#include <dce/cmaerrno.h>
#else
#if HAVE_CMA_ERRNO_H
#include <dce/cma_errno.h>
#endif
#endif

/*
 * AsyncRunner
 *    This is the function called by pthread_create to actually
 * start a thread. It disables thread cancellation (we want it off by
 * default, as documented and so Encina will work properly by default),
 * then calls "Run" on the ThreadObject it was passed.
 */

static void* AsyncRunner(void* obj)

{
  if (pthread_setcancel(CANCEL_OFF) < 0)
    {
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		  "pthread_setcancel", __LINE__, __FILE__);
    }
  if (pthread_setasynccancel(CANCEL_OFF) < 0)
    {
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		  "pthread_setasynccancel", __LINE__, __FILE__);
    }
  return (((ThreadObject*)obj)->Run());
}

/*
 * The ThreadObject class is the base class for all user-created
 * threads. It also has some handy utility functions.
 */

ThreadObject::ThreadObject()

{
}

ThreadObject::~ThreadObject()

{
}

void ThreadObject::SetTran(Transaction* t)

{
    parentTran = t;
}

void ThreadObject::YieldMe()

{
    pthread_yield();
}

void ThreadObject::SleepMe(const long secs, const long uSecs)

{
    struct timespec sleepTime;
    sleepTime.tv_sec = secs;
    sleepTime.tv_nsec = uSecs * 1000;
    if (pthread_delay_np(&sleepTime))
      {
	CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		    "pthread_delay_np", __LINE__, __FILE__);
      }
}

void ThreadObject::ExitMe(void* retVal)

{
    pthread_exit(retVal);
}

void ThreadObject::SetKillable(const Boolean on)

{
  if (on)
    {
      if (pthread_setcancel(CANCEL_ON) < 0)
	{
	  CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		      "pthread_setcancel", __LINE__, __FILE__);
	}
    }
  else
    {
      if (pthread_setcancel(CANCEL_OFF) < 0)
	{
	  CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		      "pthread_setcancel", __LINE__, __FILE__);
	}
    }
}

void ThreadObject::DoKill()

{
    pthread_testcancel();
}

/*
 * AsyncThread
 *    This is the class that actually spawns the thread. It also lets
 * other threads kill the thread, or wait for it to finish.
 */

/* 
 * Constructors
 *    Start the thread, optionally adding an attributes object if
 * necessary.
 */

AsyncThread::AsyncThread(ThreadObject& tObj)

{
  attrUsed = false;
  Signaler::InitSignals();
  ObjtranReport.Report(ErrorLevels::TraceMsg,
		       "Spawning a thread with the default attributes\n");
  Init(AsyncRunner, (void*)(&tObj));
}

AsyncThread::AsyncThread(ThreadObject& tObj, long stackSize)

{
  attrUsed = true;
  ObjtranReport.VReport(ErrorLevels::TraceMsg,
			"Spawning a thread with a stack size of %l bytes\n",
			stackSize);
  if (pthread_attr_create(&threadAttr))
    {
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		  "pthread_attr_create", __LINE__, __FILE__);
    }
  if (pthread_attr_setstacksize(&threadAttr, stackSize))
    {
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		  "pthread_attr_setstacksize", __LINE__, __FILE__);
    }
  Init(AsyncRunner, (void*)(&tObj));
}

AsyncThread::AsyncThread()

{
  attrUsed = false;
}

/*
 * Init
 *    Do the real work of firing off the thread, and checking if it
 * actually worked.
 */

void AsyncThread::Init(void*(*func)(void*), void* args)

{
    int err;
    detached = done = false;
    if (!attrUsed)
      err = pthread_create(&threadRep, pthread_attr_default, func, args);
    else
      err = pthread_create(&threadRep, threadAttr, func, args);
    if (err)
      {
        // Abort the program if this fails
	CheckStatus(errno, true, ErrorLevels::FatalMsg, 
		    "pthread_create", __LINE__, __FILE__);
      }
}    

/*
 * Destructor
 *    Delete the attributes object, since we won't need it anymore.
 * Also, detach the thread if it hasn't been detached already. We
 * should always detach, since this object is going away and all threads
 * should be detached eventually.
 */

AsyncThread::~AsyncThread()

{
  if (attrUsed)		// OK even if the thread is still running
    {
      if (pthread_attr_delete(&threadAttr))
	{
	  CheckStatus(errno, false, ErrorLevels::ErrorMsg,
		      "pthread_attr_delete", __LINE__, __FILE__);
	}
    }
  if (!detached)
    {
      if (pthread_detach(&threadRep))
	{
	  CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		      "pthread_detach", __LINE__, __FILE__);
	}
    }
}

/*
 * Wait 
 *    If the thread hasn't already terminated, and hasn't been detached,
 * wait for it to finish and return its exit status as a void*.
 */

void* AsyncThread::Wait()

{
  void* retVal = (void*)0;

  if (done)
    {
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1475);
    }
  else if (detached)
    {
      ObjtranReport.ReportMsg(ErrorLevels::ErrorMsg, 1476);
    }
  else
    {
      if (pthread_join(threadRep, &retVal))
	{
	  CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		      "pthread_join", __LINE__, __FILE__);
	  retVal = (void*)0;
	}
      else
	{
	  done = true;
	}
    }
  return retVal;
}

/*
 * Detach
 *    Detach the thread, freeing up storage the system uses to keep
 * track of it. A thread can't be waited for once it's been detached.
 */

void AsyncThread::Detach()

{
  if (detached)
    {
      ObjtranReport.ReportMsg(ErrorLevels::WarnMsg, 1477);
    }
  else
    {
      if (pthread_detach(&threadRep))
	{
	  CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		      "pthread_detach", __LINE__, __FILE__);
	}
      else
	{
	  detached = true;
	}
    }
}

/*
 * SetPriority, GetPriority
 *    Change the priority of this thread, or return it. This is not
 * supported on all platforms.
 */

void AsyncThread::SetPriority(const unsigned long newPrior)

{
  if (pthread_setprio(threadRep, (int)newPrior))
    {
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		  "pthread_setprio", __LINE__, __FILE__);
    }
}

unsigned long AsyncThread::GetPriority() const

{
    unsigned long prior = 0;
    prior = pthread_getprio(threadRep);
    if (prior < 0)
      {
	CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		    "pthread_getprio", __LINE__, __FILE__);
      }
    return prior;
}

/*
 * Kill
 *    Terminate the thread. This does not guarantee that the thread
 * actually completes, but sends the signal nonetheless.
 */

void AsyncThread::Kill()

{
  if (pthread_cancel(threadRep))
    {
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, 
		  "pthread_cancel", __LINE__, __FILE__);
    }
}

/*
 * operator void*
 *    This may or may not be an easier way to wait for a thread.
 * Just calls Wait. 
 */

AsyncThread::operator void*()

{
    return Wait();
}

