/*
 * AsyncThread
 *
 *    An AsyncThread represents an asynchronous thread of control.
 * By creating the thread object, the object's Run method is executed
 * as a separate thread. The new thread is executed outside the scope
 * of any transaction.
 *    The TranThread operates the same way as the AsyncThread, but
 * the new thread is executed inside the scope of the transaction that
 * created the object.
 *    Deleting an AsyncThread object without first detaching the thread
 * or waiting for it to terminate is an error.
 *    The ReturnValue type can be used to return a value from a thread.
 *
 * thread.H,v 1.9 1994/04/07 19:21:10 gjb Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef ASYNC_THREAD_H
#define ASYNC_THREAD_H

#include <objdce/defs.H>

extern "C"
{
#include <pthread.h>
}

class _EXPORT Transaction;

class _EXPORT ThreadObject

{
protected:
	ThreadObject();             // This is an abstract class
public:
	virtual ~ThreadObject();        
	virtual void* Run() = 0;            // Subclasses define this
	void SetTran(Transaction*);         // For internal use only
protected:
	void YieldMe();             // Yield the processor briefly
	void SleepMe(const long secs, 
		 const long uSecs);     // Sleep for a while
	void ExitMe(void*);             // Exit and return a value
	void SetKillable(const Boolean on);     // Turn cancel on or off. Default=on
	void DoKill();              // Exit if we've been cancelled
	
	Transaction* parentTran;            // Transaction we were started under
};

class _EXPORT AsyncThread

{
public:
	AsyncThread(ThreadObject&);         // Create and start thread
	AsyncThread(ThreadObject&, 
		long stackSize);
	virtual ~AsyncThread();         // Error if not done or detached
	void* Wait();               // Wait for completion
	void Detach();              // Specify we won't wait for it
	void SetPriority(const unsigned long newPrior);
	unsigned long GetPriority() const;  
	void Kill();                // Cancel the thread
	operator void*();               // Same as calling Wait
protected:
	AsyncThread();              // Used by subclasses
	void Init(void*(*func)(void*), 
		  void* args); // Used by constructors
	pthread_t threadRep;            // Internal representation
	pthread_attr_t threadAttr;          // Attributes of this thread
	Boolean attrUsed;
private:
	Boolean detached;               // Whether thread is detached
	Boolean done;
};

#endif
