/*
 * lock.C,v 1.11 1994/04/07 19:21:59 gjb Exp
 */



    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#ifndef ENCINA_CLIENT_ONLY

#include <objtran/lock.H>
#include <string.h>
#include <objdce/thread.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objdce/objtranparams.H>
#include <memory.h>
#include <objdce/defs.H>

static const int InitLockNameLen = 32;

lock_mode_t Mode2Mode(const LockMode orig)

{
    lock_mode_t ret;		    // This could just be an assignment, but
    switch (orig)		    // Not all C++es must implement enums as
    {				    // ints.
    case NoModes:
	ret = LOCK_MODE_NONE;
	break;
    case ReadMode:
	ret = LOCK_MODE_READ;
	break;
    case WriteMode:
	ret = LOCK_MODE_WRITE;
	break;
    case UpgradeMode:
	ret = LOCK_MODE_UPGRADE;
	break;
    case IntentionReadMode:
	ret = LOCK_MODE_INTENT_READ;
	break;
    case IntentionWriteMode:
	ret = LOCK_MODE_INTENT_WRITE;
	break;
    case AllModes:
	ret = LOCK_MODE_ALL;
	break;
    }
    return ret;
}

LockName::LockName()

{
  lockRep.name = new char[InitLockNameLen];
  lockRep.length = 0;
} 

LockName::LockName(void* name, const long len)

{
    ObjtranReport.VReport(ErrorLevels::TraceMsg, "Create Lock: %s\n", name);
    lockRep.name = new char[len];
    memcpy(lockRep.name, name, (const int)len);
    lockRep.length = (unsigned int)len;
}

LockName::~LockName()

{
  void* lName = lockRep.name;
  delete [] lName;
}

LockName& LockName::operator = (const lock_name_t name)

{
  void* lName = lockRep.name;
  delete [] lName;
  lockRep = name;
  return *this;
}

StringName::StringName(const String& name) 
    : LockName((void*)((const char*)name), strlen(name) + 1)
{
}

StringName::~StringName()

{
}

PtrName::PtrName(void* ptr) : LockName((void*)&ptr, sizeof(void*))

{
}

PtrName::~PtrName()

{
}

LockSpace::LockSpace()

{
    lock_CreateNameSpace(&spaceRep);
}

LockSpace::~LockSpace()

{
    // Nothing to do
}

LockSpace& LockSpace::operator = (const lock_space_t space)

{
    spaceRep = space;
    return *this;
}

/* 
 * Deadlock detection:
 *
 *    Here, we will spawn off a thread that checks for deadlocks
 * periodically and aborts the deadlocked transaction with the most locks.
 */

class DeadlockDetector : public ThreadObject

{
public:
    DeadlockDetector(const int seconds);
    virtual ~DeadlockDetector();
    virtual void* Run();
private:
    int delaySecs;
};

DeadlockDetector::DeadlockDetector(const int seconds) : ThreadObject()

{
    delaySecs = seconds;
}

DeadlockDetector::~DeadlockDetector()

{
}

static void DeadlockDetectCleanupHandler(void*)

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, "The deadlock detection thread was terminated.\n");
}

void* DeadlockDetector::Run()

{
    int inc;
    tran_tid_t* tids;
    unsigned int numLocks;
    lock_tranList_t* allTrans;

    pthread_cleanup_push(DeadlockDetectCleanupHandler, (void*)0);
    SetKillable(true);
    Boolean hellFreezesOver = false;
    while (!hellFreezesOver)
    {
	lock_GetTranList(&allTrans);
	if (allTrans != (lock_tranList_t*)0)
	  {
	    for (inc = 0; inc < allTrans->numEntries; inc++)
	      {
		lock_TranDeadlockDetect(allTrans->tids[inc], &numLocks, &tids);
		if (numLocks > 0)
		  {
		    ObjtranReport.VReport(ErrorLevels::TraceMsg,
		      "Deadlock Detected for TID: %lu. Number of Locks: %d. Aborting Transaction...\n",
					 allTrans->tids[inc],
					 numLocks);
		    ExistingTransaction deadTran(tids[0]);
		    AbortReason deadReason("Deadlock detected", 0, ObjtranID);
		    deadTran.Abort(deadReason);
		  }
		if (tids != (tran_tid_t*)0)
		  lock_Free(tids);
	      }
	    lock_Free(allTrans);
	  }
	SleepMe(delaySecs, 0);
      }
    pthread_cleanup_pop(true);
    return nil;
}

void DeadlockDetectThread(const int seconds)

{
    ObjtranReport.VReport(ErrorLevels::TraceMsg, "Starting Deadlock Detector Thread. Seconds: %d\n",
			  seconds);
    DeadlockDetector* detector = new DeadlockDetector(seconds);
    AsyncThread* deadThread = new AsyncThread(*detector);
    delete deadThread;
}

#endif
