/*
 * LockMode, LockName, LockSpace
 *
 *    These are supporting classes for transactional locks. To actually
 * acquire a transactional lock, use the AcquireLock, TestLock, 
 * ChangeLockMode and ReleaseLock methods of the Transaction class.
 * These locks are released when the transaction commits, and generally
 * should not be released before then.
 *    The LockMode class (an enumerated type) refers to the mode of a lock.
 *    A LockName is the name of a lock. The StringName subclass can be used
 * to create a lock with a string name, while the PtrName subclass can be used
 * when the name is a pointer.
 *    A LockSpace object may be created by the user to specify an alternate name
 * space so locks do not conflict. If none is specified, a default name
 * space is used. Acquiring a lock with the same name and mode as an existing 
 * lock will cause an error. Multiple locks may be acquired with the same
 * name, however, if their modes are different.
 *
 * lock.H,v 1.6 1994/04/07 19:22:01 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#ifndef ENCINA_CLIENT_ONLY

#ifndef LOCK_STUFF_H
#define LOCK_STUFF_H

extern "C"
{
#include <pthread.h>
}
#include <stdio.h>
extern "C"
{
#include <lock/lock.h>
}
#include <String.h>
#include <objdce/defs.H>

extern void DeadlockDetectThread(const int seconds);	// Check for deadlocks 

typedef enum LockMode 

{
    NoModes,
    ReadMode,
    WriteMode,
    UpgradeMode,
    IntentionReadMode,
    IntentionWriteMode,
    AllModes
};

extern lock_mode_t Mode2Mode(const LockMode);

class LockName

{
public:
    LockName();
    LockName(void* name, const long len);	// Create a new lock name
    ~LockName();
    operator lock_name_t() const;		// Conversion
    LockName& operator = (const lock_name_t);	// Set internal rep
private:
    lock_name_t lockRep;			// Internal representation
};

class StringName : public LockName

{
public:
    StringName(const String& name);		// Create a name from a string
    virtual ~StringName();
};

class PtrName : public LockName		

{
public:
    PtrName(void* ptr);				// Create a name from a pointer
    virtual ~PtrName();
};

class LockSpace

{
public:
    LockSpace();				// Create a new lock name space
    ~LockSpace();				// Destroy it
    operator lock_space_t() const;		// Conversion
    LockSpace& operator = (const lock_space_t); // Set internal rep.
private:
    lock_space_t spaceRep;			// Internal rep.
};

inline LockName::operator lock_name_t() const

{ return lockRep; }

inline LockSpace::operator lock_space_t() const

{ return spaceRep; }

#endif

#endif
