/*
 * Log, LogRecord
 *
 *    These classes interface with the Encina Log Service. A Log object 
 * represents a single Encina log file, stored by the Log Server. LogRecord
 * objects can be created and read and written from the Log object. A
 * LogSeqNumber object may be used to retrieve a particular LogRecord.
 * LogRecord objects are created by adding LogBuffer objects and LogChain
 * objects, if necessary.
 *
 * log.H,v 1.8 1994/04/07 19:22:04 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef LOG_OBJECTS_H
#define LOG_OBJECTS_H

extern "C"
{
#include <pthread.h>
#include <tran/tran.h>
#include <log/log.h>
}
#include <objdce/defs.H>
#include <objdce/bdemem.H>
#include <iostream.h>
#include <objdce/objtranparams.H>
#include <objdce/login.H>
#include <Path.h>
#include <List.h>
#include <String.h>
#include <Map.h>

/*
 * LogSeqNumber
 *    A LOG sequence number. Every record gets one of these. With this
 * class, we can compare them properly.
 */

class LogSeqNumber

{
public:
  LogSeqNumber();
  LogSeqNumber(const LogSeqNumber&);
  LogSeqNumber(const log_lsn_t&);
  LogSeqNumber& operator = (const LogSeqNumber&);
  LogSeqNumber& operator = (const log_lsn_t&);
  operator const log_lsn_t*() const;
  operator log_lsn_t() const;
  int operator == (const log_lsn_t&) const;
  int operator != (const log_lsn_t&) const;
  int operator < (const log_lsn_t&) const;
  int operator > (const log_lsn_t&) const;
  int operator <= (const log_lsn_t&) const;
  int operator >= (const log_lsn_t&) const;
private:
  log_lsn_t lsnRep;
};

/*
 * BufferBase
 *    The base class for a buffer of LOG storage. Basically a dynamic
 * array of characters.
 */

class BufferBase

{
public:
  BufferBase();
  BufferBase(const unsigned long length); 			// An empty one
  BufferBase(const void* data, const unsigned long length);	// Create buffer
  virtual ~BufferBase();
  BufferBase& operator = (const BufferBase&);
  void SetBuf(const void* data, const unsigned long length);   // Replace stuff
  void SetBufAt(const unsigned long where, const void* data,
		const unsigned long length);
  void Resize(const unsigned long length);			// Change max size
  const char* GetData() const;					// return bytes
  const char* GetDataAt(unsigned long pos) const;		// Get data at
  const unsigned long GetLength() const;			// Return len
  void SetDataLen(const unsigned long length);			// Set size
  const unsigned long GetDataLen() const;			// Return len
private:
  char* buffer;				// The data
  unsigned long bufferLength; 		// Size allocated for buffer
  unsigned long dataLength; 		// Length of data in buffer
};

/*
 * LogBuffer
 *    A type of BufferBase used to store information to be written in 
 * a LogRecord.
 */

class LogBuffer : public BufferBase

{
public:
    LogBuffer();
    LogBuffer(const unsigned long length); 
    LogBuffer(const void* data, const unsigned long length);
    LogBuffer(const LogBuffer&);
    virtual ~LogBuffer();
    Boolean operator == (const LogBuffer&) const;
    LogBuffer& operator = (const LogBuffer&);  
};

/*
 * LogChain
 *    Information on a LOG chain identifier. Chains can be used to link
 * lists of log records. Often, there's one per Encina transaction.
 */

class LogChain : public BufferBase

{
public:
    LogChain(const unsigned long length = 0);
    LogChain(const void* data, const unsigned long length, 
	     Boolean closeMe = false);
    LogChain(const LogChain&);
    virtual ~LogChain();
    LogChain& operator = (const LogChain&);
    Boolean operator == (const LogChain&) const;
    void SetDoClose(const Boolean);		// This is last rec in chain
    Boolean DoClose() const;			// Is this the last record?
    void SetPrevLSN(const LogSeqNumber&);	// Set one before this
    const LogSeqNumber& GetPrevLSN() const;	// Get it
private:
    Boolean	    doClose;		// Should chain be closed?
    LogSeqNumber    prevLSN;		// Previous record
};

/*
 * LogRecord
 *    A LogRecord consists of a list of LogBuffers containing the data in
 * the record and a list of LogChains listing which chains to which this
 * record belongs.
 */

class LogRecord

{
public:
    LogRecord();
    LogRecord(const unsigned long len);				// Make empty
    LogRecord(const tran_logRecord_t);				// Put in data
    LogRecord(const void* data, const unsigned int len);	// Put in data
    ~LogRecord();
    log_status_t SetupReadRecord();

    void AddBuffer(const LogBuffer&, 
		   Boolean addAtHead = false);			// Add a buffer
    const char* GetData(unsigned int idx = 0);			// Return one
    const unsigned long GetLength(unsigned int idx = 0) const;	// Get its length
    const unsigned long GetDataLen(unsigned int idx = 0) const;
    LogBuffer& GetBuffer(unsigned int idx = 0);			
    void AddChain(const LogChain&, 				// Add a chain
		  Boolean addAtHead = false);
    LogChain& GetChain(unsigned int idx = 0);			// Get it back
    const int GetNumChains() const;
    const int GetNumLogBuffers() const;
    const log_recordType_t& GetRecordType() const;		// Get rec. type
    void PutRecordType(log_recordType_t&);
    void SetLSN(const LogSeqNumber&);
    LogSeqNumber GetLSN() const;
private:
    void LogRecordInit();
    log_recordType_t	    recordType;		// The type of this record
    LogSeqNumber	    lsn;		// Its sequence number
    List<LogBuffer>	    buffers;		// All the buffers
    List<LogChain>	    chains;		// All the chain IDs
};

/*
 * LogFileUser
 *    The LOG service will notify clients when particular things happen
 * the the log file. In particular, LOG makes a callback go off when
 * parts of the file are about to be archived, and when the file is
 * filling up. Users of the LogFile class that want to be notified
 * of these events must declare a subclass of LogFileUser and pass an
 * pointer to a valid LogFileUser object when opening the file.
 */

class LogFileUser

{
public:
  enum LogCommand	
    {
      FileFillingUp,		// LOG_CALLBACK_CMD_CHECKPOINT in LOG manual
      ArchiveImminent,		// LOG_CALLBACK_CMD_ADVISECKPT
      Invalid
    };
protected:
  LogFileUser();
public:
  virtual ~LogFileUser();
  virtual void LogFileCallback(LogCommand command,
			       const String& archiveName,
			       const LogSeqNumber& lsn,
			       int error) = 0;
};

/*
 * LogFile
 *    This class writes LogRecord objects to a LOG file. It can also read
 * them back from an archive, and manipulate the restart record.
 */

class LogFile

{
public:
  enum WriteMode			// Whether to flush the record to disk
    {
      BufferedWrite,
      FlushWrite,
      ForceWrite
    };
public:
  LogFile(const String& name, LoginContext::ProtectionLevel authnLevel,
	  LogFileUser* user = (LogFileUser*)0);
  ~LogFile();
  log_status_t Write(LogRecord&, log_recordType_t recordType,
		     WriteMode mode);			// Write one record
  log_status_t Read(LogRecord&, log_recordType_t& recordType, 
		    const String& archive);		// Read from an archive
  log_status_t WriteRestart(LogBuffer&);		// Write restart record
  log_status_t GetHighStableLSN(LogSeqNumber& highLogSeq); // Last record on disk
  log_status_t ReadRestart(LogBuffer&);			// Get restart rec
  log_status_t Check(const LogSeqNumber&, 
		     WriteMode mode);		// Check that rec is written
  log_status_t GetLastPermanent(LogSeqNumber&);
  log_status_t TruncateArchive(const String& archive, const LogSeqNumber&,
			       Boolean includeLast);	// Set the "tail"
  log_status_t FlushToArchives();			
  log_status_t SetFilter(const String& archive, 
			 const String& filter);		// Set filter function
  log_status_t SetRestartPoint(const LogSeqNumber&);
  operator log_handle_t();
  static LogFileUser* GetUser(log_handle_t handle);
private:
  log_handle_t logRep;
  LogSeqNumber latestLSN;    // used via recovery, not updated otherwise
  static Map<log_handle_t, LogFileUser*>* callbackMap;
};

/*
 * LogCursor
 *    This class is used to read from a log file based on a range of log 
 * records.
 */

class LogCursor

{
public:
  LogCursor(const String& archive, List<LogChain>& chains,
	    const LogSeqNumber* start, const LogSeqNumber* end, LogFile&);
  ~LogCursor();
  log_status_t Move(const LogSeqNumber* start, const LogSeqNumber* end);
  log_status_t Read(LogRecord&, log_recordType_t& recordType);
  operator log_cursor_t();
private:
  log_cursor_t cursorRep;
};

inline LogFile::operator log_handle_t()
     
{ return logRep; }

inline LogSeqNumber::operator const log_lsn_t*() const

{ return &lsnRep; }

inline LogSeqNumber::operator log_lsn_t() const

{ return lsnRep; }

inline const char* BufferBase::GetData() const
{ return buffer; }

inline const char* BufferBase::GetDataAt(unsigned long pos) const
{ return (buffer + pos); }

inline const unsigned long BufferBase::GetLength() const
{ return bufferLength; }

inline const unsigned long BufferBase::GetDataLen() const
{ return dataLength; }

inline void BufferBase::SetDataLen(const unsigned long len)
{
    dataLength = (bufferLength > len) ? len : bufferLength;
}
    
inline const LogSeqNumber& LogChain::GetPrevLSN() const

{ return prevLSN; }

inline Boolean LogChain::DoClose() const

{ return doClose; }

inline LogBuffer& LogRecord::GetBuffer(unsigned int idx) 
    { return buffers[idx]; }

inline const char*  LogRecord::GetData(unsigned int idx) 
    { 
    const char * retData;
    if (buffers.length() >= (idx +1))
	{ retData = buffers[idx].GetData(); }
    else
	{ retData = NULL; } 
    return retData; 
    }

inline const unsigned long LogRecord::GetLength(unsigned int idx) const
    { 
    unsigned long retLen;
    if (buffers.length() >= (idx +1))
	{ retLen = buffers[idx].GetLength(); }
    else
	{ retLen = 0; } 
    return retLen; 
    }

inline const unsigned long LogRecord::GetDataLen(unsigned int idx) const 
    { 
    unsigned long retLen;
    if (buffers.length() >= (idx +1))
	{ retLen = buffers[idx].GetDataLen(); }
    else
	{ retLen = 0; } 
    return retLen; 
    }

inline const int LogRecord::GetNumChains() const

{ return chains.length(); }

inline const int LogRecord::GetNumLogBuffers() const
    { return buffers.length(); }

inline LogSeqNumber LogRecord::GetLSN() const

{ return lsn; }

inline const log_recordType_t& LogRecord::GetRecordType() const
    { return recordType; }

inline void LogRecord::PutRecordType(log_recordType_t& _recordType)
    { recordType = _recordType; }

inline LogCursor::operator log_cursor_t()
     
{ return cursorRep; }

#endif
