/*
 * Recoverable
 *
 *    This class implements one of the two interfaces applications should
 * use to make themselves recoverable. To have Encina manage disk storage
 * in a recoverable manner, use the RecoveryService object and the Operation
 * class. If you use this class instead, you must make sure that changes
 * to storage are logged, either by putting it in your own code or making
 * sure any remote database you talk to does so.
 *    The TranLogRecord class represents data from the transaction service that
 * must be stored. The DoReplays function must call Replay for all the 
 * transactions that have not yet been aborted or finished.
 *    The FakeRecoverable class makes TRAN think an application is recoverable.
 * At least one transaction in each application must be recoverable, and
 * writing or using a recoverable application is difficult. Applications should
 * use the FakeRecoverable class temporarily during development if necessary.
 *
 * recoverable.H,v 1.7 1994/04/07 19:22:10 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef RECOVERABLE_H
#define RECOVERABLE_H

extern "C"
{
#include <pthread.h>
#include <tran/tran.h>
}
#include <objdce/objtranparams.H>
#include <objdce/bdemem.H>
#include <Map.h>
#include <objtran/transaction.H>
#include <objtran/trancomponent.H>
#include <objdce/defs.H>
#include <objtran/log.H>

/*
 * TranLogRecord
 *    An abstraction on top of the TRAN log record structure. It should
 * be packed and unpacked before making it part of an actual buffer.
 */

/*
 * When packed, LogRecord headers are always 16 machine-independent bytes.
 * See recoverable.C for details on the format.
 */

static const unsigned int PackedRecHeaderSize = 16;

class TranLogRecord : public LogRecord
{
public:
    enum RecLogDataType_t {
	rldtNull = 0,
	rldtWrite,
	rldtPrepare,
	rldtCommit,
	rldtAbort,
	rldtFinished,
	rldtIsActive,
	rldtNotActive,
	rldtLockInfo
	};

    typedef struct {
	tran_tid_t	    topTid;
	tran_tid_t	    subTid;
	unsigned long 	    dataLen;
	tran_recOptimization_t	opt;
	RecLogDataType_t    type;
	} TranLogRecordHeader_def;

    TranLogRecord();
    TranLogRecord(const unsigned long len);		// Create empty record
    TranLogRecord(const RecLogDataType_t, 
		    const void* data, 
		    const unsigned int length); 	// Create with data
    TranLogRecord(const tran_logRecord_t);
    ~TranLogRecord();			
    void PutTopTid(const tran_tid_t);
    tran_tid_t GetTopTid() const;
    void PutSubTid(const tran_tid_t);
    tran_tid_t GetSubTid() const;
    void PutRecOpt(const tran_recOptimization_t*);
    tran_recOptimization_t GetRecOpt() const;
    void PutRecType(const RecLogDataType_t);
    RecLogDataType_t GetRecType() const;
    unsigned long GetDataLen() const;
    void PutDataLen(unsigned long);
    void SetupHeader(Transaction&, 
		     RecLogDataType_t, 
		     tran_recOptimization_t*);		// Put header in buffer
    void PrepareToWrite();		// Set up buffers for writing
    void Pack(void*) const;		// Pack into pre-allocated storage
    void Unpack(const void*);		// Unpack from bytes
private:
    void InitTranLogRecord(const RecLogDataType_t dataType = rldtNull);
    TranLogRecordHeader_def header;
};

class Recoverable : public TranComponent

{
  friend void ActiveCallback(tran_serviceId_t, tran_tid_t, int);
  friend void PrepareCallback(tran_serviceId_t, tran_tid_t,
			      tran_logRecord_t, tran_recOptimization_t*);
  friend void CommitCallback(tran_serviceId_t, tran_tid_t,
			     tran_recOptimization_t*);
  friend void AbortCallback(tran_serviceId_t, tran_tid_t,
			    tran_recOptimization_t*);
  friend void WriteCallback(tran_serviceId_t, tran_tid_t,
			    tran_logRecord_t, tran_recOptimization_t*);
  friend void FinishedCallback(tran_serviceId_t, tran_tid_t,
			       tran_recOptimization_t*);
protected:
  Recoverable();				// Make new recoverable object
public:
  virtual ~Recoverable();			// Destroy it
  virtual unsigned long PreInitialize();
  virtual unsigned long Initialize();// Do actual initialization
  virtual unsigned long Terminate();
  virtual void Active(	Transaction&,  
			TranLogRecord&) = 0;// Tran becomes active or not
  virtual void Prepare(Transaction&,
		       TranLogRecord&) = 0; // Can tran be prepared?
  virtual void Commit(Transaction&,  
			TranLogRecord&) = 0; // Tran commits
  virtual void Abort(Transaction&,  
			TranLogRecord&) = 0;	// Tran aborts
  virtual void Write(Transaction&,
		     TranLogRecord&) = 0;	// Write a log record
  virtual void Finished(Transaction&,  
			TranLogRecord&) = 0; 
					    // We're done with this one
protected:
    void ReadOnly(const Transaction&);		// We don't care if it commits
    void Refuse(const Transaction&);		// We can't commit this
    void Replay(tran_tid_t&,
		tran_logRecord_t&);		// Replay a log record
  tran_serviceId_t serviceRep;		// The actual service
private:
  static Map<tran_serviceId_t, Recoverable*>* AllServices;
};

class FakeRecoverable : public Recoverable

{
public:
    FakeRecoverable();
    virtual ~FakeRecoverable();
    virtual void Active(Transaction&, TranLogRecord&);
    virtual void Prepare(Transaction&, TranLogRecord&);
    virtual void Commit(Transaction&,  
			TranLogRecord&);
    virtual void Abort(Transaction&,  
			TranLogRecord&);
    virtual void Write(Transaction&, TranLogRecord&);
    virtual void Finished(Transaction&,  
			TranLogRecord&);
};


inline void TranLogRecord::PutTopTid(const tran_tid_t tid)
    { header.topTid = tid; }
inline tran_tid_t TranLogRecord::GetTopTid() const
    { return header.topTid; }
inline void TranLogRecord::PutSubTid(const tran_tid_t tid)
    { header.subTid = tid; }
inline tran_tid_t TranLogRecord::GetSubTid() const
    { return header.subTid; }
inline void TranLogRecord::PutRecOpt(const tran_recOptimization_t* opt)
    { header.opt = *opt; }
inline tran_recOptimization_t TranLogRecord::GetRecOpt() const
    { return header.opt; }
inline void TranLogRecord::PutRecType(const RecLogDataType_t type)
    { header.type = type; }
inline TranLogRecord::RecLogDataType_t TranLogRecord::GetRecType() const
    { return header.type; }
inline unsigned long TranLogRecord::GetDataLen() const
    { return header.dataLen; }
inline void TranLogRecord::PutDataLen(unsigned long newLen)
   { header.dataLen = newLen; }

#endif
