/*
**   RecoveryLog
**
**  xa Recoverable Logging class
**
** recoverylog.C,v 1.12 1994/01/18 17:11:26 gjb Exp
*/

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/recoverylog.H>
#include <objtran/rpctrancheckstatus.H>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <netinet/in.h>     
/** Not now
extern "C"
{
#include <admin/admin.h>
#include <admin/admin_lock.h>
}
***/
#include <objdce/binding.H>

#ifdef _AIX
#include <set.c>
#include <List.c>

#pragma declare(Set<ChainNode>)
#pragma declare(List<LogSeqNumber>)
#endif
    
/*
 * Handy constants for use by checkpointing 
 */

static const time_t DefaultTimeInterval = 1800;	// Checkpoint every 30 mins.
static const unsigned long DefaultWriteInterval = 1000;  // or every 1000 writes
static const unsigned long DefaultLockBufferSize = 512;	// Buffer for lock_Save

/*
 * Globals
 */

RecoveryLog* RecoveryLog::myself = (RecoveryLog*)0;

/*
 * ChainNode
 *    A little utility class so we can have a hash table of log chain
 * IDs. Users just access data members of this class directly.
 */

ChainNode::ChainNode()

{
  tid = TRAN_TID_NULL;
  lsn = LOG_NULL_LSN;
  commitLogged = false;
  abortLogged = false;
  finishLogged = false;
}

ChainNode::ChainNode(tran_tid_t t, log_lsn_t l)

{
  tid = t;
  lsn = l;
  commitLogged = false;
  abortLogged = false;
  finishLogged = false;
}

int ChainNode::operator == (const ChainNode& node) const

{
  return tran_TidEqual(tid, node.tid);
}

Set_or_Bag_hashval Set<ChainNode>::hash(const ChainNode& node)
     
{ 
  return (Set_or_Bag_hashval)(tran_TidHash(node.tid));
}

/*
 * RecoveryLog
 *    Initialize the data of this class, but don't actually open the
 * log file until later.
 */

RecoveryLog::RecoveryLog(const String& logFileName, 
			 const String& archiveFileName,
			 LoginContext::ProtectionLevel authnLevel,
			 Boolean initLockService)

{
  char* envVal;

  myself = this;
  logName = logFileName;
  archiveName = archiveFileName;
  logAuthn = authnLevel;
  logFileCtl = NULL;
  useLock = initLockService;
  envVal = getenv("OBJTRAN_CHECKPOINT_SECONDS");
  if (envVal == (char*)0)
    timeInterval = DefaultTimeInterval;
  else
    timeInterval = atol(envVal);
  lastCheckpointTime = time(NULL);
  envVal = getenv("OBJTRAN_CHECKPOINT_WRITES");
  if (envVal == (char*)0)
    writeInterval = DefaultWriteInterval;
 else
    writeInterval = atol(envVal);
  writeCount = 0;
  recovering = false;
#ifdef ENCINA_CLIENT_ONLY
  if (initLockService)
    {
      ObjtranReport.ReportMsg(ErrorLevels::WarnMsg, 2519);
    }
#endif
}

/*
 * ~RecoveryLog
 *    Close the logfile.
 */

RecoveryLog::~RecoveryLog()

{
  if (logFileCtl != NULL)
    {
      Checkpoint();		// Set tail before closing for fast recovery
      delete logFileCtl;
    }
}

/*
 * RecRecord
 *    If this transaction hasn't already produced a log record, then
 * insert one in the appropriate list and table. If it has, update it
 * so optimized callbacks don't get re-issued later on.
 */

Boolean RecoveryLog::RecRecord(TranLogRecord& rec,
                               Boolean* alreadyLogged)

{
  Boolean retVal;
  tran_tid_t tid = rec.GetTopTid();
  ChainNode thisOne(tid, LOG_NULL_LSN);
  *alreadyLogged = false;

  tableLatch.Lock();
  ChainNode* foundIt = (ChainNode*)(chainsByTid.contains(thisOne));
  if (foundIt == (ChainNode*)0)		// Insert a new node
    {
      ChainNode newNode(tid, LOG_NULL_LSN);
      switch (rec.GetRecType())
	{
	case TranLogRecord::rldtCommit:
	  newNode.commitLogged = true;
	  break;
	case TranLogRecord::rldtAbort:
	  newNode.abortLogged = true;
	  break;
	case TranLogRecord::rldtFinished:
	  newNode.finishLogged = true;
	  break;
	default:
	  break;
	}
      chainsByTid.insert(newNode);
      retVal = false;
    }
  else				// Update the existing node
    {
      switch(rec.GetRecType())
	{
	case TranLogRecord::rldtCommit:
	  if (foundIt->commitLogged)
	    *alreadyLogged = true;
	  else
	    foundIt->commitLogged = true;
	  break;
	case TranLogRecord::rldtAbort:
	  if (foundIt->abortLogged)
	    *alreadyLogged = true;
	  else
	    foundIt->abortLogged = true;
	  break;
	case TranLogRecord::rldtFinished:
	  if (foundIt->finishLogged)
	    *alreadyLogged = true;
	  else
	    foundIt->finishLogged = true;
	  break;
	default:
	  break;
	}
      retVal = true;
    }
  tableLatch.Unlock();
  return retVal;
}

/*
 * RecCleanup
 *    Remove information on a chain from the tables. Called after
 * a chain is closed.
 */

void RecoveryLog::RecCleanup(const TranLogRecord& rec)

{
  tran_tid_t topTid = rec.GetTopTid();
  ChainNode testNode(topTid, LOG_NULL_LSN);
  LogSeqNumber chainStart;
  log_lsn_t lsn;

  tableLatch.Lock();
  const ChainNode* foundIt = chainsByTid.contains(testNode);
  if (foundIt != (ChainNode*)0)
    {
      chainStart = foundIt->lsn;
      Listiter<LogSeqNumber> iter(chainIDs);
      iter.remove_next(chainStart);
      chainsByTid.remove(testNode);
    }
  if (ObjtranReport.Active(ErrorLevels::TraceMsg) && (foundIt != (ChainNode*)0))
    {
      lsn = chainStart;
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "TID %lu: Log chain closed at LSN %i %i %i %i\n",
			    topTid, lsn.data[0], lsn.data[1], lsn.data[2], lsn.data[3]);
    }
  tableLatch.Unlock();
}

/*
 * RecChainStart
 *    Called after a log record is written to make sure we know where a
 * particular log chain begins.
 */

void RecoveryLog::RecChainStart(const TranLogRecord& rec)

{	
  tran_tid_t topTid = rec.GetTopTid();
  ChainNode testNode(topTid, LOG_NULL_LSN);
  log_lsn_t lsn;
  LogSeqNumber lsnObj;

  tableLatch.Lock();
  const ChainNode* foundIt = chainsByTid.contains(testNode);
  ChainNode* changeIt = (ChainNode*)foundIt;
  if (changeIt != (ChainNode*)0)
    {
      lsnObj = changeIt->lsn;
      if (lsnObj == LOG_NULL_LSN)
	{
	  lsn = rec.GetLSN();
	  chainIDs.put(rec.GetLSN());	// Add to list of chain IDs
	  changeIt->lsn = lsn;
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,
				"TID %lu: New log chain at LSN %i %i %i %i\n",
				topTid, lsn.data[0], lsn.data[1], lsn.data[2], lsn.data[3]);
	}
    }
  tableLatch.Unlock();
}

/*
 * Transaction callbacks
 *    Each of these is called when the TRAN service calls the appropriate
 * recovery service callback.
 */

/* 
 * Active
 *    Transactions will be happy at recovery time if we don't worry
 * about them until they prepare, so don't log anything here.
 */

void RecoveryLog::Active(Transaction&, TranLogRecord&)

{
/**** Uncomment this if you want transaction starts logged
  Boolean alreadyLogged;
  Boolean oldChain = RecRecord(_tranLogRec, &alreadyLogged);
  if ((_tranLogRec.GetRecType() == TranLogRecord::rldtIsActive) &&
      !oldChain)
    {
      _tranLogRec.PrepareToWrite();
      log_status_t logStatus =
	WriteToLog(_tranLogRec, LogFile::ForceWrite);
      RecRecordNew(_tranLogRec);
    }
****/
}

/*
 * Prepare
 *    Force the log record to the disk. The TRAN information for this record
 * should already be in the first buffer. If optimization parameters are
 * true, then the WriteCommit, WriteAbort and WriteFinished functions
 * will place additional buffers in the record. Finally, the header for
 * this log record is placed in the first position via the PrepareToWrite
 * function, and the whole record is forced to the disk.
 */

void RecoveryLog::Prepare(Transaction& tran, 
			  TranLogRecord& _tranLogRec)
{
  Boolean alreadyLogged;
  log_status_t logStatus = LOG_SUCCESS;
  unsigned long lockErr = 0;
  LogFile::WriteMode mode = LogFile::ForceWrite;
  tran_recOptimization_t opt = _tranLogRec.GetRecOpt();

  ObjtranReport.Report(ErrorLevels::TraceMsg, "In RecoveryLog::Prepare\n");
#ifndef ENCINA_CLIENT_ONLY
  if (useLock)
    {
      lockErr = WriteLockRecord(tran, _tranLogRec);
      if (lockErr != LOCK_SUCCESS)
	{
	  ObjtranReport.Report(ErrorLevels::FatalMsg, 2510);
	  logStatus = LOG_FATAL_ERROR;
	}
    }
#endif

  RecRecord(_tranLogRec, &alreadyLogged);
  if (logStatus == LOG_SUCCESS)
    {
      if (opt.commitImminent)
	{
	ObjtranReport.Report(ErrorLevels::TraceMsg, "Commit Imminent -- Calling WriteCommit\n");
	logStatus = WriteCommit(tran, _tranLogRec);
	}
    }
  if (logStatus == LOG_SUCCESS)
    {
      if (opt.abortImminent)
	{
	ObjtranReport.Report(ErrorLevels::TraceMsg, "Abort Imminent -- Calling WriteAbort\n");
	logStatus = WriteAbort(tran, _tranLogRec);
	}
    }
  if (logStatus == LOG_SUCCESS)
    {
      if (opt.finishImminent)
	{
	ObjtranReport.Report(ErrorLevels::TraceMsg, "Finish Imminent -- Calling WriteFinished\n");
	logStatus = WriteFinished(tran, _tranLogRec);
	}
    }
  if (logStatus == LOG_SUCCESS)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling PrepareToWrite\n");
      _tranLogRec.PrepareToWrite();
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Calling WriteToLog\n");
      logStatus = WriteToLog(_tranLogRec, mode);
    }
  if (logStatus != LOG_SUCCESS)
    {
      ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2512);
      Refuse(tran);
    }
}

/*
 * Commit
 *    There's no TRAN information to log here, although a finished
 * log header may still be written if the appropriate optimization 
 * parameter is turned on.
 */

void RecoveryLog::Commit(Transaction& tran,
			 TranLogRecord& _tranLogRec)

{
  Boolean alreadyLogged;  
  log_status_t logStatus = LOG_SUCCESS;
  LogFile::WriteMode mode = LogFile::BufferedWrite;

  ObjtranReport.Report(ErrorLevels::TraceMsg, "In RecoveryLog::Commit\n");
  RecRecord(_tranLogRec, &alreadyLogged);
  if (!alreadyLogged)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg, "In RecoveryLog::CommitRecord NOT already logged\n");
      tran_recOptimization_t opt = _tranLogRec.GetRecOpt();
      if (opt.finishImminent)
	{
	ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Commit - Calling WriteFinished\n");
	  logStatus = WriteFinished(tran, _tranLogRec);
	  mode = LogFile::ForceWrite;
	}
      if (logStatus == LOG_SUCCESS)
	{
          ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Commit - Calling PrepareToWrite\n");
	  _tranLogRec.PrepareToWrite();
	ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Commit - Calling WriteToLog\n");
	  logStatus = WriteToLog(_tranLogRec, mode);
	}
      if (logStatus != LOG_SUCCESS)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2512);
	  Refuse(tran);
	}
    }
  else
    ObjtranReport.Report(ErrorLevels::TraceMsg, "\"Commit\" callback already logged\n");
}

/*
 * Abort
 *    This operation can also be buffered rather than forced to disk. 
 * The first log buffer will hold the log record header. There are no other
 * buffers.
 */

void RecoveryLog::Abort(Transaction& tran,
			TranLogRecord& _tranLogRec)
{
  Boolean alreadyLogged;  
  log_status_t logStatus = LOG_SUCCESS;
  LogFile::WriteMode mode = LogFile::BufferedWrite;

  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Abort\n");
  RecRecord(_tranLogRec, &alreadyLogged);
  if (!alreadyLogged)
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Abort - NOT alreadyLogged\n");
      tran_recOptimization_t opt = _tranLogRec.GetRecOpt();
      if (opt.finishImminent)
	{
	  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Abort - Calling WriteFinished\n");
	  logStatus = WriteFinished(tran, _tranLogRec);
	  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Abort - Setting Mode to ForceWrite\n");
	  mode = LogFile::ForceWrite;
	}
      if (logStatus == LOG_SUCCESS)
	{
	 ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Abort - Calling PrepareToWrite\n");
	  _tranLogRec.PrepareToWrite();
	 ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Abort - Calling WriteToLog\n");
	  logStatus = WriteToLog(_tranLogRec, mode);
	}
      if (logStatus != LOG_SUCCESS)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2512);
	  Refuse(tran);
	}
    }
  else
    ObjtranReport.Report(ErrorLevels::TraceMsg, "\"Abort\" callback already logged\n");
}

/*
 * Write
 *    This operation must be forced. The first buffer holds the log header,
 * and the second holds the record to be written. There are no
 * optimizations here.
 *    Write requests for TRAN_TID_NULL are special -- they're really
 * restart information, so we write them to the restart record.
 */

void RecoveryLog::Write(Transaction& tran, 
			TranLogRecord& _tranLogRec)
{  
  tran_tid_t tid = tran;
  Boolean alreadyLogged;
  log_status_t logStatus = LOG_SUCCESS;

  if (tid == TRAN_TID_NULL)		// Treat as TRAN restart information
    {
      ObjtranReport.Report(ErrorLevels::TraceMsg,
			   "In RecoveryLog::Write - TID is NULL - Calling SetTranRestart\n");
      logStatus = SetTranRestart(_tranLogRec.GetData(), 
				 _tranLogRec.GetDataLen());
      if (logStatus != LOG_SUCCESS)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 2505);
	  PrintTranMessage(logStatus, ErrorLevels::ErrorMsg);
	  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2512);
	  Refuse(tran);
	}
    }
  else					// Treat like other log records
    {
      RecRecord(_tranLogRec, &alreadyLogged);
      _tranLogRec.PrepareToWrite();
      logStatus = WriteToLog(_tranLogRec, LogFile::ForceWrite);
      if (logStatus != LOG_SUCCESS)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2512);
	  Refuse(tran);
	}
    }
}

/*
 * Finished
 *    This is the last record written for a transaction. It must be forced
 * to disk. It holds no log record in the second buffer, only a header.
 * It's only written if a previous callback didn't put this information
 * in a previous log record.
 *    We also call lock_TranCompleted here to tell the lock service to drop
 * all this transaction's locks.
 */

void RecoveryLog::Finished(Transaction& tran, 
			   TranLogRecord& _tranLogRec)
{
  Boolean alreadyLogged;
  log_status_t logStatus = LOG_SUCCESS;
  tran_tid_t* wholeFamily;
  unsigned int familyCount;
  unsigned int inc;

  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::Finished\n");
  RecRecord(_tranLogRec, &alreadyLogged);
  if (!alreadyLogged)
    {
      _tranLogRec.PrepareToWrite();
      logStatus = WriteToLog(_tranLogRec, LogFile::ForceWrite);
      if (logStatus != LOG_SUCCESS)
	{
	  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2512);
	  Refuse(tran);
	}
    }
  else
    ObjtranReport.Report(ErrorLevels::TraceMsg, "\"Finished\" callback already logged\n");

#ifndef ENCINA_CLIENT_ONLY
  if (useLock)
    {
      tran_TidKnownDescendents(tran, &wholeFamily, &familyCount);
      for (inc = 0; inc < familyCount; inc++)
	lock_TranCompleted(wholeFamily[inc]);
      tran_TidArrayDestroy(wholeFamily);
    }
#endif
  RecCleanup(_tranLogRec);
}

/*
 * WriteCommit
 *    This is called internally when the TRAN optimizations tell us we
 * can write a commit record early.
 */

log_status_t RecoveryLog::WriteCommit(Transaction& tran,
				      TranLogRecord& realRec)

{
  TranLogRecord rec;
  Boolean alreadyLogged;
  tran_recOptimization_t opt;
  log_status_t retVal = LOG_SUCCESS;
  char packedHeader[PackedRecHeaderSize];

  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::WriteCommit\n");
  memset((char*)&opt, '\0', sizeof(tran_recOptimization_t));
  rec.SetupHeader(tran, TranLogRecord::rldtCommit, &opt);
  RecRecord(rec, &alreadyLogged);
  if (!alreadyLogged)
    {	
      rec.Pack((void*)packedHeader);
      realRec.AddBuffer(LogBuffer(packedHeader, PackedRecHeaderSize), false);
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Added \"Commit\" header to log record\n");
    }
  else
    ObjtranReport.Report(ErrorLevels::TraceMsg, "\"Commit\" callback already logged\n");
  return retVal;
}

/*
 * WriteAbort
 *    This is called internally when the TRAN optimizations tell us we
 * can write an abort record early.
 */

log_status_t RecoveryLog::WriteAbort(Transaction& tran,
				     TranLogRecord& realRec)

{
  TranLogRecord rec;
  Boolean alreadyLogged;
  tran_recOptimization_t opt;
  log_status_t retVal = LOG_SUCCESS;
  char packedHeader[PackedRecHeaderSize];

  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::WriteAbort\n");
  memset((char*)&opt, '\0', sizeof(tran_recOptimization_t));
  rec.SetupHeader(tran, TranLogRecord::rldtAbort, &opt);
  RecRecord(rec, &alreadyLogged);
  if (!alreadyLogged)
    {	
      rec.Pack((void*)packedHeader);
      realRec.AddBuffer(LogBuffer(packedHeader, PackedRecHeaderSize), false);
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Added \"Abort\" header to log record\n");
    }
  else
    ObjtranReport.Report(ErrorLevels::TraceMsg, "\"Abort\" callback already logged\n");
  return retVal;
}

/*
 * WriteFinished
 *    This is called internally when the TRAN optimizations tell us we
 * can write an finished record early.
 */

log_status_t RecoveryLog::WriteFinished(Transaction& tran,
					TranLogRecord& realRec)

{
  TranLogRecord rec;
  Boolean alreadyLogged;
  tran_recOptimization_t opt;
  log_status_t retVal = LOG_SUCCESS;
  char packedHeader[PackedRecHeaderSize];

  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::WriteFinished\n");
  memset((char*)&opt, '\0', sizeof(tran_recOptimization_t));
  rec.SetupHeader(tran, TranLogRecord::rldtFinished, &opt);
  RecRecord(rec, &alreadyLogged);
  if (!alreadyLogged)
    {	
      rec.Pack((void*)packedHeader);
      realRec.AddBuffer(LogBuffer(packedHeader, PackedRecHeaderSize), false);
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Added \"Finished\" header to log record\n");
    }	// Don\'t call RecCleanup here, because Finished will be invoked later
  else
    ObjtranReport.Report(ErrorLevels::TraceMsg, "\"Finished\" callback already logged\n");
  return retVal;
}

/*
 * WriteLockRecord
 *    Write a record to the log containing recovery information we got
 * from the lock service. For this to work properly with nested
 * transactions, we must save the lock information for each transaction
 * in the family. The recovery logic should have no problem with this.
 */

unsigned long RecoveryLog::WriteLockRecord(Transaction& tran,
					   TranLogRecord& realRec)

{
#ifdef ENCINA_CLIENT_ONLY
  return 0;
#else
  tran_recOptimization_t opt;
  char lockBuffer[DefaultLockBufferSize];
  void* realLockBuffer;
  unsigned long realLockLen;
  lock_status_t retVal = LOCK_SUCCESS;
  Boolean alreadyLogged;
  char packedHeader[PackedRecHeaderSize];
  tran_tid_t curTid;
  tran_tid_t* wholeFamily;
  unsigned int familyCount;
  unsigned int inc;
  
  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::WriteLockRecord\n");
  memset((char*)&opt, '\0', sizeof(tran_recOptimization_t));
  tran_TidKnownDescendents(tran, &wholeFamily, &familyCount);
  for (inc = 0; inc < familyCount; inc++)
    {
      curTid = wholeFamily[inc];
      retVal = lock_Save(curTid, (void*)lockBuffer, DefaultLockBufferSize,
			 &realLockBuffer, &realLockLen);
      if ((realLockLen > 0) && (retVal == LOCK_SUCCESS))
	{
	  TranLogRecord rec;
	  RecRecord(rec, &alreadyLogged);
	  rec.SetupHeader(tran, TranLogRecord::rldtLockInfo, &opt);
	  rec.PutDataLen(realLockLen);		// So recovery knows
	  rec.Pack((void*)packedHeader);
	  realRec.AddBuffer(LogBuffer(packedHeader, PackedRecHeaderSize), 
			    false);
	  ObjtranReport.VReport(ErrorLevels::TraceMsg,"Writing %li bytes of LOCK data\n", realLockLen);
	  if (realLockLen > DefaultLockBufferSize)
	    {
	      realRec.AddBuffer(LogBuffer(realLockBuffer, realLockLen), false);
	      lock_Free(realLockBuffer);
	    }
	  else
	    realRec.AddBuffer(LogBuffer((void*)lockBuffer, realLockLen), false);
	}
    }
  tran_TidArrayDestroy(wholeFamily);
  return retVal;
#endif
}

/*
 * WriteToLog
 *    Here, we actually write the record to the log file. This means
 * adding the new record to the appropriate chains so we can deal with
 * it later.
 *    If the write to the log file fails, we call tran_RecRefuse. The
 * manual says this is the right thing to do.
 */

log_status_t RecoveryLog::WriteToLog(TranLogRecord& _tranLogRec,
				     LogFile::WriteMode mode)
{  
    // setup chain buffers prior to writing to log
  Boolean closeChain = false;
  tran_tid_t tid = _tranLogRec.GetTopTid();

  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::WriteToLog\n");
  if ((_tranLogRec.GetRecType() == TranLogRecord::rldtFinished) ||
      (_tranLogRec.GetRecOpt().finishImminent == true))
    closeChain = true;
  LogChain	tidChain(&tid, sizeof tid);

    if (closeChain)
      tidChain.SetDoClose(true);
    _tranLogRec.AddChain(tidChain);
//    tid = _tranLogRec.GetSubTid();
//    LogChain	subTidChain(&tid, sizeof tid);
//    if (closeChain)
//      subTidChain.SetDoClose(true);
//    _tranLogRec.AddChain(subTidChain);

    log_status_t logStatus = 
	logFileCtl->Write(_tranLogRec, _tranLogRec.GetRecType(), mode);
    RecChainStart(_tranLogRec);
    if (logStatus == LOG_SUCCESS)
      {
	if (CheckpointNecessary())
	  {
	  ObjtranReport.Report(ErrorLevels::TraceMsg,"In RecoveryLog::WriteToLog -- Calling Checkpoint\n");
	  Checkpoint();
	  }
      }
    return logStatus;
}

/*
 * Restart record format:
 *    The restart record consists of, in this order: four four-byte unsigned
 * integers. The first is the length of the TRAN restart data, and the second
 * is the space allocated for that data. The third is the length of the LOG
 * restart data, and the fourth is the space allocated for that data.
 * All integers are converted to network byte order using ntohl and ntohs.
 * The total length of the header is 16 bytes.
 *    Next, the restart data from TRAN is written verbatim, and the buffer size
 * adjusted accordingly. Finally, the same thing happens to the data from
 * LOG.
 */

/*
 * CheckpointNecessary
 *    Return true if it's time to do a checkpoint.
 */

Boolean RecoveryLog::CheckpointNecessary()

{
  Boolean retVal;
  time_t now = time(NULL);

  restartLatch.Lock();
  writeCount++;
  if ((lastCheckpointTime + timeInterval <= now) ||
      (writeCount >= writeInterval))
    retVal = true;
  else
    retVal = false;
  restartLatch.Unlock();
  ObjtranReport.VReport(ErrorLevels::TraceMsg,"In RecoveryLog::CheckpointNecessary -- %s\n",
			(retVal == true) ? "Yes" : "No");
  return retVal;
}

/*
 * Checkpoint
 *    Do whatever's involved in a checkpoint. Right now, it
 * means setting the tail of the archive, so we do it infrequently.
 */

void RecoveryLog::Checkpoint()

{
  LogSeqNumber* listHead;
  log_lsn_t lsn;
  log_status_t logErr = LOG_SUCCESS;
  
  tableLatch.Lock();
  if (!recovering)		// Don\'t truncate the archive while recovering
    {
      listHead = chainIDs.head();
      if (listHead == (LogSeqNumber*)0)
	{
	  lsn = LOG_HEAD_LSN;		// Here, don\'t include the tail
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 2517, lsn.data[0], lsn.data[1],
				   lsn.data[2], lsn.data[3]);
	  logErr = logFileCtl->TruncateArchive(archiveName, lsn, false);
	}
      else
	{
	  lsn = *listHead;		// Here, we must leave the tail in
	  ObjtranReport.VReportMsg(ErrorLevels::InfoMsg, 2517, lsn.data[0], lsn.data[1],
				   lsn.data[2], lsn.data[3]);
	  logErr = logFileCtl->TruncateArchive(archiveName, lsn, true);
	}
      if (logErr != LOG_SUCCESS)
	ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2518, (const char*)archiveName);
      writeCount = 0;
      lastCheckpointTime = time(NULL);
    }
  tableLatch.Unlock();
}
  
 
/*
 * Handy Constants 
 *    WARNING: The sum of this constants can't be bigger than
 *    LOG_MAX_RESTART_RECORD_LENGTH (512 in 1.0.3)
 */

static const unsigned long RestartHeaderLen = 16;
static const unsigned long InitTRANRestartLen = 128;
static const unsigned long InitLOCKRestartLen = 128;

/*
 * SetTranRestart
 *    Add the data from TRAN to the restart record, then write it out.
 */

log_status_t RecoveryLog::SetTranRestart(const void* newData, 
                                         unsigned long newLen)

{
  Boolean didResize = false;
  unsigned char* tmpLockBuf;
  unsigned long offset;
  unsigned long oldTranRestartBufLen;

  restartLatch.Lock();
  unsigned long bufSize = restartBuf.GetLength();
  oldTranRestartBufLen = tranRestartBufLen;
  while (newLen > tranRestartBufLen)
    {		// Resize the TRAN buffer if necessary
      didResize = true;
      tranRestartBufLen += InitTRANRestartLen;
      bufSize += InitTRANRestartLen;
      restartBuf.Resize(bufSize);
    }
  if (didResize)
    {		// Move the LOCK data to the new location if necessary
      tmpLockBuf = (unsigned char*)malloc((unsigned int)lockRestartLen);
      offset = RestartHeaderLen + oldTranRestartBufLen;	
      // Use memcpy to move LOCK data to a temporary location
      memcpy(tmpLockBuf, restartBuf.GetDataAt(offset), 
	     (unsigned int)lockRestartLen);
      offset = RestartHeaderLen + tranRestartBufLen;
      restartBuf.SetBufAt(offset, tmpLockBuf, lockRestartLen);
      free((char*)tmpLockBuf);
    }
  offset = RestartHeaderLen;
  restartBuf.SetBufAt(offset, newData, newLen);
  tranRestartLen = newLen;
  restartLatch.Unlock();
  return WriteRestart();
}

/*
 * SetLockRestart
 *    Add the data from the LOCK service to the restart record, then write it.
 */

log_status_t RecoveryLog::SetLockRestart(const void* newData, 
					 unsigned long newLen)

{
  unsigned long offset;

  restartLatch.Lock();
  unsigned long bufSize = restartBuf.GetLength();
  while (newLen > lockRestartBufLen)
    {		// Resize the LOCK buffer if necessary
      lockRestartBufLen += InitLOCKRestartLen;
      bufSize += InitLOCKRestartLen;
      restartBuf.Resize(bufSize);
    }
  offset = RestartHeaderLen + tranRestartBufLen;
  restartBuf.SetBufAt(offset, newData, newLen);
  lockRestartLen = newLen;
  restartLatch.Unlock();
  return WriteRestart();
}

/*
 * WriteRestart
 *    Write out the restart record. We're assuming the TRAN and LOCK data
 * has already been placed there. Here, we write the header and then put
 * the whole thing to the actual logfile.
 */

log_status_t RecoveryLog::WriteRestart()

{
  idl_ulong_int tmpLong;
  unsigned char restartData[RestartHeaderLen];
  unsigned char* pos = restartData;
  log_status_t retVal = LOG_SUCCESS;

  // Put the header info in place
  restartLatch.Lock();
  tmpLong = htonl(tranRestartLen);
  memcpy(pos, (char*)&tmpLong, 4);
  pos += 4;
  tmpLong = htonl(tranRestartBufLen);
  memcpy(pos, (char*)&tmpLong, 4);
  pos += 4;
  tmpLong = htonl(lockRestartLen);
  memcpy(pos, (char*)&tmpLong, 4);
  pos += 4;
  tmpLong = htonl(lockRestartBufLen);
  memcpy(pos, (char*)&tmpLong, 4);
  restartBuf.SetBufAt(0, restartData, RestartHeaderLen);
  restartBuf.SetDataLen(RestartHeaderLen + tranRestartBufLen +
			lockRestartBufLen);

  ObjtranReport.VReport(ErrorLevels::TraceMsg, "Writing TRAN restart: TRAN: %li/%li bytes\n",
			tranRestartLen, tranRestartBufLen);
  ObjtranReport.VReport(ErrorLevels::TraceMsg, "                      LOCK: %li/%li bytes\n",
			lockRestartLen, lockRestartBufLen);
  ObjtranReport.VReport(ErrorLevels::TraceMsg, "                      total: %li bytes\n",
			restartBuf.GetDataLen());
  
  if (retVal == LOG_SUCCESS)		// Don\'t write if couldn\'t set tail
    {
      retVal = logFileCtl->WriteRestart(restartBuf);
      if (retVal == LOG_SUCCESS)
	{
	  lastCheckpointTime = time(NULL);
	  writeCount = 0;
	}
    }	
  restartLatch.Unlock();
  return retVal;
}

/*
 * ReadRestart
 *    Read the restart record. This is done at recovery time. That's
 * why it's called the "restart record".
 */

log_status_t RecoveryLog::ReadRestart()

{
  log_status_t logErr;
  unsigned long tranErr;
  char* pos;
  idl_ulong_int tmpLong;
  tran_logRecord_t tranRec;

  restartLatch.Lock();
  logErr = logFileCtl->ReadRestart(restartBuf);
  if ((logErr == LOG_NO_RECORD) || 
      ((logErr == LOG_SUCCESS) && (restartBuf.GetDataLen() == 0)))
    {		// Initialize a new restart record, because there wasn\'t one
      restartBuf.Resize(RestartHeaderLen + 
			InitTRANRestartLen + InitLOCKRestartLen);
      tranRestartLen = 0;
      tranRestartBufLen = InitTRANRestartLen;
      lockRestartLen = 0;
      lockRestartBufLen = InitLOCKRestartLen;
      restartLatch.Unlock();
      return LOG_SUCCESS;
    }
  else if (logErr != LOG_SUCCESS)
    {
      restartLatch.Unlock();
      return logErr;
    }
  else
    {		// Unpack the header and replay the records
      pos = (char*)(restartBuf.GetData());
      memcpy((char*)&tmpLong, pos, 4);
      tranRestartLen = ntohl(tmpLong);
      pos += 4;
      memcpy((char*)&tmpLong, pos, 4);
      tranRestartBufLen = ntohl(tmpLong);
      pos += 4;
      memcpy((char*)&tmpLong, pos, 4);
      lockRestartLen = ntohl(tmpLong);
      pos += 4;
      memcpy((char*)&tmpLong, pos, 4);
      lockRestartBufLen = ntohl(tmpLong);
      

      ObjtranReport.VReport(ErrorLevels::TraceMsg, "Read TRAN restart: TRAN: %li/%li bytes\n",
			    tranRestartLen, tranRestartBufLen);
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "                   LOCK: %li/%li bytes\n",
			    lockRestartLen, lockRestartBufLen);
      ObjtranReport.VReport(ErrorLevels::TraceMsg, "                   total: %li bytes\n",
			    restartBuf.GetDataLen());


      // Replay the TRAN restart record
      pos = (char*)(restartBuf.GetDataAt(RestartHeaderLen));
      tranRec = tran_LogRecordCreate(pos, tranRestartLen);
      tranErr = tran_RecReplay(serviceRep, TRAN_TID_NULL, tranRec);
      tran_LogRecordDestroy(tranRec);
      if (tranErr != TRAN_SUCCESS)
	{
	  PrintTranMessage(tranErr, ErrorLevels::ErrorMsg);
	  restartLatch.Unlock();
	  return LOG_FATAL_ERROR;
	}
      else
	{
	  restartLatch.Unlock();
	  return LOG_SUCCESS;
	}
    }
}

/*
 * LogFileCallback
 *    This callback is set off by the LOG service when something happens
 * to the log file. In our case, it tells us to do a checkpoint.
 */

void RecoveryLog::LogFileCallback(LogFileUser::LogCommand command,
				  const String&, const LogSeqNumber&,
				  int logError)
{
  if (logError != 0)
    ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2506, logError);
  if ((command == LogFileUser::FileFillingUp) ||
      (command == LogFileUser::ArchiveImminent))
    {
      ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2507);
      Checkpoint();
    }
}

/*
 * LockCallback
 *    This callback is called by LOCK when it wants us to log some
 * restart data. We also use this to tell us when to write the restart record.
 * This call is a friend of the RecoveryLog class.
 */

void LockCallback(void* data, unsigned int length)

{
  RecoveryLog* theLog = RecoveryLog::Instance();
  if (theLog != (RecoveryLog*)0)
    {
      ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2509);
      theLog->SetLockRestart(data, length);
    }
}

/*
 * Replay
 *    Do recovery. Read the restart record to determine where to start
 * reading, call ReadLog to deal with the records and headers, then
 * replay each record to the TRAN service.
 */

void RecoveryLog::Replay()

{
    //
    //	go thru log records for this xa service
    //
    const String   archive;
    LogBuffer restartBytes;
    List<LogChain> cursorChains;
    log_status_t err;
    unsigned long lockErr;

    tableLatch.Lock();
    recovering = true;
    tableLatch.Unlock();
    // Read the restart data
    err = ReadRestart();
    if (err != LOG_SUCCESS)
      {
	ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 2504);
	ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
	exit(-1);
      }

#ifndef ENCINA_CLIENT_ONLY
    if (useLock)		// Initialize the LOCK service
      {
	if (lockRestartLen == 0)
	  lockErr = lock_Init((void*)0, 0, LockCallback);
	else
	  {
	    restartLatch.Lock();
	    const void* lockData = 
	      restartBuf.GetDataAt(RestartHeaderLen + tranRestartBufLen);
	    restartLatch.Unlock();	
	    lockErr = lock_Init((void*)lockData, (unsigned int)lockRestartLen, 
				LockCallback);
	  }
	if (lockErr != LOCK_SUCCESS)
	  {
	    PrintTranMessage(lockErr, ErrorLevels::ErrorMsg);
	    ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 2508);
	    ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);
	    exit(-1);
	  }
	else
	  {
/****
	    int didInit = 0;
	    admin_RegisterInterface(admin_lock_v1_0_c_ifspec, 0,
				    "admin_lock", &didInit);
*****/
	  }
      }
#endif

    // set the log cursor for reading back the recovery log records
    LogCursor recCursor(archiveName, cursorChains,
			(LogSeqNumber*)0, (LogSeqNumber*)0, *logFileCtl);
    while (ReadLog(recCursor) != LOG_END_OF_CURSOR);

    tableLatch.Lock();
    recovering = false;
    tableLatch.Unlock();
  }

/*
 * ReadLog
 *    This is called for each record that's being read back from the
 * log file. It deals with the header and makes sure the right
 * data is placed in the TRAN Log record.
 *    In this function, we read the log record back, then decide what to do
 * with it based on the type of record. For commit and abort records, we
 * play back the special TRAN log records for this purpose. For prepare and
 * write records, we play back the data written to the log record by our
 * code (which is really a TRAN log record). For finished records (if we
 * get any, which we shouldn't) we play back an empty LOG record, since that's
 * what the manual says to do. We don't play back LOG records for "active"
 * callbacks, and there shouldn't be any.
 *    Records containing LOCK service data are also played back, but in this
 * case they're played back to the LOCK service rather than to TRAN.
 * This ought to work.
 *    We shouldn't get any other types of LOG records.
 */

log_status_t RecoveryLog::ReadLog(LogCursor& recCursor)

{
    log_status_t    logStatus = LOG_SUCCESS;
    log_recordType_t cursorRecType;
    TranLogRecord::RecLogDataType_t recDataType;
    tran_tid_t tid;
    tran_logRecord_t newLogRec;
    char * bufDataP;
    unsigned long lockErr;
    TranLogRecord tranLogRec;
    TranLogRecord tranLogHeader;
    unsigned long pos = 0;
    unsigned long totalLen;
    unsigned long bufLen;

    logStatus = recCursor.Read(tranLogRec, cursorRecType);
    LogBuffer theBuffer = tranLogRec.GetBuffer();
    totalLen = theBuffer.GetDataLen();
    while ((logStatus == LOG_SUCCESS) && (pos < (totalLen - 1)))
      {
	if (LOG_SUCCESS == logStatus)
	  { 
	    // set the TranLogRecord header from the beginning of the
	    // log record;
	    if (theBuffer.GetDataLen() < PackedRecHeaderSize)
	      {
		logStatus = LOG_FATAL_ERROR;
	      }
	    else
	      {
		// Get the header from the first buffer
		tranLogHeader.Unpack(theBuffer.GetDataAt(pos));
		pos += PackedRecHeaderSize;
		tid = tranLogHeader.GetTopTid();
		recDataType = tranLogHeader.GetRecType();

		// based upon the type of log record, a canned log record 
		// may have to be returned to the recovery routine.
		switch(recDataType)
		  {
		  case TranLogRecord::rldtCommit:
		    { 
		      Recoverable::Replay(tid, TRAN_LOG_RECORD_COMMIT);
		      ObjtranReport.VReport(ErrorLevels::TraceMsg,
					   "TID %lu: Replaying \"%s\" record\n", tid, "commit");
		      break; 
		    }
		  case TranLogRecord::rldtAbort:
		    { 
		      Recoverable::Replay(tid, TRAN_LOG_RECORD_ABORT);
		      ObjtranReport.VReport(ErrorLevels::TraceMsg,
					   "TID %lu: Replaying \"%s\" record\n", tid, "abort");
		      break; 
		    }
		  case TranLogRecord::rldtWrite:
		  case TranLogRecord::rldtPrepare:
		    { 
 		      // Get the actual TRAN data
		      bufDataP = (char*)theBuffer.GetDataAt(pos);
		      bufLen = tranLogHeader.GetDataLen();
		      pos += bufLen;
		      if (bufLen == 0)
			{
			  ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2513);
			  logStatus = LOG_FATAL_ERROR;
			}
		      else
			{
			  newLogRec = tran_LogRecordCreate(bufDataP, bufLen);
			  Recoverable::Replay(tid, newLogRec);
			  tran_LogRecordDestroy(newLogRec);
			  if (recDataType == TranLogRecord::rldtWrite)
			    ObjtranReport.VReport(ErrorLevels::TraceMsg,
						 "TID %lu: Replaying \"%s\" record\n", 
						 tid, "write");
			  else
			    ObjtranReport.VReport(ErrorLevels::TraceMsg,
						  "TID %lu: Replaying \"%s\" record\n", 	
						  tid, "prepare");
			  ObjtranReport.VReport(ErrorLevels::TraceMsg,
						"Replayed %li bytes of TRAN data\n",
						bufLen);
			}
		      break; 
		    }
		  case TranLogRecord::rldtLockInfo:
		    {
#ifndef ENCINA_CLIENT_ONLY
		      if (useLock)
			{
 			  // Get the actual lock data
			  bufDataP = (char*)theBuffer.GetDataAt(pos);
			  bufLen = tranLogHeader.GetDataLen();

			  pos += bufLen;
			  if (bufLen == 0)
			    {
			      ObjtranReport.ReportMsg(ErrorLevels::InfoMsg, 2514);
			      logStatus = LOG_FATAL_ERROR;
			    }
			  else
			    {
			      lockErr = lock_Restore(tid, bufDataP, bufLen);
			      ObjtranReport.VReport(ErrorLevels::TraceMsg,
						    "TID %lu: Replaying \"%s\" record\n",
						    tid, "lock data");
			      if (lockErr != LOCK_SUCCESS)
				PrintTranMessage(lockErr, ErrorLevels::ErrorMsg);
			      ObjtranReport.VReport(ErrorLevels::TraceMsg,
						    "Replayed %li bytes of LOCK data\n", bufLen);
			    }
			}
		      else
			ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 2511);
#endif
		      break;
		    }
		  case TranLogRecord::rldtFinished:
		    {
		      newLogRec = tran_LogRecordCreate((void*)0, 0);
		      Recoverable::Replay(tid, newLogRec);
		      tran_LogRecordDestroy(newLogRec);
		      ObjtranReport.VReport(ErrorLevels::TraceMsg,
					    "TID %lu: Replaying \"finished\" record\n", tid);
		      break;
		    }
		  case TranLogRecord::rldtNull:
		  case TranLogRecord::rldtIsActive:
		  case TranLogRecord::rldtNotActive:
		    break;
		  }
	      }
	  }
      }
    return logStatus;
}

unsigned long RecoveryLog::Initialize()

{

    ObjtranReport.Report(ErrorLevels::TraceMsg, "In RecoveryLog::Initialize\n");
    unsigned long retVal = Recoverable::Initialize();
    ObjtranReport.VReport(ErrorLevels::TraceMsg,
			  "Instantiating LogFile %s\n", (char*)(const char *)logName);
    logFileCtl = new LogFile(logName, logAuthn, this);
    
    // Setup the Archive to archive only open chains
    log_status_t logStatus;
    logStatus = logFileCtl->SetFilter(archiveName,
				      "(AnyChainOpen())"); 

    // replay all open log chains for incomplete transactions
    Replay();
    return retVal;
}
