/*
 * tran_transaction.C
 *    This module lets us separate Encina Monitor-specific functionality from
 * plain TRAN functionality. Here, we implement member functions of the
 * transaction class that will be different depending on whether they
 * are being used by the Encina Monitor or by objtran by itself. We
 * also implement callback functions here, which are very dependent
 * on which environment they're using.
 *
 * tran_transaction.C,v 1.12 1994/01/18 17:11:36 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/tranaux.H>
#include <objdce/objtranparams.H>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <objtran/transaction.H>
#include <objtran/tranproperty.H>
#include <objtran/tranmember.H>
#include <objdce/abortreason.H>
#include <objdce/error.H>
#include <String.h>
#include <objdce/mutex.H>
#include <objtran/tranclient.H>
#include <objtran/trantimeout.H>
#include <objtran/rpctrancheckstatus.H>
extern "C"
{
#include <pthread.h>
#include <threadTid/threadTid.h>
}
#include <objdce/defs.H>
#if HAVE_MALLOC_H
#include <malloc.h>
#endif

extern tran_propertyKey_t tc_abortReasonKey;

void MonAbortCall(void*, unsigned long, unsigned long)

{
}

void MonFinishCall(void*, unsigned long, unsigned long)

{
}

/*
 * Commit:
 *    Here, we must be sure to drop any locks we may have acquired
 * and commit the transaction, plus pop the tid from the thread.
 */

void Transaction::Commit()

{
    String errMsg;
    tran_status_t err;

    ObjtranReport.VReport(ErrorLevels::TraceMsg, "About to Commit Transaction: %lu\n", tranRep);
    myEntry->Latch();
    Boolean isPushed = myEntry->IsThreadPushed();
    if (!myEntry->CheckFlag(TXN_ABORTED_OR_COMMITTED))
    {
        myEntry->SetFlag(TXN_COMMITTED);
	EndTimeout(myEntry);
	if (isPushed)
	  {
	    if (ObjtranReport.Active(ErrorLevels::TraceMsg))
	      {
		tran_tid_t tmpTid = threadTid_Lookup();
		ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Popped from thread\n", tmpTid);
	      }
	    myEntry->Unlatch();
	    threadTid_End();	    // Only here, Abort and in UpdateState
	    err = tran_End(tranRep);
	    myEntry->Latch();
	  }
	else
	  {
	    myEntry->Unlatch();
	    err = tran_End(tranRep);
	    myEntry->Latch();
	  }
	if ((err != TRAN_SUCCESS) && !myEntry->CheckFlag(TXN_ABORTED))
	  {
	    String  aString;

	    ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2050, tranRep);
	    GetTranStatus(aString, err, true, ErrorLevels::FatalMsg);
	  } 
	else 
	  {
	    if (myEntry->CheckFlag(TXN_ABORTED))
		{
		ObjtranReport.VReport(ErrorLevels::TraceMsg,
				      "TID %lu: Aborted while committing\n",
				      tranRep);
		}
	    else
		{
		ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Committed\n", tranRep);
		}
	  }
      }
    myEntry->Unlatch();
}

/*
 * Abort:
 *    Using the AbortReason object passed us, we set the abort properties
 * of the transaction so we can get it later. Also, pop the tid from the
 * thread. Again, we have to do some brouhaha to see what happens when one
 * thread aborts a transaction started in another.
 */

void Transaction::Abort(const AbortReason& reason)

{
  String codeTxt;
  String errMsg;
  void* dataBytes;
  tran_status_t err = TRAN_SUCCESS;
  TransactionProperty keyVal;
  TransactionProperty dataVal;
  int dataLen;

  ObjtranReport.Report(ErrorLevels::TraceMsg, "Aborting Transaction...\n");
  myEntry->Latch();
  Boolean isPushed = myEntry->IsThreadPushed();
  if (!myEntry->CheckFlag(TXN_ABORTED))
    {
      if (myEntry->GetAbortReason() == (AbortReason*)0)
	{
	  AbortReason* tmpReason = new AbortReason(reason);
	  myEntry->SetAbortReason(tmpReason);
	 
	  dataLen = reason.PackStringLength();
	  dataBytes = (void*)malloc(dataLen + 1);
	  reason.PackString(dataBytes);
	  keyVal.SetKey(TRAN_PROPERTY_KEY_ABORT_FORMAT);
	  keyVal.SetValue(ReasonName);
	  AddProperty(keyVal);
	  dataVal.SetKey(ReasonName);
	  dataVal.SetValue(dataBytes, dataLen);
	  AddProperty(dataVal);
	  free((char*)dataBytes);

	  dataLen = reason.PackLength();
	  dataBytes = (void*)malloc(dataLen + 1);
	  reason.Pack(dataBytes);
	  dataVal.SetKey(MoreReasonName);
	  dataVal.SetValue(dataBytes, dataLen);
	  AddProperty(dataVal);
	  free((char*)dataBytes);
	}

	if (isPushed)
	  {
	    if (ObjtranReport.Active(ErrorLevels::TraceMsg))
	      {
		tran_tid_t tmpTid = threadTid_Lookup();
		ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Popped from thread\n", tmpTid);
	      }
	    myEntry->Unlatch();
	    threadTid_End();	// here, Commit, UpdateState only
	    myEntry->Latch();
	  }
	if (!myEntry->CheckFlag(TXN_ABORTED))
	  {
	    myEntry->SetFlag(TXN_ABORTED);
	    myEntry->ClearFlag(TXN_COMMITTED);
	    myEntry->Unlatch();
	    err = tran_Abort(tranRep);
	    myEntry->Latch();
	  }
	if (err != TRAN_SUCCESS)
	  {
	    ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2051, tranRep);
	    GetTranStatus(codeTxt, err, true, ErrorLevels::FatalMsg);
	  }
    }
  myEntry->Unlatch();
}

void Transaction::Abort(const String& text, const int code, const int applNo)

{
    AbortReason reason(text, code, applNo);
    Abort(reason);
}

/*
 * UpdateState:
 *    If this transaction is going away and we've pushed the tid onto this
 * thread, then pop it here.
 *    The latch on the TranEntry object must be locked before we call this. 
 */

void Transaction::UpdateState()

{
  if (ObjtranReport.Active(ErrorLevels::TraceMsg))
    {
      ObjtranReport.VReport(ErrorLevels::TraceMsg,
			    "TID %lu: TID %lu pushed on thread\n",
			    tranRep, threadTid_Lookup());
    }
  Boolean pushed = myEntry->IsThreadPushed();
  if (myEntry->CheckFlag(TXN_ABORTED_OR_COMMITTED))
    if (pushed)	    		    // Timing is critical: Someone else may
      {			    	    // Call this while we pop the tid
	if (ObjtranReport.Active(ErrorLevels::TraceMsg))
	  {
	    tran_tid_t tmpTid = threadTid_Lookup();
	    ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Popped from thread\n", tmpTid);
	  }
	myEntry->Unlatch();
	threadTid_End();
	myEntry->Latch();
      }
}

/*
 * InitNewTran
 *    This function takes care of the work that must be done to begin a 
 * new TRAN transaction. It starts the transaction, calls SetUp to maintain
 * the transaction hash table, and registers the callbacks if necessary.
 * It also decides whether to request ephemeral or distributed outcome,
 * and sets up the transaction timeout.
 */

void NewTransaction::InitNewTran(const int timeout)

{
  tran_tid_t parent = threadTid_Lookup();	// This is set by a TRPC callback
  tran_status_t err;
  Boolean distributedOutcome = true;

  if (ObjtranMainMode == 0)
    {
      ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 2063);	// Not initialized
      ObjtranReport.ReportMsg(ErrorLevels::FatalMsg, 900);	// Aborting
      abort();
    }
  err = tran_Begin(parent, &tranRep);
  if (err != TRAN_SUCCESS)
    {
      String errMsg;

      ObjtranReport.VReportMsg(ErrorLevels::FatalMsg, 2061);
      GetTranStatus(errMsg, err, true, ErrorLevels::FatalMsg);
    }
  else
    {
      threadTid_Begin(tranRep);
      SetUp();			    // Register object and callbacks
      if (ObjtranMainMode & OBJTRAN_RECOVERABLE)
	{
	  distributedOutcome = false;
	}
      if (distributedOutcome)
	{
	  RequireDistributedOutcome(true);
	  SetEphemeralOutcomeRequirementLimit(defaultOutcomeRequirement);
	}
      else
	SetEphemeralOutcomeDeliveryLimit(defaultOutcomeDelivery);
      if (parent != TRAN_TID_NULL)
	HoldEntry();		// Register "Finished" callback for nested tran
      if (DefaultReport.Active(ErrorLevels::TraceMsg))
	{
	  if (parent == TRAN_TID_NULL)	    // Non-nested tran
	    {
	      if (distributedOutcome)
		ObjtranReport.VReport(ErrorLevels::TraceMsg, 
		      "TID %lu: Started transaction w/distributed outcome\n", 
				      tranRep);
	      else
		ObjtranReport.VReport(ErrorLevels::TraceMsg, 
				      "TID %lu: Started transaction\n", 
				      tranRep);
	    }
	  else				    // Nested tran
	    {
	      if (distributedOutcome)
		ObjtranReport.VReport(ErrorLevels::TraceMsg,
	      "TID %lu: Started nested transaction w/distributed outcome\n",
			      tranRep);
		ObjtranReport.VReport(ErrorLevels::TraceMsg,
        	  "TID %lu: Started nested transaction \n",
				  tranRep);
	    }
	  ObjtranReport.VReport(ErrorLevels::TraceMsg, "TID %lu: Pushed on thread\n", tranRep);
	}
#ifndef NO_TRAN_RANDOM_KEY
      securityKey = tran_SecurityKeyCreate((void*)&secureUUID, sizeof(uuid_t));
      tran_Secure(tranRep, securityKey);
#endif
      if (timeout > 0)
	TranAppl::GetTranTimer()->PushTran(myEntry, timeout);
    }
}

/*
 * RegisterBasicCallbacks
 *    Register the callback functions that must be called for every
 * transaction. In our case, this is just a pre-abort callback.
 */

void Transaction::RegisterBasicCallbacks()

{
  tran_CallBeforeAbort(tranRep, AbortCall, (void*)myEntry);
}  

/*
 * EndTimeout
 *    Clear this transaction from the transaction timeout list.
 */

void Transaction::EndTimeout(TranEntry* entry)

{
  TranAppl::GetTranTimer()->RemoveTran(entry);
}
