/*
 * tranaux.H:
 *
 *    These classes are only used internally to the Transaction class. The
 * TranEntry and TranObj classes are used to implement the internal table
 * of active transactions, while the AbortData class is used to pass information
 * on transaction abort reasons around.
 *    NOTE: The TranEntry class does not do locking on its own. Any code that
 * expects to share a TranEntry object between threads (like the Transaction
 * class) MUST lock the object using the "Latch" function, then call "Unlatch"
 * when its done.
 *    Here, we also define the TranEntryNode object, which is an entry in
 * the global hash table of transactions. This is simply a structure with a
 * tran_tid_t and a pointer to a TranEntry. This way, the Set class can "move"
 * the TranEntryNode without affecting the location of the TranEntry. Thus,
 * the Transaction class can keep a pointer to the TranEntry without fear.
 *
 * tranaux.H,v 1.11 1994/04/07 19:22:19 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRAN_AUX_H
#define TRAN_AUX_H

extern "C"
{
#include <pthread.h>
#include <tran/tran.h>
#include <threadTid/threadTid.h>
}
#include <objdce/defs.H>
#include <sys/types.h>
#include <sys/time.h>
#include <iostream.h>
#include <String.h>
#include <objdce/objtranparams.H>
#include <objdce/mutex.H>
#include <Path.h>

/*
 * Transaction state flags: These define the state of a transaction, and are
 * stored in each TranEntry object. These should be externs, but in this
 * case I'm making them static for efficiency's sake.
 */

static const int TXN_ABORTED =			0x0001;
static const int TXN_COMMITTED =		0x0002;
static const int TXN_ABORTED_OR_COMMITTED =	TXN_ABORTED | TXN_COMMITTED;
static const int TXN_SUSPENDED =		0x0008;
static const int TXN_GOT_LOCK =			0x0010;
static const int TXN_HOLD_ENTRY =		0x0020;
static const int TXN_VIA_TRPC =			0x0040;
static const int TXN_MON_LOCAL_END = 		0x0080;

class Transaction;
class AbortReason;
class MutexLock;
class Condition;
class SFSBatchParam;

class TranEntry

{
    friend ostream& operator << (ostream&, TranEntry&);
public:
    TranEntry();
    TranEntry(tran_tid_t);
    TranEntry(const TranEntry&);
    ~TranEntry();
    TranEntry& operator = (const TranEntry&);
    Boolean operator == (const TranEntry&) const;
    Boolean operator < (const TranEntry&) const;
    Boolean operator > (const TranEntry&) const;
    void AddTranObj();	    	    // Add transaction to list
    Boolean InvalidateTranObj(Boolean finished);    // Mark it invalid
    void SetFlag(const unsigned int);	    // Set, clear, check flags
    void ClearFlag(const unsigned int);
    Boolean CheckFlag(const unsigned int) const;
    const AbortReason* GetAbortReason() const; 
    AbortReason* GetAbortReason(); 
    void SetAbortReason(AbortReason*);
    Boolean IsThreadPushed() const;	    	    // Is this thread pushed
    void Latch();			    // Lock and unlock mutex
    void Unlatch();
    operator tran_tid_t() const;		    // Conversion
private:
    tran_tid_t tid;		    // The transaction
    unsigned long validObjs;  // number of rhose objects that are valid
    AbortReason* abortReason; // The reason we aborted
    unsigned int tranFlags;  // transaction state flags set by callbacks
    MutexLock latch;	    // Lock for the whole object
};

class TranEntryNode

{
public:
  TranEntryNode();
  TranEntryNode(tran_tid_t tid, TranEntry* entry = (TranEntry*)0);
  TranEntryNode(const TranEntryNode&);
  TranEntryNode& operator = (const TranEntryNode&);
  int operator == (const TranEntryNode&) const;

  tran_tid_t tid;
  TranEntry* entry;
};

class RandomKey

{
public:
    long rand1;
    long rand2;
};

inline TranEntry::operator tran_tid_t() const

{ return tid; }

inline void TranEntry::SetFlag(const unsigned int flag) 

{ tranFlags |= flag; }

inline void TranEntry::ClearFlag(const unsigned int flag) 

{ tranFlags &= ~flag; }

inline Boolean TranEntry::CheckFlag(const unsigned int flag) const 

{ return (tranFlags & flag); }	

inline const AbortReason* TranEntry::GetAbortReason() const 

{ return abortReason; }

inline AbortReason* TranEntry::GetAbortReason()

{ return abortReason; }

inline void TranEntry::SetAbortReason(AbortReason* reason) 

{ abortReason = reason; }

inline Boolean TranEntry::IsThreadPushed() const

{ return (threadTid_Lookup() == tid); }	

inline void TranEntry::Latch() 

{ latch.Lock(); }

inline void TranEntry::Unlatch() 

{ latch.Unlock(); }

inline void TranEntry::AddTranObj() 
     
{ validObjs++; }

inline int TranEntryNode::operator == (const TranEntryNode& n) const

{ return (tran_TidEqual(tid, n.tid)); }

#endif
