/*
 * Transaction, ExistingTransaction, NewTransaction
 *
 *    The Transaction class represents a single transaction. When a
 * NewTransaction object is created, a new transaction is begun. When an
 * ExistingTransaction is created, it refers to the currently executing
 * transaction. Transaction objects should not be reused. They can be used
 * throughout the program to control distributed transactions.
 *
 * transaction.H,v 1.14 1994/04/07 19:22:37 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef TRANSACTION_H
#define TRANSACTION_H

/* 
 * Constants 
 */

#define ReasonName "C++Reason"
#define MoreReasonName "C++ExtendedReason"

extern "C"
{
#include <pthread.h>
#include <tran/tran.h>
#include <trpc/trpc.h>
}
#include <iostream.h>
#include <objdce/error.H>
#ifndef ENCINA_CLIENT_ONLY
#include <objtran/lock.H>
#endif
#include <String.h>
#include <objdce/objtranparams.H>
#include <objdce/bdemem.H>
#include <objdce/defs.H>
#include <objtran/tranaux.H>
#include <Path.h>
#include <Set.h>

class TransactionMember;
class TransactionProperty;
class AbortReason;
class ThreadInfo;
class MutexLock;
class TranThread;

class Transaction
 
{
friend void AbortCall(tran_tid_t, void*);
friend void FinishCall(tran_tid_t, void*);
friend void MonAbortCall(void*, unsigned long, unsigned long);
friend void MonFinishCall(void*, unsigned long, unsigned long);
friend void SendReplyCallback(rpc_binding_handle_t, void*, 
			      trpc_tranInfo_t*, trpc_ifSpec_t*);
friend void* TranRunner(void*);
friend class TranEntry;

protected:
    Transaction();			    // Refers to current trans.
public:
    virtual ~Transaction();			// Clean up whatever
    void Commit();				// Commit this transaction
    void Abort(const AbortReason&);		// Abort this transaction
    void Abort(const String& text, const int code, 
	       const int applNo = 0);		// Abort using different reason
    void Suspend();				// Suspend it
    void Resume();				// Resume if suspended
    void Register(TransactionMember*, 
	          unsigned int regFlags);	// Register callbacks
    const AbortReason* GetAbortReason() const;  // Find out why we aborted
    void SetAbortReason(AbortReason*);		// Set why
#ifndef ENCINA_CLIENT_ONLY
    void AcquireLock(const LockMode, 
		     const LockName&,
		     const LockSpace* = nil);	// Acquire transactional lock
#endif
/**** Information functions ****/
    Boolean Aborted();				// Did we abort?
    Boolean Committed();			// Did we commit?
    Boolean AbortedOrCommitted();		// Did we do either?
    Boolean IsTopLevel() const;			// Are we a top-level transaction?
    Boolean IsRelated(const Transaction&) const;
    Boolean IsOurParent(const Transaction&) const;
    Boolean IsOurChild(const Transaction&) const;
/**** Utility functions ****/
    static void AbortAll();			// Abort all transactions
    Boolean AddProperty(const TransactionProperty&);    // Add property
    Boolean GetProperty(const String& propName, TransactionProperty*) const;
    Boolean GetProperty(const tran_propertyKey_t, TransactionProperty*) const;
    void GetParent(Transaction*) const;		// Return parent transaction
    void GetTopAncestor(Transaction*) const;	// Create and get top-level parent
    Transaction** GetAllChildren() const;	// Get null-terminated list
/**** Advanced transaction control functions ****/
    static void SetDefaultOutcomeRequirement(unsigned long seconds);
    static void SetDefaultOutcomeDelivery(unsigned long seconds);
    static unsigned long GetDefaultOutcomeRequirement();
    static unsigned long GetDefaultOutcomeDelivery();
    Boolean PrePrepare();		    	// These match similarly-named
    Boolean RequireDistributedOutcome(Boolean);	//     TRAN functions
    Boolean RequireCompleteOutcome(Boolean);
    Boolean SetEphemeralOutcomeRequirementLimit(unsigned long microseconds);
    Boolean SetEphemeralOutcomeDeliveryLimit(unsigned long microseconds);
    Boolean DelayAbort(Boolean);
    Boolean ProlongResolution(Boolean);
    Boolean RequestPromptFinish();
/**** Operators ****/
    int operator == (const Transaction&) const;
    int operator != (const Transaction&) const;
    operator String() const;
    operator tran_tid_t() const;
/**** Internal operations -- Users shouldn't call these ****/
    Boolean ReadAbortProperties();	    // Translate props into AbortReason
    unsigned long Hash() const; 	    // Hash transaction ID
    TranEntry* GetMyEntry();		    // Get my TranEntry
    void HoldEntry();			    // Prolong TranEntry lifetime
    void HoldEntryViaTRPC();		    // Prolong TranEntry lifetime
    static void InitTranTable();	    // Initialize tranTable
    static void InitRandom();		    // Init random value
    static void PrintTidTable();	    // Print out tranTable
    static unsigned long NumActive();	    // Number of active transactions
protected:
    Transaction& operator = (const tran_tid_t);	// For internal use
    static TranEntry* Find(const tran_tid_t);
    static void ClearTran(const tran_tid_t); // Clear from tranTable
    static void EndTimeout(TranEntry*);	    // Clean from timeout list
    void UpdateState();	    		    // Update thread-specific stats info.
    Boolean StillOK();			    // Have we aborted yet?
    void SetUp();			    // Set up state table & callbacks
    void RegisterBasicCallbacks();	    // Called by SetUp

    static Set<TranEntryNode>* tranTable;   // Hash table of transaction info
    static MutexLock* tranTableLatch;	    // Latch on that table
    static uuid_t secureUUID;		    // UUID security key
    static volatile unsigned long activeCount;   // Real count of active trans
    static unsigned long defaultOutcomeRequirement;   // outcome requirement
    static unsigned long defaultOutcomeDelivery;      // outcome delivery
    tran_tid_t tranRep;			    // The actual TRAN transaction ID
    TranEntry* myEntry;			    // Pointer to my entry in tranTable
};

class NewTransaction : public Transaction

{
public:
    NewTransaction();			    // Start new (perhaps nested) tran
    NewTransaction(const int timeout);	    // Start and set timeout in seconds
    virtual ~NewTransaction();		    // Abort if necessary
private:
    void InitNewTran(const int timeout);    // Do more construction
    tran_securityKey_t securityKey;	    // The security key
};

class ExistingTransaction : public Transaction

{
public:
    ExistingTransaction();		    // Create object for current tran
    ExistingTransaction(const tran_tid_t);  // Refer to named transaction
    virtual ~ExistingTransaction();
};

/*
 * Functions you can call from the debugger
 */

extern "C" void ObjtranPrintTransactionTable();

/*
 * Inline functions
 */

inline Transaction::operator tran_tid_t() const

{ return tranRep; }

inline Transaction& Transaction::operator = (const tran_tid_t tid)

{ tranRep = tid;  return *this; }

inline unsigned long Transaction::Hash() const

{ return tran_TidHash(tranRep); }

inline TranEntry* Transaction::GetMyEntry()

{ return myEntry; }

inline unsigned long Transaction::GetDefaultOutcomeDelivery()
     
{ return defaultOutcomeDelivery; }

inline unsigned long Transaction::GetDefaultOutcomeRequirement()

{ return defaultOutcomeRequirement; }

#endif


