/*
 * trantimeout.C,v 1.11 1994/04/07 19:22:43 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objtran/trantimeout.H>
#include <objtran/tranaux.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objdce/mutex.H>
#include <objdce/condition.H>
#include <Block.h>
#include <objdce/error.H>
#include <string.h>
#include <iostream.h>
#include <limits.h>
#include <objdce/defs.H>

#ifdef _AIX
#include <Map.c>
#include <Block.c>
#pragma define (Map<tran_tid_t, time_t>)
#pragma define (Block<ExistingTransaction*>)
#endif

/*
 * TimeoutThread:
 *    This object manages a linked list of TranEntry objects that will be
 * used to abort transactions when they time out.
 */

TimeoutThread::TimeoutThread() : ThreadObject(), latch(MutexLock::FAST)

{
    String reas("Transaction timed out");	// Doing this explicitly
    String file (__FILE__);			// rather than via auto.
    String func("TimeoutThread::Run()");	// temporaries works better
    String appl("");

    timedOutReason = new AbortReason(reas, 2, ObjtranID, file, func, appl);
}

TimeoutThread::~TimeoutThread()

{
    delete timedOutReason;
}

/*
 * PushTran:
 *    Put the TranEntry in question on the list of transactions that might
 * time out. This object scans this list to see which transactions should be
 * aborted.
 */

void TimeoutThread::PushTran(TranEntry* newEntry, const int seconds)

{
    Boolean wakeMeUp = false;
    latch.Lock();
    if (timeouts.size() == 0)
	wakeMeUp = true;
    time_t deadTime = time(nil);
    deadTime += (time_t)seconds;
    timeouts[*newEntry] = deadTime;
    latch.Unlock();
    if (wakeMeUp)
	emptyCondition.Signal();
}

/*
 * RemoveTran:
 *    Remove a TranEntry from the timeout list. This happens once
 * a transaction is aborted or committed.
 *    NOTE: This shouldn't be called by a transaction while its
 * TranEntry is locked, or a deadlock may result.
 */

void TimeoutThread::RemoveTran(TranEntry* deadEntry)

{
    if (deadEntry != nil)
    {
	latch.Lock();
	timeouts.remove(*deadEntry);
	latch.Unlock();
    }
}

static void TimeoutThreadCleanupHandler(void*)

{
  ObjtranReport.Report(ErrorLevels::TraceMsg, 
		       "The transaction timeout thread was terminated.\n");
}

/*
 * Run:
 *    This function is executed in a separate thread. It sleeps for a
 * specified number of seconds, then wakes up and checks for timed-out 
 * transactions. TranEntry objects on the timeout list that have had their
 * timeout period expire are moved to a separate list, then aborted.
 */

void* TimeoutThread::Run()

{
  String msg;
  time_t curTime;
  time_t outTime;
  int curEntry;
  Block<ExistingTransaction*> deadEntries;
  int numDead;
  ExistingTransaction* curTran;
  tran_tid_t tid;
  Boolean forever = 1;
  
  pthread_cleanup_push(TimeoutThreadCleanupHandler, (void*)0);
  SetKillable(true);
  while (forever)
    {
      SleepMe(TranTimeoutGranularity, 0);
      latch.Lock();
      while (timeouts.size() == 0)
	emptyCondition.Wait(latch);
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Started timeout checking.\n");
      numDead = 0;
      Mapiter<tran_tid_t, time_t> iterator = timeouts.first();
      curTime = time(nil);
      while (iterator)
	{
	  outTime = iterator.curr()->value;
	  if (outTime <= curTime)
	    {		
	      curTran = new ExistingTransaction((iterator.curr()->key));
	      deadEntries.reserve(numDead);
	      deadEntries[numDead++] = curTran;
	    }
	  iterator.next();
	}
      latch.Unlock();
      if (numDead > 0)
	ObjtranReport.Report(ErrorLevels::TraceMsg, "Aborting timed-out transactions.\n");
      for (curEntry = 0; curEntry < numDead; curEntry++)
	{
	  curTran = deadEntries[curEntry];
	  if (curTran != nil)
	    {
	      if (!curTran->AbortedOrCommitted())
		{
		  if (DefaultReport.Active(ErrorLevels::TraceMsg))
		    {
		      tid = *curTran;
		      ObjtranReport.VReport(ErrorLevels::TraceMsg, "Transaction %lu timed out\n",
					       (unsigned long)tid);
		    }
		  if (!(ObjtranMainMode | OBJTRAN_RECOVERABLE))
		    curTran->RequireDistributedOutcome(false);
		  curTran->Abort(*timedOutReason);
		}
	      delete curTran;
	    }
	}
      ObjtranReport.Report(ErrorLevels::TraceMsg, "Timeout processing completed.\n");
    }
  pthread_cleanup_pop(true);
  return nil;
}
