/*
 * acctblock.C
 *
 *    This is not a database, but it plays one on TV.
 *
 * acctblock.C,v 1.2 1993/08/06 22:05:20 gjb Exp
 */

#include <acctblock.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>

#ifdef _AIX
#include <Block.c>
#include <List.c>
#pragma declare (Block<SimpleAccount>)
#pragma declare (List<unsigned long>)
#endif

AccountBlock::AccountBlock(AccountID low, AccountID high, double initBalance)

{
  AccountID inc;
  unsigned int curID;
  SimpleAccount nullAcct(0, "none", initBalance);
  
  base = low;
  numItems = 0;
  for (inc = low; inc <= high; inc++)
    {
      curID = (unsigned int)(inc - base);
      theData.reserve(curID);
      nullAcct.ID(curID);
      theData[curID] = nullAcct;
      numItems++;
    }
}

AccountBlock::AccountBlock(const AccountBlock& b)

{
  this->operator = (b);
}

AccountBlock::~AccountBlock()

{
}

void AccountBlock::ThrowError(AcctStatus err)

{
  String txt;
  ExistingTransaction tran;

  switch(err)
    {
    case ACCT_SUCCESS:
      txt = "Success";
      break;
    case ACCT_RECORD_NOT_FOUND:
      txt = "Record not found";
      break;
    case ACCT_NOT_AUTHORIZED:
      txt = "Authorization denied";
      break;
    default:
      txt = "Unknown error text";
      break;
    }
  if (err != ACCT_SUCCESS)
    tran.Abort(txt, (int)err, 4);
}

AcctStatus AccountBlock::Lock(const AccountID id, LockMode mode)

{
  unsigned long realLock = (unsigned long)(id - base);
  LockName lName((void*)&realLock, sizeof(unsigned long));
  ExistingTransaction tran;
  
  tran.AcquireLock(mode, lName);
  return ACCT_SUCCESS;
}

AcctStatus AccountBlock::Update(const AccountID key, const double newBalance)

{
  AcctStatus retVal = ACCT_SUCCESS;
  unsigned int realKey = (unsigned int)(key - base);

  blockLock.Lock();
  if ((numItems == 0) || (realKey < 0) || (realKey >= numItems))
    {
      retVal = ACCT_RECORD_NOT_FOUND;
      ThrowError(retVal);
    }
  else
    {
      theData[realKey].Balance(newBalance);
    }
  blockLock.Unlock();
  return retVal;
}

AcctStatus AccountBlock::Get(const AccountID key, SimpleAccount& a)

{
  AcctStatus retVal = ACCT_SUCCESS;
  unsigned int realKey = (unsigned int)(key - base);

  blockLock.Lock();
  if ((numItems == 0) || (realKey < 0) || (realKey >= numItems))
    {
      retVal = ACCT_RECORD_NOT_FOUND;
      ThrowError(retVal);
    }
  else
    {
      a = theData[realKey];
    }
  blockLock.Unlock();
  return retVal;
}

AcctStatus AccountBlock::Find(const String& name, List<AccountID>& out)

{
  unsigned int inc;
  Boolean foundOne = false;
  AcctStatus retVal = ACCT_SUCCESS;

  blockLock.Lock();
  for (inc = (unsigned int)base; inc < (numItems + base); inc++)
    {
      if (theData[inc].Name() == name)
	{
	  out.put(theData[inc].ID());
	  foundOne = true;
	}
    }
  if (!foundOne)
    {
      retVal = ACCT_RECORD_NOT_FOUND;
      ThrowError(retVal);
    }
  blockLock.Unlock();
  return retVal;
}

AcctStatus AccountBlock::Find(const double balance, Comparator compare,
			      List<AccountID>& out)

{
  unsigned int inc;
  Boolean foundOne = false;
  Boolean foundMe;
  AcctStatus retVal = ACCT_SUCCESS;

  blockLock.Lock();
  for (inc = (unsigned int)base; inc < (numItems + base); inc++)
    {
      foundMe = false;
      switch(compare)
	{
	case LessThan:
	  foundMe = (theData[inc].Balance() < balance);
	  break;
	case GreaterThan:
	  foundMe = (theData[inc].Balance() > balance);
	  break;
	case LessThanOrEqual:
	  foundMe = (theData[inc].Balance() <= balance);
	  break;
	case GreaterThanOrEqual:
	  foundMe = (theData[inc].Balance() >= balance);
	  break;
	}
      if (foundMe)
	{
	  out.put(theData[inc].ID());
	  foundOne = true;
	}
    }
  if (!foundOne)
    {
      retVal = ACCT_RECORD_NOT_FOUND;
      ThrowError(retVal);
    }
  blockLock.Unlock();
  return retVal;
}


