/*
 * AccountClient
 *
 *    This is a little client for my test "account server". We can use
 * this to interactively start and stop transactions, acquire locks, and
 * make TRPCs. This program might be a pain to use, but it's good for 
 * testing.
 * 
 * acctclnt.C,v 1.4 1994/03/17 18:46:40 gjb Exp
 */

#include <objdce/statics.H>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <objtran/tranclient.H>
#include <objtran/tranbinding.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objdce/mutex.H>
#include <objtran/tranthread.H>
#include <String.h>
#include <objdce_common.H>
#include <acctstatus.H>
#include <testconfig.H>
extern "C"
{
#include <acctsvr.h>
}

#ifdef _AIX
#include <List.c>
#pragma declare (List_of_p<Transaction>)
#pragma declare (List_of_piter<Transaction>)
#pragma declare (List_of_p<TranBinding>)
#pragma declare (List_of_piter<TranBinding>)
#endif

class InteractiveAcctClient

{
public:
  InteractiveAcctClient(LoginContext* context, const String& configFile);
  ~InteractiveAcctClient();
  void Begin();
  void Abort();
  void Commit();
  void Change();
  void Get(int numGets);
  void Find();
  Boolean Happy() const;

  Boolean CheckTran() const;
  AccountID GetAccountID() const;
  void HandleAborted();
  AcctStatus Lock(AccountID, RPCLockMode);
  TranBinding* FindServer(const AccountID);
private:
  TestConfig config;
  Boolean happy;
  List_of_p<TranBinding> allBindings;
  List_of_p<Transaction> tranStack;
  Transaction* curTran;
  int nestDepth;
  MutexLock bindingLock;
};

inline Boolean InteractiveAcctClient::Happy() const

{ 
  return happy;
}

InteractiveAcctClient::InteractiveAcctClient(LoginContext* context,
					     const String& configFile)
: config(configFile)
{
  ServerInfo curInfo;
  unsigned int inc;
  TranBinding* tmpBinding;

  happy = true;
  for (inc = 0; inc < config.NumServers(); inc++)
    {
      config.GetServer(inc, curInfo);
      tmpBinding = new TranBinding(curInfo.name, RPCClient(TestAccountSvr,1,0),
				   context, curInfo.principal);
      if (!tmpBinding->IsValid())
	{
	  fprintf(stderr, "Couldn't find the server \"%s\"\n",
		  (const char*)(curInfo.name));
	  happy = false;
	}
      else
	allBindings.put(tmpBinding);
    }
  nestDepth = 0;
  curTran = (Transaction*)0;
}

InteractiveAcctClient::~InteractiveAcctClient()

{
  TranBinding* tmpBinding;
  if (nestDepth > 0)
    fprintf(stderr, "Warning: exiting with %i unfinished transactions\n", 
	    nestDepth);
  List_of_piter<TranBinding> iter(allBindings);
  while (iter.remove_next(tmpBinding))
    delete tmpBinding;
}

TranBinding* InteractiveAcctClient::FindServer(const AccountID id)

{
  unsigned int whatServer = config.FindServer(id);

  bindingLock.Lock();
  if (whatServer >= config.NumServers())
    {
      curTran->Abort("Record out of range", 3);
      bindingLock.Unlock();
      return (TranBinding*)0;
    }
  else
    {
      const TranBinding* retVal = allBindings[whatServer];
      bindingLock.Unlock();
      return (TranBinding*)retVal;
    }
}

Boolean InteractiveAcctClient::CheckTran() const

{
  Boolean retVal = true;
  if (curTran == (Transaction*)0)
    {
      retVal = false;
      printf("There is no currently active transaction\n");
    }
  else if (curTran->Aborted())
    {
      retVal = false;
      printf("The current transaction has been aborted\n");
    }
  return retVal;
}

AccountID InteractiveAcctClient::GetAccountID() const

{
  AccountID retVal;
  char line[1024];

  printf("Enter a numeric account ID: ");
  gets(line);
  retVal = (AccountID)atol(line);
  return retVal;
}

void InteractiveAcctClient::HandleAborted()

{
  Transaction* deadTran;
  tranStack.get(deadTran);
  if (nestDepth == 1)
    curTran = (Transaction*)0;
  else
    curTran = tranStack[0];
  nestDepth--;
  delete deadTran;
}

void InteractiveAcctClient::Begin()

{
  NewTransaction* newTran = new NewTransaction();
  curTran = newTran;
  tranStack.unget(newTran);
  nestDepth++;
  if (nestDepth == 1)
    printf("Began a top-level transaction\n");
  else
    printf("Begin a nested transaction at depth %i\n", nestDepth);
}

void InteractiveAcctClient::Abort()

{
  if (nestDepth == 0)
    printf("There is no transaction to abort\n");
  else
    {
      Transaction* deadTran;
      tranStack.get(deadTran);
      deadTran->Abort("User-initiated abort", 1);
      if (nestDepth == 1)
	{
	  printf("Aborted the top-level transaction\n");
	  curTran = (Transaction*)0;
	}
      else
	{
	  printf("Aborted the nested transaction at level %i\n", nestDepth);
	  curTran = tranStack[0];
	}
      nestDepth--;
      delete deadTran;
   }
}

void InteractiveAcctClient::Commit()

{
  if (nestDepth == 0)
    printf("There is no transaction to commit\n");
  else
    {
      Transaction* deadTran;
      tranStack.get(deadTran);
      deadTran->Commit();
      if (deadTran->Committed())
	{
	  if (nestDepth == 1)
	    {
	      printf("Committed the top-level transaction\n");
	      curTran = (Transaction*)0;
	    }
	  else
	    {
	      printf("Commited the nested transaction at level %i\n", 
		     nestDepth);
	      curTran = tranStack[0];
	    }
	}
      else
	{
	  if (nestDepth == 1)
	    {
	      printf("The top-level transaction has already been aborted\n");
	      curTran = (Transaction*)0;
	    }
	  else
	    {
	      printf("The nested transaction at level %i has been aborted\n",
		     nestDepth);
	      curTran = tranStack[0];
	    }
	}
      nestDepth--;
      delete deadTran;
    }
}

AcctStatus InteractiveAcctClient::Lock(AccountID id, RPCLockMode mode)

{
  char* modeName;
  AcctStatus retVal = ACCT_SUCCESS;

  if (CheckTran())
    {
      switch(mode)
	{
	case NoRPCModes:
	  modeName = "none";
	  break;
	case ReadRPCMode:
	  modeName = "read";
	  break;
	case WriteRPCMode:
	  modeName = "write";
	  break;
	case UpgradeRPCMode:
	  modeName = "upgrade";
	  break;
	case IntentionReadRPCMode:
	  modeName = "intention read";
	  break;
	case IntentionWriteRPCMode:
	  modeName = "intention write";
	  break;
	case AllRPCModes:
	  modeName = "all";
	  break;
	}
      TranBinding* svrBinding = FindServer(id);
      if (svrBinding != (TranBinding*)0)
	LockAccount(*svrBinding, id, mode);
      if (curTran->Aborted())
	{
	  printf("Couldn't lock the account: %s\n", 
		 (const char*)(curTran->GetAbortReason()->GetReason()));
	  retVal = curTran->GetAbortReason()->GetCode();
	  HandleAborted();
	}
    }
  else
    retVal = ACCT_NO_TRANSACTION;
  return retVal;
}
      
void InteractiveAcctClient::Change()

{
  char line[1024];
  double newBalance;
  AccountID id;
  
  if (CheckTran())
    {
      id = GetAccountID();
      printf("Enter the new balance: ");
      gets(line);
      sscanf(line, "%lf", &newBalance);
      Lock(id, WriteRPCMode);
    }
  if (CheckTran())
    {
      TranBinding* svrBinding = FindServer(id);
      if (svrBinding != (TranBinding*)0)
	UpdateBalance(*svrBinding, id, newBalance);
      if (curTran->Aborted())
	{
	  printf("Couldn't change the balance: %s\n",
		 (const char*)(curTran->GetAbortReason()->GetReason()));
	  HandleAborted();
	}
      else
	printf("Changed the balance\n");
    }
}

class GetValues : public ThreadObject

{
public:
  GetValues(AccountID id, TranBinding* b, InteractiveAcctClient* c);
  virtual void* Run();
private:
  AccountID whatAccount;
  TranBinding* whatServer;
  InteractiveAcctClient* clnt;
};

GetValues::GetValues(AccountID id, TranBinding* b,
		     InteractiveAcctClient* c) : ThreadObject()

{
  whatAccount = id;
  whatServer = b;
  clnt = c;
}

void* GetValues::Run()

{
  SimpleAccountData data;
  ExistingTransaction curTran;

  clnt->Lock(whatAccount, ReadRPCMode);
  if (whatServer != (TranBinding*)0) 
    {
      GetAccount(*whatServer, whatAccount, &data);
    }
  if (curTran.Aborted())
    {
      printf("Couldn't read the account: %s\n",
	     (const char*)(curTran.GetAbortReason()->GetReason()));
      clnt->HandleAborted();
    }
  else
    {
      printf("Account ID: %li\n", data.id);
      printf("      Name: %s\n", data.name);
      printf("   Balance: %10.2f\n", data.balance);
    }
  return (void*)0;
}

void InteractiveAcctClient::Get(int numGets)

{
  AccountID* ids = new AccountID[numGets];
  GetValues** valueObjs = new GetValues*[numGets];
  TranThread** valueThreads = new TranThread*[numGets];
  int inc;

  if (CheckTran())
    {
      for (inc = 0; inc < numGets; inc++)
	{
	  ids[inc] = GetAccountID();
	}
      for (inc = 0; inc < numGets; inc++)
	{
	  valueObjs[inc] = new GetValues(ids[inc], FindServer(ids[inc]), this);
	  valueThreads[inc] = new TranThread(*(valueObjs[inc]), *curTran);
	}
      for (inc = 0; inc < numGets; inc++)
	{
	  valueThreads[inc]->Wait();
	  delete valueThreads[inc];
	  delete valueObjs[inc];
	}
    }
  delete [] valueThreads;
  delete [] valueObjs;
  delete [] ids;
}

void InteractiveAcctClient::Find()

{
  printf("This doesn't work yet.\n");
}
	
static void PrintUsage(char* execName)

{
  printf("Usage: %s <config file>\n", execName);
}

int main (int argc, char** argv)

{	
  Boolean loginOK;
  CurrentLoginContext context;
  String configName;
  char line[1024];
  char command;
  int numGets;

  if (argc != 2)
    {
      PrintUsage(argv[0]);
      return -1;
    }
  configName = argv[1];
  loginOK = CurrentLogin("PACKET_AUTHENTICATION", context);
  if (!loginOK)
    {
      fprintf(stderr, "Error: Login failed\n");
      return -1;
    }

  TranClient client;
  InteractiveAcctClient acctClient(&context, configName);
  
  command = '\0';
  while ((acctClient.Happy()) && (command != 'q'))
    {
      printf("[<B>egin <A>bort <C>ommit <G>et <U>pdate <Q>uit] ");
      gets(line);
      command = line[0];
      tolower(command);
      if (strlen(line) != 1)
	printf("Invalid command\n");
      else
	{
	  switch(command)
	    {
	    case 'b':
	      acctClient.Begin();
	      break;
	    case 'a':
	      acctClient.Abort();
	      break;
	    case 'c':
	      acctClient.Commit();
	      break;
	    case 'g':
	      printf("Number of items to get: ");
	      gets(line);
	      numGets = atoi(line);
	      acctClient.Get(numGets);
	      break;
	    case 'u':
	      acctClient.Change();
	      break;
	    case 'q':
	      break;
	    default:
	      printf("Invalid command\n");
	      break;
	    }
	}
    }
  return 0;
}

