/*
 * bindingtest.C
 *
 *    This program tests all the functionality of the Binding class.
 * For it to work, fire up an instance of "simple_svr" with 
 * OBJTRAN_DEBUG_LEVEL set to "i". You'll need to know the exact endpoint
 * of the server.
 *
 * bindingtest.C,v 1.1 1993/11/27 01:40:15 gjb Exp
 */

#include <objdce/statics.H>
#include <objdce/binding.H>
#include <objdce_common.H>
#include <stdio.h>
#include <stdlib.h>
#include <rpc_one.h>
#include <rpc_client.h>

static Boolean CheckBindingSuccess(Binding& bind)

{
  Boolean retVal = bind.IsValid();
  String nsName = bind.GetNSName();

  if (retVal)
    {
      String strBinding = bind;
      if (nsName.length() > 0)
	{
	  printf("Successfully bound to %s at:\n%s\n", (const char*)nsName,
		 (const char*)strBinding);
	}
      else
	{
	  printf("Successful binding to %s\n", (const char*)strBinding);
	}
    }
  else
    {
      String errMsg;
      Binding::CheckErrorType(bind.GetDCEError(), errMsg);
      printf("Could not bind to %s:\n%s\n", (const char*)nsName,
	     (const char*)errMsg);
    }
  return retVal;
}

static Boolean CheckRPCSuccess(error_status_t err)

{
  String errMsg;

  Binding::ErrorType eType = Binding::CheckErrorType(err, errMsg);
  switch(eType)
    {
    case Binding::ET_SUCCESS:
      printf("SUCCESS\n");
      break;
    case Binding::ET_AUTHENTICATION_ERROR:
      printf("AUTHENTICATION_ERROR\n");
      break;
    case Binding::ET_COMMUNICATION_ERROR:
      printf("COMMUNICATION_ERROR\n");
      break;
    case Binding::ET_APPLICATION_ERROR:
      printf("APPLICATION_ERROR\n");
      break;
    case Binding::ET_REMOTE_FAULT:
      printf("REMOTE_FAULT\n");
      break;
    case Binding::ET_OTHER_DCE_ERROR:
      printf("OTHER_DCE_ERROR\n");
      break;
    }
  if (eType != Binding::ET_SUCCESS)
    {
      printf("DCE error message: %s\n", (const char*)errMsg);
      return false;
    }
  else
    {
      printf("RPC was successful\n");
      return false;
    }
}

void DoBasicBinding(const String& serverName)

{
  printf("Trying to bind to %s\n", (const char*)serverName);
  Binding bind(serverName, RPCClient(RpcTestOne,1,0));
  if (CheckBindingSuccess(bind))
    {
      error_status_t rpcErr = SampleString(bind, 
					   (unsigned char*)"DoBasicBinding");
      CheckRPCSuccess(rpcErr);
    }
}

void DoBasicContextBinding(const String& serverName,
			   LoginContext* context,
			   const String& serverPrincipal)
{
  error_status_t rpcErr;

  printf("Trying to bind to %s with principal %s\n",
	 (const char*)serverName, (const char*)serverPrincipal);
  Binding bind(serverName, RPCClient(RpcTestOne,1,0),
	       context, serverPrincipal);
  if (CheckBindingSuccess(bind))
    {
      rpcErr = SampleString(bind, (unsigned char*)"DoBasicContextBinding");
      CheckRPCSuccess(rpcErr);

      printf("Raising protection level to PACKET_PRIVACY\n");
      bind.SetAuthenticationLevel(LoginContext::PACKET_PRIVACY, 
				  serverPrincipal);
      rpcErr = SampleString(bind, (unsigned char*)"DoBasicContextBinding");
      CheckRPCSuccess(rpcErr);

      printf("Lowering protection level to NONE\n");
      bind.SetAuthenticationLevel(LoginContext::NONE);
      rpcErr = SampleString(bind, (unsigned char*)"DoBasicContextBinding");
      CheckRPCSuccess(rpcErr);
    }
}

void DoProtocolBindings(const String& serverName,
			LoginContext::ProtectionLevel level,
			const String& serverPrincipal)
{
  Binding useBinding;
  Binding* tmpBinding;
  error_status_t rpcErr;

  printf("Trying %s with protocol ALL\n", (const char*)serverName);
  tmpBinding = new Binding(serverName, RPCClient(RpcTestOne,1,0),
			   (DCEuuid*)0, level, serverPrincipal,
			   Binding::ALL);
  printf("Copying the binding\n");
  useBinding = *tmpBinding;
  delete tmpBinding;
  if (CheckBindingSuccess(useBinding))
    {
      rpcErr = SampleString(useBinding, (unsigned char*)"DoProtocolBindings");
      CheckRPCSuccess(rpcErr);
    }

  printf("Trying %s with protocol IP_UDP\n", (const char*)serverName);
  tmpBinding = new Binding(serverName, RPCClient(RpcTestOne,1,0),
			   (DCEuuid*)0, level, serverPrincipal,
			   Binding::IP_UDP);
  printf("Copying the binding\n");
  useBinding = *tmpBinding;
  delete tmpBinding;
  if (CheckBindingSuccess(useBinding))
    {
      rpcErr = SampleString(useBinding, (unsigned char*)"DoProtocolBindings");
      CheckRPCSuccess(rpcErr);
    }

  printf("Trying %s with protocol IP_TCP\n", (const char*)serverName);
  tmpBinding = new Binding(serverName, RPCClient(RpcTestOne,1,0),
			   (DCEuuid*)0, level, serverPrincipal,
			   Binding::IP_TCP);
  printf("Copying the binding\n");
  useBinding = *tmpBinding;
  delete tmpBinding;
  if (CheckBindingSuccess(useBinding))
    {
      rpcErr = SampleString(useBinding, (unsigned char*)"DoProtocolBindings");
      CheckRPCSuccess(rpcErr);
    }

  printf("Trying %s with protocol ALL_DG\n", (const char*)serverName);
  tmpBinding = new Binding(serverName, RPCClient(RpcTestOne,1,0),
			   (DCEuuid*)0, level, serverPrincipal,
			   Binding::ALL_DG);
  printf("Copying the binding\n");
  useBinding = *tmpBinding;
  delete tmpBinding;
  if (CheckBindingSuccess(useBinding))
    {
      rpcErr = SampleString(useBinding, (unsigned char*)"DoProtocolBindings");
      CheckRPCSuccess(rpcErr);
    }

  printf("Trying %s with protocol ALL_CN\n", (const char*)serverName);
  tmpBinding = new Binding(serverName, RPCClient(RpcTestOne,1,0),
			   (DCEuuid*)0, level, serverPrincipal,
			   Binding::ALL_CN);
  printf("Copying the binding\n");
  useBinding = *tmpBinding;
  delete tmpBinding;
  if (CheckBindingSuccess(useBinding))
    {
      rpcErr = SampleString(useBinding, (unsigned char*)"DoProtocolBindings");
      CheckRPCSuccess(rpcErr);
    }
}

void DoWellKnownBindings(LoginContext::ProtectionLevel level,
			 const String& serverHost,
			 const long serverPort)
{
  Binding* bind;
  error_status_t rpcErr;
  char strBinding[256];
  
  printf("Binding to port %li of %s\n", serverPort,
	 (const char*)serverHost);
  bind = new Binding(serverHost, serverPort, LoginContext::NONE);
  if (CheckBindingSuccess(*bind))
    {
      rpcErr = SampleString(*bind, (unsigned char*)"DoWellKnownBindings");
      CheckRPCSuccess(rpcErr);
      
      printf("Lowering timeout to MIN and trying again\n");
      bind->SetCommTimeout(Binding::CT_MIN);
      rpcErr = SampleString(*bind, (unsigned char*)"DoWellKnownBindings");
      CheckRPCSuccess(rpcErr);
      
      printf("Raising timeout to MAX and trying again\n");
      bind->SetCommTimeout(Binding::CT_MAX);
      rpcErr = SampleString(*bind, (unsigned char*)"DoWellKnownBindings");
      CheckRPCSuccess(rpcErr);
    }
  delete bind;
  
  printf("Binding again with security\n");
  bind = new Binding(serverHost, serverPort, level, 
		     RPCClient(RpcTestOne,1,0));
  if (CheckBindingSuccess(*bind))
    {
      rpcErr = SampleString(*bind, (unsigned char*)"DoWellKnownBindings");
      CheckRPCSuccess(rpcErr);
    }
  delete bind;
  
  sprintf(strBinding, "ncadg_ip_udp:%s[%li]", (const char*)serverHost,
	  serverPort);
  printf("Trying to bind to string binding %s...\n", strBinding);
  bind = new Binding(strBinding, level);
  if (CheckBindingSuccess(*bind))
    {
      rpcErr = SampleString(*bind, (unsigned char*)"DoWellKnownBindings");
      CheckRPCSuccess(rpcErr);
    }
  delete bind;

  printf("Making partial binding to host %s\n", (const char*)serverHost);
  bind = new Binding(serverHost, 0L, level, RPCClient(RpcTestOne,1,0));
  if (CheckBindingSuccess(*bind))
    {
      rpcErr = SampleString(*bind, (unsigned char*)"DoWellKnownBindings");
      CheckRPCSuccess(rpcErr);
      
      printf("Calling RPC again\n");
      rpcErr = SampleString(*bind, (unsigned char*)"DoWellKnownBindings");
      CheckRPCSuccess(rpcErr);
    }
  delete bind;
  
  printf("Making binding for broadcast RPC\n");
  bind = new Binding("", 0L, LoginContext::NONE);
  if (CheckBindingSuccess(*bind))
    {
      BroadcastMessage(*bind, (unsigned char*)"DoWellKnownBindings");

      printf("Calling RPC again\n");
      BroadcastMessage(*bind, (unsigned char*)"DoWellKnownBindings");
    }
  delete bind;
}

void ShutdownServer(const String& serverName, 
		    LoginContext::ProtectionLevel level)
{
  printf("Binding to %s\n", (const char*)serverName);
  Binding bind(serverName, RPCClient(RpcTestOne,1,0), level);
  printf("Trying to shut down the server\n");
  if (bind.StopServerListening())
    {
      CheckRPCSuccess(bind.GetDCEError());
    }
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <protection level> <server name> <client principal> <server prin> <server host> <server port #>\n", execName);
}

int main (int argc, char** argv)

{
  String protLevel;
  String serverName;
  String principal;
  String serverPrincipal;
  String tmpProt;
  String serverHost;
  String tmpPort;
  int serverPort;
  CurrentLoginContext context;
  LoginContext::ProtectionLevel level;

 if (argc != 7)
   {
     PrintUsage(argv[0]);
     return 1;
   }
  tmpProt = argv[1];
  serverName = argv[2];
  principal = argv[3];
  serverPrincipal = argv[4];
  serverHost = argv[5];
  tmpPort = argv[6];
  protLevel = tmpProt.upper();
  serverPort = atoi(tmpPort);

  if (protLevel == "NONE")
    {
      printf("Please use a login context greater than \"NONE\"\n");
      return 1;
    }
  if (!CurrentLogin(protLevel, context))
    {
      printf("Login failed.\n");
      return 1;
    }
  FindProtectionLevel(protLevel, level);
  DoBasicBinding(serverName);
  DoBasicContextBinding(serverName, &context, serverPrincipal);
  DoProtocolBindings(serverName, level, serverPrincipal);
  DoWellKnownBindings(level, serverHost, serverPort);
  ShutdownServer(serverName, level);
  printf("\nDone.\n");

  return 0;
}
