/*
 * c_client
 *
 *    This is a very simple program that tests the C API for Objdce
 * by binding to a server and making some RPC calls. It should be used
 * with a server that exports the "RpcTestOne" interface, like
 * simple_svr or multi_svr.
 *
 * c_client.c,v 1.1 1993/11/27 01:40:17 gjb Exp
 */
/*
 * Copyright (C) 1993 by Citibank, N.A.  (Subject to limited distribution
 * and restricted disclosure only).  All rights reserved.
 */

#include <pthread.h>
#include <objdce/c_api.h>
#include <rpc_one.h>
#include <stdio.h>
#include <stdlib.h>

static bindingP svrBinding;

/*
 * RpcCaller
 *    Make repeated RPC calls in an infinite loop. Call 
 * Bind_RebindIfNecessary before each call in case the last RPC resulted
 * in a communication error. Also, call GetBindingHandle before each call
 * and assign its result to a local variable. Otherwise, another thread
 * may call Bind_RebindIfNecessary and change the binding pointed to by
 * svrBinding and we won't get the new copy. This way, we always have
 * a correct binding in a fast, thread-safe manner.
 *    After the RPC, print out the RPC error if there was one. This will
 * not be necessary for all clients, but only those that wish to log the
 * exact reason for an RPC failure.
 */

static void* RpcCaller(void* arg)

{
  handle_t svrHandle;
  long result;
  error_status_t rpcErr;
  char errString[256];
  enum ErrorType rpcErrType;

  rpcErr = 0;
  for (;;)
    {
      if (Bind_RebindIfNecessary(svrBinding, rpcErr, 100, 10))
	{
	  printf("Rebound to server...\n");
	}
      svrHandle = GetBindingHandle(svrBinding);
      rpcErr = SampleAdd(svrHandle, 2, 2, &result);
      if (GetErrorType(rpcErr) != ERT_SUCCESS)
	{
	  GetErrorString(rpcErr, errString, 255);
	  printf("RPC error: %s\n", errString);
	}
      else
	{
	  printf(".");
	  fflush(stdout);
	}
    }
  return NULL;
}

/*
 * PrintUsage
 *    Print the appropriate command-line arguments for this program.
 */

static void PrintUsage(char* execName)

{
  printf("Usage: %s <server> <# threads>\n", execName);
}

/*
 * main
 *    First, check the command line arguments. Then, establish a login context
 * from the current context. If that works, get a binding to the server using
 * Bind_NameServer_PL. If that works, spawn threads to call the RPC. Since
 * there's no signal handler here, there's no way to shut down the program
 * cleanly. So, we don't worry about it.
 */

int main (int argc, char** argv)

{
  char* svrName;
  int numThreads;
  unsigned32 dceErr;
  pthread_t* rpcThreads;
  int inc;
  void* threadStatus;

  if (argc != 3)
    {
      PrintUsage(argv[0]);
      return 1;
    }

  svrName = argv[1];
  numThreads = atoi(argv[2]);

  if (Obj_Login_CurrentCtx(PRL_CALL) != LGI_SUCCESS)
    {
      printf("DCE login failed.\n");
      return 1;
    }

  svrBinding = Bind_NameServer_PL(svrName, &RpcTestOne_v1_0_c_ifspec,
				  PRL_CALL, "", &dceErr);
  if (dceErr != 0)
    {
      printf("Could not look up binding to server.\n");
      Obj_LogOut();
      return 1;
    }
  
  printf("Spawning %i threads to call RPCs...\n", numThreads);
  rpcThreads = (pthread_t*)malloc(sizeof(pthread_t) * numThreads);
  for (inc = 0; inc < numThreads; inc++)
    {
      pthread_create(&(rpcThreads[inc]), pthread_attr_default,
		     RpcCaller, NULL);
    }
  for (inc = 0; inc < numThreads; inc++)
    {
      pthread_join(rpcThreads[inc], &threadStatus);
    }

  return 0;
}
