/*
 * RWTest
 *
 *     This is a simple test that tries out the read-write lock class. It
 * spawns a bunch of threads that semi-randomly read or write lock an object.
 * It displays what they're doing on a curses-managed screen so one can see 
 * what's going on.
 *
 * rwtest.C,v 1.2 1993/08/06 22:05:27 gjb Exp
 */

#include <objdce/statics.H>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <objdce/mutex.H>
#include <objdce/readwrite.H>
#include <objdce/thread.H>
#include <objdce/transignals.H>
#ifdef _AIX
#include <cur01.h>
#include <cur02.h>
#include <cur03.h>
#include <cur04.h>
#include <cur05.h>
#else
#include <curses.h>
#endif

/*
 * Global stuff  
 */

MutexLock ScreenLock;		// Lock for curses routines
MutexLock RandLock;		// Lock for drand48
ReadWriteLock MainLock;		// The lock we're testing
volatile Boolean Done;		// When we're done

static double MaxWaitTime = 100000.0;
static int NumX = 16;
static int NumY = 8;

/*
 * RWTestThread: the thread that does the testing
 */

class RWTestThread : public ThreadObject

{
public:
  RWTestThread(int ID, double percent, int beginX, int beginY, 
	       int width, int height);
  virtual ~RWTestThread();
  virtual void* Run();
private:
  WINDOW* myWindow;
  char myID[32];
  int width;
  int height;
  double percent;
};

/*
 * RWTest
 *    The thread that spawns the threads that do the testing
 */

class RWTest : public Signaler, public Signaled

{
public:
  RWTest(double percent, ReadWriteLock::Priority);
  virtual ~RWTest();
  virtual Signaled::SigStatus HandleSignal(const int);
private:
  WINDOW* mainWindow;
  RWTestThread** threadObjs;
  AsyncThread** threads;
  int numThreads;
};
  
RWTestThread::RWTestThread(int ID, double per, int beginX, 
			   int beginY, int w, int h)
{
  sprintf(myID, "%i", ID);
  width = w;
  height = h;
  percent = per;
  ScreenLock.Lock();
  myWindow = newwin(height, width, beginY, beginX);
  wclear(myWindow);
  mvwaddstr(myWindow, 0, 0, myID);
  wrefresh(myWindow);
  ScreenLock.Unlock();
}

RWTestThread::~RWTestThread()
     
{
  ScreenLock.Lock();
  delwin(myWindow);
  ScreenLock.Unlock();
}

void* RWTestThread::Run()

{
  long sleepTime;
  double randVal;
  int centerX = (width / 2) - 1;
  int centerY = (width / 2) - 1;

  while (!Done)
    {
      RandLock.Lock();
      randVal = drand48();
      RandLock.Unlock();
      if (randVal < percent)
	{
	  MainLock.WriteLock();
	  ScreenLock.Lock();
	  mvwaddch(myWindow, centerY, centerX, 'W');
	  wrefresh(myWindow);
	  ScreenLock.Unlock();
	}
      else
	{
	  MainLock.ReadLock();
	  ScreenLock.Lock();
	  mvwaddch(myWindow, centerY, centerX, 'R');
	  wrefresh(myWindow);
	  ScreenLock.Unlock();
	}
      RandLock.Lock();
      randVal = drand48();
      RandLock.Unlock();
      sleepTime = (long)(randVal * MaxWaitTime);
      SleepMe(0, sleepTime);
      MainLock.Unlock();
      RandLock.Lock();
      randVal = drand48();
      RandLock.Unlock();
      ScreenLock.Lock();
      mvwaddch(myWindow, centerY, centerX, ' ');
      wrefresh(myWindow);
      ScreenLock.Unlock();
      sleepTime = (long)(randVal * MaxWaitTime);
      SleepMe(0, sleepTime);
    }
  return (void*)0;
}

RWTest::RWTest(double percent, ReadWriteLock::Priority priority)

{
  int inc = 0;
  int incX;
  int incY;
  int widthEach;
  int heightEach;
  int curX = 0;
  int curY = 0;

  Done = false;
  CatchSignal(SIGINT, *this);
  CatchSignal(SIGTERM, *this);
  MainLock.SetPriority(priority);
  numThreads = NumX * NumY;		// This is constant for now
  threadObjs = new RWTestThread*[numThreads];
  threads = new AsyncThread*[numThreads];
  ScreenLock.Lock();
  mainWindow = initscr();
  widthEach = COLS / NumX;
  heightEach = LINES / NumY;
  ScreenLock.Unlock();
  RandLock.Lock();
  srand48(time(NULL));
  RandLock.Unlock();
  for (incY = 0; incY < NumY; incY++)
    {
      curY = (incY * heightEach);
      for (incX = 0; incX < NumX; incX++)
	{
	  curX = (incX * widthEach);
	  threadObjs[inc] = new RWTestThread(inc, percent, curX, curY, 
					     widthEach, heightEach);
	  threads[inc] = new AsyncThread(*(threadObjs[inc]));
	  inc++;
	}
    }
  for (inc = 0; inc < numThreads; inc++)
    threads[inc]->Wait();
}

RWTest::~RWTest()

{
  int inc;

  for (inc = 0; inc < numThreads; inc++)
    {
      delete threads[inc];
      delete threadObjs[inc];
    }
  delete [] threads;
  delete [] threadObjs;
  endwin();
}

Signaled::SigStatus RWTest::HandleSignal(const int)

{
  Done = true;
  return Signaled::HANDLED;
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <percent writes> <priority>\n", execName);
  printf("       Priority is \"r\", \"w\", or \"d\"\n");
}

int main(int argc, char** argv)

{
  if (argc != 3)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  else
    {
      double percent;
      int tmpPercent;
      ReadWriteLock::Priority priority;

      tmpPercent = atoi(argv[1]);
      percent = (double)tmpPercent / 100.0;
      if (!strcmp(argv[2], "r"))
	priority = ReadWriteLock::READ_PRIORITY;
      else if (!strcmp(argv[2], "w"))
	priority = ReadWriteLock::WRITE_PRIORITY;
      else if (!strcmp(argv[2], "d"))
	priority = ReadWriteLock::DEFAULT_PRIORITY;
      else
	{
	  PrintUsage(argv[0]);
	  return 1;
	}
      RWTest tester(percent, priority);
    }
  return 0;
}

