/*
 * test_rig.C
 *
 *    This is the test rig. It runs programs both locally and remotely.
 * If all goes well, simple modifications to this program will suffice to
 * change the testing procedures. Yeah, right.
 *
 * test_rig.C,v 1.1 1993/11/27 01:40:24 gjb Exp
 */

#include <objdce/binding.H>
#include <objdce/login.H>
#include <test_rig_funcs.H>
#include <test_rig.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

static unsigned char argsArray[16][64];

/*
 * Constants: the name in () is the name of the environment variable that
 * overrides the constant. All times are in seconds.
 *    ServerStartupTime (TEST_SVR_STARTUP): Time to wait when starting a server
 *    ShortRunTime (TEST_SHORT_RUN): Time for a "short" run of a program
 *    LongRunTime (TEST_LONG_RUN): Time for a "long" run of a program
 *    ReallyLongRunTime (TEST_REALLY_LONG_RUN): Time for a really long run
 */

static const int DefaultServerStartupTime = 30;
static const int DefaultShortRunTime = 90;
static const int DefaultLongRunTime = 600;
static const int DefaultReallyLongRunTime = 7200; 
static int ServerStartupTime;
static int ShortRunTime;
static int LongRunTime;
static int ReallyLongRunTime; 

static void SetUpDefaults()

{
  char* envVal;

  envVal = getenv("TEST_SVR_STARTUP");
  if (envVal != (char*)0)
    {
      ServerStartupTime = atoi(envVal);
    }
  else
    {
      ServerStartupTime = DefaultServerStartupTime;
    }
  envVal = getenv("TEST_SHORT_RUN");
  if (envVal != (char*)0)
    {
      ShortRunTime = atoi(envVal);
    }
  else
    {
      ShortRunTime = DefaultShortRunTime;
    }
  envVal = getenv("TEST_LONG_RUN");
  if (envVal != (char*)0)
    {
      LongRunTime = atoi(envVal);
    }
  else
    {
      LongRunTime = DefaultLongRunTime;
    }
  envVal = getenv("TEST_REALLY_LONG_RUN");
  if (envVal != (char*)0)
    {
      ReallyLongRunTime = atoi(envVal);
    }
  else
    {
      ReallyLongRunTime = DefaultReallyLongRunTime;
    }
}

static void DelayMe(int seconds)

{	
  timespec sleepTime;
  sleepTime.tv_sec = seconds;
  sleepTime.tv_nsec = 0;
  printf("Sleeping for %i seconds...\n", seconds);
  if (pthread_delay_np(&sleepTime) < 0)
    {
      perror("Couldn't sleep!");
    }
}

static void CopyArgs(long numArgs, ...)

{
  va_list args;
  int inc;

  va_start(args, numArgs);
  for (inc = 0; (inc < numArgs) && (inc < 16); inc++)
    {
      strcpy((char*)argsArray[inc], va_arg(args, char*));
    }
  va_end(args);
  if (inc >= 16)
    {
      printf("Too many args passed to CopyArgs\n");
    }
}

static Boolean CheckIt(error_status_t err)

{
  if (err == TEST_SUCCESS)
    {
      return true;
    }
  else
    {
      String rpcMsg;
      switch(err)
	{
	case TEST_COULDNT_START:
	  printf("Could't start process\n");
	  break;
	case TEST_COULDNT_SIGNAL:
	  printf("Couldn't signal process\n");
	  break;
	case TEST_UNKNOWN_SIGNAL:
	  printf("Unknown signal passed to SignalProcess\n");
	  break;
	case TEST_COULDNT_WAIT:
	  printf("Couldn't wait for process\n");
	  break;
	default:
	  Binding::CheckErrorType(err, rpcMsg);
	  printf("RPC error: %s\n", (const char*)rpcMsg);
	  break;
	}
      return false;
    }
}
	
/*
 * SimpleTest
 *    Test the simple_clnt and simple_svr programs. Start up both on the
 * local machine, and run them for a while. Then, kill the local server
 * and start it up on the remote machine. After a while, kill the local
 * client, then the remote server.
 */

void SimpleTest()

{
  ProgramContext simpleSvr;
  ProgramContext simpleClnt;
  error_status_t rpcErr = 0;

  if (rpcErr == 0)
    {
      // Set up the arguments and start simple_svr locally
      CopyArgs(4, "CALL", "/.:/testing/objdce/simple_svr", 
	       "objdce_svr", "test.keys");
      rpcErr = LocalStartProgram((unsigned char*)"simple_svr", 
				 4, argsArray, &simpleSvr);
      CheckIt(rpcErr); 
    }
  if (rpcErr == 0)
    {
      // Give it some time to get ready
      DelayMe(ServerStartupTime);
      // Start simple_clnt locally
      CopyArgs(3, "CALL", "/.:/testing/objdce/simple_svr", "8");
      rpcErr = LocalStartProgram((unsigned char*)"simple_clnt",
				 3, argsArray, &simpleClnt);
      CheckIt(rpcErr);
    }
  if (rpcErr == 0)
    {
      // Let it run
      DelayMe(ShortRunTime);
      // Kill the server
      rpcErr = LocalSignalProgram(simpleSvr, (unsigned char*)"SIGKILL");
      CheckIt(rpcErr);
    }
  if (rpcErr == 0)
    {
      rpcErr = LocalWaitForProgram(&simpleSvr);
      CheckIt(rpcErr);
    }
  if (rpcErr == 0)
    {
      // Start it up remotely
      CopyArgs(4, "CALL", "/.:/testing/objdce/simple_svr",
	       "objdce_svr", "test.keys");
      rpcErr = StartProgram((unsigned char*)"simple_svr",
			    4, argsArray, &simpleSvr);
      CheckIt(rpcErr);
    }
  if (rpcErr == 0)
    {
      // Let it run
      DelayMe(ShortRunTime);
      // Kill the client
      rpcErr = LocalSignalProgram(simpleClnt, (unsigned char*)"SIGTERM");
      CheckIt(rpcErr);
    }
  if (rpcErr == 0)
    {
      rpcErr = LocalWaitForProgram(&simpleClnt);
      CheckIt(rpcErr);
    }
  if (rpcErr == 0)
    {
      // Kill the server
      rpcErr = SignalProgram(simpleSvr, (unsigned char*)"SIGTERM");
      CheckIt(rpcErr);
    }
  if (rpcErr == 0)
    {
      rpcErr = WaitForProgram(&simpleSvr);
      CheckIt(rpcErr);
    }
}



static void PrintUsage(const char* execName)

{
  printf("Usage: %s <server name>\n", execName);
}

int main(int argc, char** argv)

{
  if (argc != 2)
    {
      PrintUsage(argv[0]);
      return 1;
    }

  String svrName = argv[1];
  
  CurrentLoginContext context;
  context.LogIn(LoginContext::CALL);
  Binding* testSvr = new Binding(svrName, RPCClient(TestRig,1,0), &context);
  if (!testSvr->IsValid())
    {
      printf("Could not bind to \"%s\"\n", (const char*)svrName);
    }
  else
    {
      SetUpDefaults();
      TestRigHandle = *testSvr;
      SimpleTest();
    }
  delete testSvr;
  return 0;
}
