/*
 * test_rig_funcs.C
 *
 *    This file contains the code for the RPC functions used by the test rig.
 * We want to be able to call these functions locally, or as part of an
 * RPC. So, we define the functions in test_rig.idl with different names.
 * The server then creates a manager epv structure, and assigns its
 * function pointers to these re-named functions.
 *
 * test_rig_funcs.C,v 1.1 1993/11/27 01:40:30 gjb Exp
 */

#include <test_rig.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <objdce/objtranparams.H>
#include <test_rig_funcs.H>

typedef struct 

{
  pid_t pid;
} RealProgramContext;

/*
 * LocalStartProgram
 *    Implement the StartProgram function -- start the given program,
 * and return a context handle if we could.
 */

error_status_t LocalStartProgram(unsigned char* programPath,
				 long numArgs,
				 unsigned char theArgs[][64],
				 ProgramContext* ctxt)
{
  pid_t forkPid;
  error_status_t retVal;
  int inc;
  RealProgramContext* realCtxt;

  forkPid = vfork();
  if (forkPid < 0)
    {				// Error
      perror("Error forking process");
      retVal = TEST_COULDNT_START;
    }
  else if (forkPid == 0)
    {			      // In child
      char** execArgs = (char**)malloc(sizeof(char*) * (numArgs + 2));
      for (inc = 0; inc < numArgs; inc++)
	{
	  execArgs[inc + 1] = (char*)theArgs[inc];
	}
      execArgs[inc + 1] = (char*)0;
      execArgs[0] = (char*)programPath;
      printf("Starting:\n");
      for (inc = 0; inc < numArgs; inc++)
	{
	  printf("%s ", execArgs[inc]);
	}
      printf("\n");
      if (execv((const char*)programPath, execArgs) < 0)
	{
	  perror("Error starting process");
	  exit(TEST_COULDNT_START);
	}
    }
  else
    {				// In parent, successful
      realCtxt = new RealProgramContext;
      realCtxt->pid = forkPid;
      *ctxt = (ProgramContext)realCtxt;
      printf("%s started at PID %i\n", programPath, (int)forkPid);
      return TEST_SUCCESS;
    }
}

/*
 * LocalSignalProgram
 *
 *    Send the specified signal to the specified proram. Real simple.
 * By sending signal 15 or 2, the program can be terminated. Since signal
 * numbers aren't the same on all hosts, we pass the signal as a string
 * and translate it to a number.
 */

error_status_t LocalSignalProgram(ProgramContext ctxt,
				  unsigned char* sigName)
{
  pid_t pid;
  RealProgramContext* realCtxt = (RealProgramContext*)ctxt;
  int sigNum;
  error_status_t retVal = TEST_SUCCESS;

  pid = realCtxt->pid;
  if (!strcmp((const char*)sigName, "SIGINT"))
    {
      sigNum = SIGINT;
    }
  else if (!strcmp((const char*)sigName, "SIGTERM"))
    {
      sigNum = SIGTERM;
    }
  else if (!strcmp((const char*)sigName, "SIGTSTP"))
    {
      sigNum = SIGTSTP;
    }
  else if (!strcmp((const char*)sigName, "SIGCONT"))
    {
      sigNum = SIGCONT;
    }
  else if (!strcmp((const char*)sigName, "SIGUSR1"))
    {
      sigNum = SIGUSR1;
    }
  else if (!strcmp((const char*)sigName, "SIGHUP"))
    {
      sigNum = SIGHUP;
    }
  else if (!strcmp((const char*)sigName, "SIGKILL"))
    {
      sigNum = SIGKILL;
    }
  else
    {
      printf("Illegal signal name: %s\n", sigName);
      retVal = TEST_UNKNOWN_SIGNAL;
    }

  if (retVal == TEST_SUCCESS)
    {
      if (kill(pid, sigNum) < 0)
	{
	  perror("Couldn't signal process");	
	  // Don't return an error -- it could be dead already
	}
      else
	{
	  printf("Sent signal %i to PID %i\n", sigNum, (int)pid);
	}
    }
  return retVal;
}

/*
 * LocalWaitForProgram
 *
 *    Wait for a program to finish, using wait, and get rid of its context.
 */

error_status_t LocalWaitForProgram(ProgramContext* ctxt)

{
  RealProgramContext* realCtxt = (RealProgramContext*)*ctxt;
  pid_t pid = realCtxt->pid;
  int waitStat;
  error_status_t retVal = TEST_SUCCESS;

  if (waitpid(pid, &waitStat, WUNTRACED) < 0)
    {
      perror("Couldn't wait for process");
      retVal = TEST_COULDNT_WAIT;
    }
  else
    {
      if (WIFEXITED(waitStat) && (WEXITSTATUS(waitStat) != 0))
	{
	  printf("PID %i exited with status %i\n", 
		 (int)pid, WEXITSTATUS(waitStat));
	}
      else if (WIFSIGNALED(waitStat))
	{	
	  printf("PID %i terminated by signal %i\n",
		 (int)pid, WTERMSIG(waitStat));
	}
      else
	{
	  printf("PID %i terminated\n", (int)pid);
	}
      delete realCtxt;
      *ctxt = (ProgramContext)0;
    }
  return retVal;
}

/*
 * LocalIsProgramRunning
 *
 *    Using waitpid, figure out if a process is still running. If the status
 * isn't available, it's still running. Otherwise, terminate it as if wait
 * was called.
 */

error_status_t LocalIsProgramRunning(ProgramContext* ctxt, short* up)

{
  RealProgramContext* realCtxt = (RealProgramContext*)*ctxt;
  pid_t pid = realCtxt->pid;
  int waitStat;
  error_status_t retVal = TEST_SUCCESS;
  pid_t waitResult;

  waitResult = waitpid(pid, &waitStat, WUNTRACED | WNOHANG);
  if (waitResult < 0)
    {
      perror("Couldn't wait for process");
      retVal = TEST_COULDNT_WAIT;
      *up = 0;
    }
  else if (waitResult == 0)
    {
      printf("PID %i is still running\n", (int)pid);
      *up = 1;
    }
  else
    {
      if (WIFEXITED(waitStat) && (WEXITSTATUS(waitStat) != 0))
	{
	  printf("PID %i exited with status %i\n", 
		 (int)pid, WEXITSTATUS(waitStat));
	}
      else if (WIFSIGNALED(waitStat))
	{	
	  printf("PID %i terminated by signal %i\n",
		 (int)pid, WTERMSIG(waitStat));
	}
      else
	{
	  printf("PID %i terminated\n", (int)pid);
	}
      delete realCtxt;
      *ctxt = (ProgramContext)0;
      *up = 0;
    }
  return retVal;
}

/*
 * Context rundown
 *
 *    If the client goes away, DCE will automatically invoke this procedure
 * to let us know it's gone. So, we kill the affected process, wait for
 * it to terminate, and clean up the context.
 */

void ProgramContext_rundown(ProgramContext ctxt)

{
  LocalSignalProgram(ctxt, (unsigned char*)"SIGTERM");
  LocalWaitForProgram(&ctxt);
}


